@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -578,8 +578,7 @@ aftermath_genmove(int *aftermath_move, int color,
 	    break;
 	}
 	if (k < 4) {
-	  if (trymove(move, color, "aftermath-B", move + delta[k],
-		      EMPTY, NO_MOVE)) {
+	  if (trymove(move, color, "aftermath-B", move + delta[k])) {
 	    int adjs[MAXCHAIN];
 	    int neighbors;
 	    int r;
@@ -649,7 +648,7 @@ aftermath_genmove(int *aftermath_move, int color,
      * a dead opponent string at (target).
      */
     
-    if (!trymove(pos, color, "aftermath-A", target, EMPTY, NO_MOVE))
+    if (!trymove(pos, color, "aftermath-A", target))
       continue;
     
     /* It is frequently necessary to sacrifice own stones in order
@@ -697,7 +696,7 @@ aftermath_genmove(int *aftermath_move, int color,
 	int lib;
 	findlib(pos, 1, &lib);
 	move = lib;
-	if (!trymove(move, color, "aftermath-B", target, EMPTY, NO_MOVE))
+	if (!trymove(move, color, "aftermath-B", target))
 	  break;
       }
       
@@ -790,6 +789,8 @@ reduced_genmove(int *move, int color)
 {
   float val;
   int save_verbose;
+  float upper_bound, lower_bound;
+  float our_score;
 
   /* no move is found yet. */
   *move = NO_MOVE;  
@@ -799,35 +800,31 @@ reduced_genmove(int *move, int color)
   reset_engine();
 
   /* Find out information about the worms and dragons. */
-  examine_position(color, EXAMINE_ALL);
+  examine_position(EXAMINE_ALL);
 
   /* Make a score estimate. This can be used in later stages of the 
    * move generation.  If we are ahead, we can play safely and if
    * we are behind, we have to play more daringly.
    */
-  if (level >= 8) {
-    estimate_score(&upper_bound, &lower_bound);
-    if (verbose || showscore) {
-      if (lower_bound == upper_bound)
-	gprintf("\nScore estimate: %s %f\n",
-		lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound));
-      else
-	gprintf("\nScore estimate: %s %f to %s %f\n",
-		lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound),
-		upper_bound > 0 ? "W " : "B ", gg_abs(upper_bound));
-      fflush(stderr);
-    }
-
-    /* The score will be used to determine when we are safely
-     * ahead. So we want the most conservative score.
-     */
-    if (color == WHITE)
-      score = lower_bound;
+  estimate_score(&upper_bound, &lower_bound);
+  if (verbose || showscore) {
+    if (lower_bound == upper_bound)
+      gprintf("\nScore estimate: %s %f\n",
+	      lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound));
     else
-      score = upper_bound;
+      gprintf("\nScore estimate: %s %f to %s %f\n",
+	      lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound),
+	      upper_bound > 0 ? "W " : "B ", gg_abs(upper_bound));
+    fflush(stderr);
   }
+
+  /* The score will be used to determine when we are safely
+   * ahead. So we want the most conservative score.
+   */
+  if (color == WHITE)
+    our_score = lower_bound;
   else
-    score = 0.0;
+    our_score = -upper_bound;
 
   gg_assert(stackp == 0);
   
@@ -847,7 +844,7 @@ reduced_genmove(int *move, int color)
   gg_assert(stackp == 0);
 
   /* Review the move reasons and estimate move values. */
-  if (review_move_reasons(move, &val, color, 0.0, score, NULL))
+  if (review_move_reasons(move, &val, color, 0.0, our_score, NULL))
     TRACE("Move generation likes %1m with value %f\n", *move, val);
   gg_assert(stackp == 0);
 
@@ -859,7 +856,7 @@ reduced_genmove(int *move, int color)
   else
     TRACE("reduced_genmove() recommends %1m with value %f\n", *move, val);
  
- return val;
+  return val;
 }
 
 /* Preliminary function for playing through the aftermath. */
@@ -1035,7 +1032,7 @@ aftermath_compute_score(int color, float komi, SGFTree *tree)
  * Possible results are ALIVE, DEAD, ALIVE_IN_SEKI, WHITE_TERRITORY,
  * BLACK_TERRITORY, and DAME.
  */
-int
+enum dragon_status
 aftermath_final_status(int color, int pos)
 {
   ASSERT_ON_BOARD1(pos);
