@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -121,7 +121,7 @@ find_double_threats(int color)
     num_a_threatened_groups = get_attack_threats(ii, MAX_THREATENED_STRINGS,
 						 a_threatened_groups);
     if (num_a_threatened_groups > 1) {
-      if (trymove(ii, color, "find_double_threats-A", ii, EMPTY, NO_MOVE)) {
+      if (trymove(ii, color, "find_double_threats-A", ii)) {
 	for (k = 0; k < num_a_threatened_groups - 1; ++k)
 	  for (l = k + 1; l < num_a_threatened_groups; ++l) {
 	    /* Note: If we used attack_either() here instead of trymove()
@@ -314,7 +314,7 @@ atari_atari(int color, int *attack_move, char defense_moves[BOARDMAX],
       if (!ON_BOARD(pos) || !defense_moves[pos])
 	continue;
 
-      if (!trymove(pos, other, "atari_atari", NO_MOVE, EMPTY, NO_MOVE)) {
+      if (!trymove(pos, other, "atari_atari", NO_MOVE)) {
 	defense_moves[pos] = 0;
 	if (save_verbose)
 	  gprintf("%1m deleted defense point, illegal\n", pos);
@@ -349,6 +349,9 @@ atari_atari(int color, int *attack_move, char defense_moves[BOARDMAX],
  * The arrays saved_dragons[] and saved_worms[] should be one for
  * stones belonging to dragons or worms respectively, which are
  * supposedly saved by (move).
+ *
+ * FIXME: We probably want to change the calling convention of this
+ *        function to return all defense moves.
  */
 int
 atari_atari_confirm_safety(int color, int move, int *defense, int minsize,
@@ -356,26 +359,42 @@ atari_atari_confirm_safety(int color, int move, int *defense, int minsize,
 			   const char saved_worms[BOARDMAX])
 {
   char safe_stones[BOARDMAX];
+  char defense_moves[BOARDMAX];
+  int pos;
+  int blunder_size;
 
   mark_safe_stones(color, move, saved_dragons, saved_worms, safe_stones);
+  blunder_size = atari_atari_blunder_size(color, move, defense_moves,
+					  safe_stones);
 
-  return (atari_atari_blunder_size(color, move, defense, safe_stones)
-	  >= minsize);
+  if (defense) {
+    /* Return one arbitrary defense move. */
+    *defense = NO_MOVE;
+    for (pos = BOARDMIN; pos < BOARDMAX; pos++)
+      if (ON_BOARD(pos) && defense_moves[pos]) {
+	*defense = pos;
+	break;
+      }
+  }
+  
+  return blunder_size >= minsize;
 }
- 
+
 
 /* This function checks whether any new combination attack appears after
  * move at (move) has been made, and returns its size (in points).
  * safe_stones marks which of our stones are supposedly safe after this move.
  */
 int
-atari_atari_blunder_size(int color, int move, int *defense,
+atari_atari_blunder_size(int color, int move, char defense_moves[BOARDMAX],
 			 const char safe_stones[BOARDMAX])
 {
   int apos;
   int defense_point = NO_MOVE, after_defense_point = NO_MOVE;
   int aa_val, after_aa_val;
   int other = OTHER_COLOR(color);
+  char defense_points[BOARDMAX];
+  int last_forbidden = NO_MOVE;
 
   /* If aa_depth is too small, we can't see any combination attacks,
    * so in this respect the move is safe enough.
@@ -384,18 +403,19 @@ atari_atari_blunder_size(int color, int move, int *defense,
     return 0;
 
   memset(forbidden, 0, sizeof(forbidden));
+  memset(defense_points, 0, sizeof(defense_points));
 
   /* FIXME: Maybe these should be moved after the tryko() below? */
   compute_aa_status(other, safe_stones);
   compute_aa_values(other);
 
   /* Accept illegal ko capture here. */
-  if (!tryko(move, color, NULL, EMPTY, NO_MOVE))
+  if (!tryko(move, color, NULL))
     /* Really shouldn't happen. */
     abortgo(__FILE__, __LINE__, "trymove", move);
   increase_depth_values();
 
-  aa_val = do_atari_atari(other, &apos, &defense_point, NULL,
+  aa_val = do_atari_atari(other, &apos, &defense_point, defense_points,
 			  NO_MOVE, 0, 0, NULL);
   after_aa_val = aa_val;
 
@@ -421,6 +441,7 @@ atari_atari_blunder_size(int color, int move, int *defense,
      */
     after_defense_point = defense_point;
     forbidden[apos] = 1;
+    last_forbidden = apos;
     aa_val = do_atari_atari(other, &apos, &defense_point, NULL,
 			    NO_MOVE, 0, aa_val, NULL);
   }
@@ -433,14 +454,58 @@ atari_atari_blunder_size(int color, int move, int *defense,
    */
   compute_aa_status(other, NULL);
   compute_aa_values(other);
+  forbidden[last_forbidden] = 0;
   aa_val = do_atari_atari(other, NULL, NULL, NULL, NO_MOVE, 0, 0, NULL);
-  if (after_aa_val - aa_val > 0) {
-    if (defense)
-      *defense = after_defense_point;
-    return after_aa_val - aa_val;
-  }
-  else
+  if (aa_val >= after_aa_val)
     return 0;
+
+  /* Try the potential defense moves to see which are effective. */
+  if (defense_moves) {
+    int pos;
+    /* defense_points[] contains potential defense moves. Now we
+     * examine which of them really work.
+     */
+    
+    /* FIXME: Maybe these should be moved after the tryko() below? */
+    compute_aa_status(other, safe_stones);
+    compute_aa_values(other);
+      
+    memcpy(defense_moves, defense_points, sizeof(defense_points));
+    for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+      if (!ON_BOARD(pos) || !defense_moves[pos] || pos == move)
+	continue;
+
+      if (!trymove(pos, color, "atari_atari", NO_MOVE)) {
+	defense_moves[pos] = 0;
+	continue;
+      }
+      increase_depth_values();
+
+      if (attack(pos, NULL)) {
+	defense_moves[pos] = 0;
+	decrease_depth_values();
+	popgo();
+	continue;
+      }
+      
+      /* Accept illegal ko capture here. */
+      if (!tryko(move, color, NULL))
+	/* Really shouldn't happen. */
+	abortgo(__FILE__, __LINE__, "trymove", move);
+      increase_depth_values();
+      
+      if (do_atari_atari(other, &apos, &defense_point, NULL, NO_MOVE,
+			 0, after_aa_val, NULL) >= after_aa_val)
+	defense_moves[pos] = 0;
+
+      decrease_depth_values();
+      popgo();
+      decrease_depth_values();
+      popgo();
+    }
+  }
+  
+  return after_aa_val - aa_val;
 }
 
 
@@ -507,12 +572,22 @@ compute_aa_status(int color, const char safe_stones[BOARDMAX])
     if (board[pos] == other
 	&& worm[pos].origin == pos
 	&& worm[pos].liberties == 2
-	&& aa_status[pos] == ALIVE
-	&& !owl_substantial(pos)) {
-      int pos2;
-      for (pos2 = BOARDMIN; pos2 < BOARDMAX; pos2++)
-	if (ON_BOARD(pos2) && is_worm_origin(pos2, pos))
-	  aa_status[pos2] = INSUBSTANTIAL;
+	&& aa_status[pos] == ALIVE) {
+      int libs[2];
+      findlib(pos, 2, libs);
+      /* Don't waste time running owl_substantial() if we can't safely
+       * atari anyway.
+       */
+      if (is_self_atari(libs[0], color)
+	  && is_self_atari(libs[1], color))
+	continue;
+      
+      if (!owl_substantial(pos)) {
+	int pos2;
+	for (pos2 = BOARDMIN; pos2 < BOARDMAX; pos2++)
+	  if (ON_BOARD(pos2) && is_worm_origin(pos2, pos))
+	    aa_status[pos2] = INSUBSTANTIAL;
+      }
     }
   }
     
@@ -672,7 +747,7 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
     int bpos;
     int r;
     
-    if (!trymove(apos, color, "do_atari_atari-A", str, EMPTY, NO_MOVE))
+    if (!trymove(apos, color, "do_atari_atari-A", str))
       continue;
     
     if (all_potential_defenses) {
@@ -713,7 +788,7 @@ do_atari_atari(int color, int *attack_point, int *defense_point,
       if (all_potential_defenses)
 	all_potential_defenses[bpos] = 1;
 
-      if (trymove(bpos, other, "do_atari_atari-B", str, EMPTY, NO_MOVE)) {
+      if (trymove(bpos, other, "do_atari_atari-B", str)) {
 	int new_aa_val;
 	char new_goal[BOARDMAX];
 	/* These moves may have been irrelevant for later
@@ -982,7 +1057,7 @@ atari_atari_attack_callback(int anchor, int color,
       /*
        * Play (move) and see if there is an attack.
        */
-      if (trymove(move, color, "attack_callback", str, EMPTY, NO_MOVE)) {
+      if (trymove(move, color, "attack_callback", str)) {
 	int acode;
 	int attack_point = NO_MOVE;
 
@@ -1061,8 +1136,7 @@ atari_atari_find_defense_moves(int targets[AA_MAX_TARGETS_PER_MOVE],
     liberties = findlib(str, 4, libs);
     for (k = 0; k < liberties; k++) {
       if (!mx[libs[k]]
-	  && trymove(libs[k], board[str], "aa_defend-A", str,
-		     EMPTY, NO_MOVE)) {
+	  && trymove(libs[k], board[str], "aa_defend-A", str)) {
 	if (attack(str, NULL) == 0) {
 	  moves[num_moves++] = libs[k];
 	  mx[libs[k]] = 1;
@@ -1092,8 +1166,7 @@ atari_atari_find_defense_moves(int targets[AA_MAX_TARGETS_PER_MOVE],
 	for (s = 0; s < liberties; s++) {
 	  if (!mx[libs[s]]
 	      && !is_self_atari(libs[s], board[str])
-	      && trymove(libs[s], board[str], "aa_defend-B", str,
-			 EMPTY, NO_MOVE)) {
+	      && trymove(libs[s], board[str], "aa_defend-B", str)) {
 	    if (attack(str, NULL) == 0) {
 	      moves[num_moves++] = libs[s];
 	      mx[libs[s]] = 1;
