@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -49,11 +49,12 @@ static int initial_influence2_examined = -1;
 static int dragons_refinedly_examined = -1;
 
 static int revise_semeai(int color);
-static int revise_thrashing_dragon(int color, float advantage);
+static int revise_thrashing_dragon(int color, float our_score,
+    			 	   float advantage);
 
+static void break_mirror_go(int color);
 static int find_mirror_move(int *move, int color);
-static int test_symmetry_after_move(int move, int color);
-static int should_resign(int color, float score);
+static int should_resign(int color, float our_score);
 
 void sgfShowConsideredMoves(void);
 
@@ -77,7 +78,8 @@ reset_engine()
 
   /* Initialize things for hashing of positions. */
   reading_cache_clear();
-  hashdata_recalc(&hashdata, board, board_ko_pos);
+
+  hashdata_recalc(&board_hash, board, board_ko_pos);
 
   worms_examined = -1;
   initial_influence_examined = -1;
@@ -105,13 +107,11 @@ reset_engine()
  */
 
 void
-examine_position(int color, int how_much)
+examine_position(int how_much)
 {
   int save_verbose = verbose;
 
-  purge_persistent_reading_cache();
-  purge_persistent_connection_cache();
-  purge_persistent_breakin_cache();
+  purge_persistent_caches();
   
   /* Don't print reading traces during make_worms and make_dragons unless 
    * the user really wants it (verbose == 3). 
@@ -142,14 +142,14 @@ examine_position(int color, int how_much)
 
     if (how_much == EXAMINE_DRAGONS_WITHOUT_OWL) {
       if (NEEDS_UPDATE(dragons_examined_without_owl))
-	make_dragons(color, 1);
+	make_dragons(1);
       verbose = save_verbose;
       gg_assert(test_gray_border() < 0);
       return;
     }
     
     if (NEEDS_UPDATE(dragons_examined)) {
-      make_dragons(color, 0);
+      make_dragons(0);
       /* We have automatically done a partial dragon analysis as well. */
       dragons_examined_without_owl = position_number;
     }
@@ -195,7 +195,7 @@ examine_position(int color, int how_much)
  * output, and sgf traces are turned off.
  */
 void
-silent_examine_position(int color, int how_much)
+silent_examine_position(int how_much)
 {
   int save_verbose = verbose;
   SGFTree *save_sgf_dumptree = sgf_dumptree;
@@ -209,7 +209,7 @@ silent_examine_position(int color, int how_much)
   debug = 0;
   printmoyo = 0;
   
-  examine_position(color, how_much);
+  examine_position(how_much);
 
   verbose = save_verbose;
   sgf_dumptree = save_sgf_dumptree;
@@ -291,8 +291,8 @@ void
 collect_move_reasons(int color)
 {
   worm_reasons(color);
-  owl_reasons(color);
   semeai_move_reasons(color);
+  owl_reasons(color);
   break_in_move_reasons(color);
 }
 
@@ -313,22 +313,17 @@ do_genmove(int *move, int color, float pure_threat_value,
 	   int allowed_moves[BOARDMAX])
 {
   float val;
+  float upper_bound, lower_bound;
+  float average_score, our_score;
   int save_verbose;
   int save_depth;
 
   start_timer(0);
+  clearstats();
   
   /* Prepare our table of moves considered. */
   memset(potential_moves, 0, sizeof(potential_moves));
   
-  /* Reset all the statistics for each move. */
-  stats.nodes = 0;
-  stats.position_entered    = 0;
-  stats.position_hits       = 0;
-  stats.read_result_entered = 0;
-  stats.read_result_hits    = 0;
-  stats.hash_collisions     = 0;
-  
   /* no move is found yet. */
   *move = NO_MOVE;  
   val = -1; 
@@ -357,37 +352,41 @@ do_genmove(int *move, int color, float pure_threat_value,
 
   /* Find out information about the worms and dragons. */
   start_timer(1);
-  examine_position(color, EXAMINE_ALL);
+  examine_position(EXAMINE_ALL);
   time_report(1, "examine position", NO_MOVE, 1.0);
 
   /* Make a score estimate. This can be used in later stages of the 
    * move generation.  If we are ahead, we can play safely and if
    * we are behind, we have to play more daringly.
    */
-  if (level >= 8) {
-    estimate_score(&upper_bound, &lower_bound);
-    if (verbose || showscore) {
-      if (lower_bound == upper_bound)
-	gprintf("\nScore estimate: %s %f\n",
-		lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound));
-      else
-	gprintf("\nScore estimate: %s %f to %s %f\n",
-		lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound),
-		upper_bound > 0 ? "W " : "B ", gg_abs(upper_bound));
-      fflush(stderr);
-    }
-    time_report(1, "estimate score", NO_MOVE, 1.0);
-
-    /* The score will be used to determine when we are safely
-     * ahead. So we want the most conservative score.
-     */
-    if (color == WHITE)
-      score = lower_bound;
+  estimate_score(&upper_bound, &lower_bound);
+  if (verbose || showscore) {
+    if (lower_bound == upper_bound)
+      gprintf("\nScore estimate: %s %f\n",
+	      lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound));
     else
-      score = upper_bound;
+      gprintf("\nScore estimate: %s %f to %s %f\n",
+	      lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound),
+	      upper_bound > 0 ? "W " : "B ", gg_abs(upper_bound));
+    fflush(stderr);
   }
+  time_report(1, "estimate score", NO_MOVE, 1.0);
+  if (color == WHITE)
+    average_score = (upper_bound + lower_bound)/2.0;
   else
-    score = 0.0;
+    average_score = -(upper_bound + lower_bound)/2.0;
+  choose_strategy(color, average_score, game_status(color));
+
+  /* The score will be used to determine when we are safely
+   * ahead. So we want the most conservative score.
+   *
+   * We always want to have the score from our point of view. So
+   * negate it if we are black.
+   */
+  if (color == WHITE)
+    our_score = lower_bound;
+  else
+    our_score = -upper_bound;
 
   /*
    * Print some of the information if the user wants to.
@@ -422,14 +421,17 @@ do_genmove(int *move, int color, float pure_threat_value,
     verbose--;
   collect_move_reasons(color);
   verbose = save_verbose;
+  time_report(1, "generate move reasons", NO_MOVE, 1.0);
   
   /* Try to find empty corner moves. */
   if (!limit_search)
     fuseki(color);
   gg_assert(stackp == 0);
 
+  /* Look for moves too break mirror play by the opponent. */
+  break_mirror_go(color);
+  
   /* The general pattern database. */
-  start_timer(1);
   shapes(color);
   time_report(1, "shapes", NO_MOVE, 1.0);
   gg_assert(stackp == 0);
@@ -441,7 +443,7 @@ do_genmove(int *move, int color, float pure_threat_value,
 
   /* Review the move reasons and estimate move values. */
   if (review_move_reasons(move, &val, color, 
-			  pure_threat_value, score, allowed_moves))
+			  pure_threat_value, our_score, allowed_moves))
     TRACE("Move generation likes %1m with value %f\n", *move, val);
   gg_assert(stackp == 0);
   time_report(1, "review move reasons", NO_MOVE, 1.0);
@@ -451,17 +453,17 @@ do_genmove(int *move, int color, float pure_threat_value,
    * UNKNOWN. This may generate a move.
    */
   if (val < 10.0 && !doing_scoring && !limit_search) {
-    if (revise_thrashing_dragon(color, 15.0)) {
+    if (revise_thrashing_dragon(color, our_score, 15.0)) {
       shapes(color);
       if (!disable_endgame_patterns)
 	endgame_shapes(color);
-      if (review_move_reasons(move, &val, color, pure_threat_value, score,
+      if (review_move_reasons(move, &val, color, pure_threat_value, our_score,
 			      allowed_moves)) {
 	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
 	      *move, val); 
       }
     }
-    time_report(1, "move reasons with revised semeai status", NO_MOVE, 1.0);
+    time_report(1, "move reasons with revised thrashing status", NO_MOVE, 1.0);
   }
 
   /* If the move value is 6 or lower, we look for endgame patterns too. */
@@ -469,7 +471,7 @@ do_genmove(int *move, int color, float pure_threat_value,
     endgame_shapes(color);
     endgame(color);
     gg_assert(stackp == 0);
-    if (review_move_reasons(move, &val, color, pure_threat_value, score,
+    if (review_move_reasons(move, &val, color, pure_threat_value, our_score,
 			    allowed_moves))
       TRACE("Move generation likes %1m with value %f\n", *move, val);
     gg_assert(stackp == 0);
@@ -481,17 +483,18 @@ do_genmove(int *move, int color, float pure_threat_value,
    * run shapes and endgame_shapes again. This may turn up a move.
    */
   if (val < 0.0) {
-    if (revise_thrashing_dragon(color, 0.0)
+    if (revise_thrashing_dragon(color, our_score, 0.0)
 	|| revise_semeai(color)) {
       shapes(color);
       endgame_shapes(color);
-      if (review_move_reasons(move, &val, color, pure_threat_value, score,
+      if (review_move_reasons(move, &val, color, pure_threat_value, our_score,
 			      allowed_moves)) {
 	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
 	      *move, val); 
       }
     }
-    time_report(1, "move reasons with revised semeai status", NO_MOVE, 1.0);
+    time_report(1, "move reasons with revised semeai or thrashing status",
+		NO_MOVE, 1.0);
   }
 
   /* If still no move, fill a remaining liberty. This should pick up
@@ -516,9 +519,7 @@ do_genmove(int *move, int color, float pure_threat_value,
       && !doing_scoring
       && (play_out_aftermath 
 	  || capture_all_dead 
-	  || (thrashing_dragon
-	      && ((color == BLACK && score < -15.0)
-		  || (color == WHITE && score > 15.0))))
+	  || (thrashing_dragon && our_score > 15.0))
       && aftermath_genmove(move, color, NULL, 0) > 0
       && (!allowed_moves || allowed_moves[*move])) {
     ASSERT1(is_legal(*move, color), *move);
@@ -553,23 +554,17 @@ do_genmove(int *move, int color, float pure_threat_value,
     TRACE("genmove() recommends %1m with value %f\n", *move, val);
     /* Maybe time to resign...
      */
-    if (resign_allowed && val < 10.0 && should_resign(color, score)) {
-      TRACE("... though, genmove() thinks the position is hopeless\n" );
+    if (resign_allowed && val < 10.0 && should_resign(color, our_score)) {
+      TRACE("... though, genmove() thinks the position is hopeless\n");
       /* Signal resignation by negating the move value */
       val = -val;
     }
   }
   
   /* If statistics is turned on, this is the place to show it. */
-  if (showstatistics) {
-    gprintf("Nodes:                %d\n", stats.nodes);
-    gprintf("Positions entered:    %d\n", stats.position_entered);
-    gprintf("Position hits:        %d\n", stats.position_hits);
-    gprintf("Read results entered: %d\n", stats.read_result_entered);
-    gprintf("Read result hits:     %d\n", stats.read_result_hits);
-    gprintf("Hash collisions:      %d\n", stats.hash_collisions);
-  }
- 
+  if (showstatistics)
+    showstats();
+
   if (showtime) {
     double spent = time_report(0, "TIME to generate move at ", *move, 1.0);
     total_time += spent;
@@ -579,8 +574,6 @@ do_genmove(int *move, int color, float pure_threat_value,
       slowest_movenum = movenum + 1;
     }
   }
-  if (0 && (debug & DEBUG_BREAKIN))
-    print_persistent_breakin_cache();
 
   /* Some consistency checks to verify that things are properly
    * restored and/or have not been corrupted.
@@ -659,7 +652,7 @@ revise_semeai(int color)
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     if (ON_BOARD(pos)
 	&& dragon[pos].color == other
-	&& DRAGON2(pos).semeai
+	&& DRAGON2(pos).semeais
 	&& dragon[pos].status == DEAD) {
       found_one = 1;
       dragon[pos].status = UNKNOWN;
@@ -680,15 +673,14 @@ revise_semeai(int color)
  */
 
 static int
-revise_thrashing_dragon(int color, float advantage)
+revise_thrashing_dragon(int color, float our_score, float advantage)
 {
   int pos;
   char safe_stones[BOARDMAX];
   float strength[BOARDMAX];
 
   /* Trust the owl code's opinion if we are behind. */
-  if ((color == BLACK && score > -advantage)
-      || (color == WHITE && score < advantage))
+  if (our_score < advantage)
     return 0;
 
   if (disable_threat_computation
@@ -720,7 +712,6 @@ revise_thrashing_dragon(int color, float advantage)
  * To be able to deal with handicap stones we use a somewhat weak
  * definition of symmetry.
  */
-#define MIRROR_MOVE(pos) POS(board_size - 1 - I(pos), board_size - 1 - J(pos))
 
 static int
 find_mirror_move(int *move, int color)
@@ -729,7 +720,7 @@ find_mirror_move(int *move, int color)
   int mirror_move;
   if (last_move != NO_MOVE) {
     mirror_move = MIRROR_MOVE(last_move);
-    if (test_symmetry_after_move(mirror_move, color)) {
+    if (test_symmetry_after_move(mirror_move, color, 0)) {
       *move = mirror_move;
       return 1;
     }
@@ -737,7 +728,7 @@ find_mirror_move(int *move, int color)
   else {
     for (mirror_move = BOARDMIN; mirror_move < BOARDMAX; mirror_move++) {
       if (ON_BOARD(mirror_move)
-	  && test_symmetry_after_move(mirror_move, color)) {
+	  && test_symmetry_after_move(mirror_move, color, 0)) {
 	*move = mirror_move;
 	return 1;
       }
@@ -747,33 +738,31 @@ find_mirror_move(int *move, int color)
   return 0;
 }
 
-static int
-test_symmetry_after_move(int move, int color)
+
+/* Detect if a white opponent has played mirror go for at least 10
+ * moves and if so play on tengen.
+ *
+ * Mirror breaking moves in other situations are handled by patterns
+ * in patterns.db.
+ */
+static void
+break_mirror_go(int color)
 {
-  int pos;
-  int result = 1;
-
-  if (board[move] != EMPTY)
-    return 0;
-  if (!trymove(move, color, "find_mirror_move", NO_MOVE, EMPTY, NO_MOVE))
-    return 0;
-  
-  for (pos = BOARDMIN; pos <= MIRROR_MOVE(pos); pos++) {
-    if ((board[pos] == EMPTY) ^ (board[MIRROR_MOVE(pos)] == EMPTY)) {
-      result = 0;
-      break;
-    }
+  int tengen = POS((board_size - 1) / 2, (board_size - 1) / 2);
+  if (board[tengen] == EMPTY
+      && color == BLACK
+      && stones_on_board(BLACK | WHITE) > 10
+      && test_symmetry_after_move(tengen, color, 1)) {
+    set_minimum_move_value(tengen, 30.0);
+    TRACE("Play %1m to break mirror go, value 30.\n", tengen);
   }
-  
-  popgo();
-
-  return result;
 }
 
+
 /* Helper to decide whether GG should resign a game
  */
 static int
-should_resign(int color, float score)
+should_resign(int color, float our_score)
 {
   float status;
   int d;
@@ -782,9 +771,7 @@ should_resign(int color, float score)
    * behind (of course).
    */
   if (board_size < 19
-      || gg_abs(score) < 45
-      || (color == WHITE && score >= 0.0)
-      || (color == BLACK && score <= 0.0))
+      || our_score > -45.0) 
     return 0;
   /* Check dragon statuses. If a friendly dragon is critical, we are
    * possibly not that much behind after we save it. If some hostile
@@ -847,7 +834,8 @@ set_search_diamond(int pos)
 	search_mask[POS(m, n)] = 0;      
     }
   limit_search = pos;
-  if (0) draw_search_area();
+  if (0)
+    draw_search_area();
 }
 
 /* unmarks the entire board */
@@ -878,7 +866,7 @@ draw_search_area(void)
     for (n = 0; n < board_size; n++) {
       int col, c;
 	
-      if (search_mask[POS(m,n)])
+      if (search_mask[POS(m, n)])
 	col = GG_COLOR_RED;
       else
 	col = GG_COLOR_BLACK;
