@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -32,40 +32,14 @@
  * Define all global variables used within the engine.
  */
 
-
-/* The go board and position. */
-int          board_size = DEFAULT_BOARD_SIZE; /* board size */
-Intersection board[BOARDSIZE];
-int          board_ko_pos;
-int          white_captured;    /* number of black and white stones captured */
-int          black_captured;
-
-Intersection initial_board[BOARDSIZE];
-int          initial_board_ko_pos;
-int          initial_white_captured;
-int          initial_black_captured;
-int          move_history_color[MAX_MOVE_HISTORY];
-int          move_history_pos[MAX_MOVE_HISTORY];
-int          move_history_pointer;
-
 int thrashing_dragon = NO_MOVE; /* Dead opponent's dragon trying to live. */
 char thrashing_stone[BOARDMAX]; /* All thrashing stones. */
 
-float        komi;
-int          movenum;
-
-Intersection shadow[BOARDMAX];
-
-/* Hashing of positions. */
-Hash_data    hashdata;
-
 int hashflags = HASH_DEFAULT;
 
 float potential_moves[MAX_BOARD][MAX_BOARD];
 
 /* Used by reading. */
-int stackp;             /* stack pointer */
-int position_number;    /* position number */
 int depth;              /* deep reading cut off */
 int backfill_depth;     /* deep reading cut off */
 int backfill2_depth;    /* deep reading cut off */
@@ -75,6 +49,7 @@ int fourlib_depth;      /* deep reading cut off */
 int ko_depth;           /* deep reading cut off */
 int branch_depth;       /* deep reading cut off */
 int aa_depth;
+int depth_offset;       /* keeps track of temporary depth changes */
 int owl_distrust_depth;   /* below this owl trusts the optics code */
 int owl_branch_depth;     /* below this owl tries only one variation */
 int owl_reading_depth;    /* owl does not read below this depth */
@@ -110,18 +85,12 @@ int allpats           = 0;  /* generate all patterns, even small ones */
 int printworms        = 0;  /* print full data on each string */
 int printmoyo         = 0;  /* print moyo board each move */
 int printboard        = 0;  /* print board each move */
-int count_variations  = 0;  /* used by decide_string */
-int sgf_dump          = 0;  /* used by decide_string */
-SGFTree *sgf_dumptree = NULL;
 int loading           = 0;  /* TRUE if last loaded move comes from file */
 int fusekidb          = 1;  /* use fuseki database */
 int disable_fuseki    = 0;  /* do not generate fuseki moves */
 int josekidb          = 1;  /* use fuseki database */
 int showtime          = 0;  /* print time to find move */
 int showscore         = 0;  /* print estimated score */
-float score           = 0.0;
-float lower_bound     = 0.0;
-float upper_bound     = 0.0;
 int level             = DEFAULT_LEVEL; /* strength; up to 10 supported */
 int min_level         = 0;
 int max_level         = gg_max(DEFAULT_LEVEL, 10);
@@ -138,8 +107,6 @@ int disable_endgame_patterns   = 0;
 int doing_scoring              = 0;
 
 int chinese_rules       = 0;    /* ruleset choice for GMP connection */
-/* use experimental semeai module */
-int experimental_semeai = EXPERIMENTAL_SEMEAI;
 /* use experimental connection module */
 int experimental_connections = EXPERIMENTAL_CONNECTIONS;
 /* use alternate connection reading algorithm */
@@ -150,11 +117,13 @@ int owl_threats = OWL_THREATS;
 int experimental_owl_ext = EXPERIMENTAL_OWL_EXT;
 /* use experimental territory break-in module */
 int experimental_break_in = USE_BREAK_IN;
+/* use central oriented influence */
+int cosmic_gnugo = COSMIC_GNUGO;
 
-int allow_suicide       = 0;    /* allow opponent to make suicide moves */
 int capture_all_dead    = 0;    /* capture all dead opponent stones */
 int play_out_aftermath  = 0;    /* make everything unconditionally settled */
 int resign_allowed      = 0;    /* resign hopeless games */
+int large_scale         = 0;    /* search for large scale owl moves */
 
 int play_mirror_go      = 0;    /* try to play mirror go if possible */
 int mirror_stones_limit = -1;   /* but stop at this number of stones */
@@ -174,23 +143,31 @@ int number_close_white_worms[BOARDMAX];
 int false_eye_territory[BOARDMAX];
 int forced_backfilling_moves[BOARDMAX];
 
-/* Various statistics are collected here. */
-struct stats_data stats;
-
 struct worm_data      worm[BOARDMAX];
 struct dragon_data    dragon[BOARDMAX];
 int                   number_of_dragons;
 struct dragon_data2   *dragon2 = NULL;
 struct half_eye_data  half_eye[BOARDMAX];
-struct half_eye_data  owl_half_eye[BOARDMAX];
 struct eye_data       black_eye[BOARDMAX];
 struct eye_data       white_eye[BOARDMAX];
-struct eye_data       owl_black_eye[BOARDMAX];
-struct eye_data       owl_white_eye[BOARDMAX];
+struct vital_eye_points black_vital_points[BOARDMAX];
+struct vital_eye_points white_vital_points[BOARDMAX];
 struct surround_data  surroundings[MAX_SURROUND];
 int                   surround_pointer;
 
+int cutting_points[BOARDMAX];
+
 double slowest_time = 0.0;
 int    slowest_move = NO_MOVE;
 int    slowest_movenum = 0;
 double total_time = 0.0;
+
+
+int use_optimistic_territory = 0;
+float minimum_value_weight  = 1.0;
+float maximum_value_weight  = 1.0;
+float invasion_malus_weight = 1.0;
+float territorial_weight    = 1.0;
+float strategical_weight    = 1.0;
+float attack_dragon_weight  = 1.0;
+float followup_weight       = 1.0;
