@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -66,15 +66,15 @@ static struct influence_data escape_influence;
 /* Pointer to influence data used during pattern matching. */
 static struct influence_data *current_influence = NULL;
 
-#if COSMIC_GNUGO
 
-/* Threholds values used in the whose_moyo() function */
+/* Thresholds values used in the whose_moyo() functions */
 struct moyo_determination_data moyo_data;
+struct moyo_determination_data moyo_restricted_data;
  
-/* Threholds value used in the whose_territory() function */
-float territory_determination_value;
+/* Thresholds value used in the whose_territory() function */
+float territory_determination_value; 
  
-#endif
+
 
 /* This curve determines how much influence is needed at least to claim
  * an intersection as territory, in dependence of the "center value".
@@ -382,41 +382,68 @@ init_influence(struct influence_data *q, int color,
   int ii;
   float attenuation;
   
-#if COSMIC_GNUGO
-    
-  float t;
-
   /* Initialisation of some global positional values, based on 
    * game stage. 
    */
-    if ((board_size != 19) || (movenum <= 2) || ((movenum / 2) % 2)   )
+  if (cosmic_gnugo) {
+    float t;
+
+    if ((board_size != 19) || (movenum <= 2) || ((movenum / 2) % 2))
       cosmic_importance = 0.0;
     else {
-      cosmic_importance = 1.0 - (movenum / 120.0)*(movenum / 120.0); 
+      cosmic_importance = 1.0 - (movenum / 150.0)*(movenum / 150.0); 
       cosmic_importance = gg_max(0.0, cosmic_importance);
     }
 
     t = cosmic_importance;
-    moyo_data.influence_balance     = t * 5.0  +  (1.0-t) * 7.0;  
-    moyo_data.my_influence_minimum  = t * 5.0  +  (1.0-t) * 5.0;
-    moyo_data.opp_influence_maximum = t * 20.0 +  (1.0-t) * 10.0;
     
-    territory_determination_value   = t * 0.95 +  (1.0-t) * 0.95;  
-     
-    /* non-cosmic values are: 
-     * { 6,  0.0, 24.0, { 6.0, 15.0, 26.0, 36.0, 45.0, 50.0, 55.0 }};
-     */
-      
-    min_infl_for_territory.values[0] = t * 20.0  +  (1.0-t) *  6.0;
-    min_infl_for_territory.values[1] = t * 20.0  +  (1.0-t) * 15.0;
-    min_infl_for_territory.values[2] = t * 20.0  +  (1.0-t) * 26.0;
-    min_infl_for_territory.values[3] = t * 25.0  +  (1.0-t) * 36.0;
-    min_infl_for_territory.values[4] = t * 25.0  +  (1.0-t) * 45.0;
-    min_infl_for_territory.values[5] = t * 25.0  +  (1.0-t) * 50.0;
-    min_infl_for_territory.values[6] = t * 25.0  +  (1.0-t) * 55.0;   
+    moyo_data.influence_balance     = t * 15.0  +  (1.0-t) * 5.0;  
+    moyo_data.my_influence_minimum  = t * 5.0   +  (1.0-t) * 5.0;
+    moyo_data.opp_influence_maximum = t * 30.0  +  (1.0-t) * 30.0;
     
-#endif  
+    /* we use the same values for moyo and moyo_restricted */
+    moyo_restricted_data.influence_balance     = moyo_data.influence_balance;
+    moyo_restricted_data.my_influence_minimum  = moyo_data.my_influence_minimum;
+    moyo_restricted_data.opp_influence_maximum = moyo_data.opp_influence_maximum;
 
+    territory_determination_value   = t * 0.95 +  (1.0-t) * 0.95; 
+      
+    min_infl_for_territory.values[0] = t * 6.0   +  (1.0-t) * 10.0;
+    min_infl_for_territory.values[1] = t * 10.0  +  (1.0-t) * 15.0;
+    min_infl_for_territory.values[2] = t * 20.0  +  (1.0-t) * 15.0;
+    min_infl_for_territory.values[3] = t * 20.0  +  (1.0-t) * 20.0;
+    min_infl_for_territory.values[4] = t * 20.0  +  (1.0-t) * 20.0;
+    min_infl_for_territory.values[5] = t * 15.0  +  (1.0-t) * 15.0;
+    min_infl_for_territory.values[6] = t * 10.0  +  (1.0-t) * 15.0;   
+    
+  }
+  else {  
+  
+    /* non-cosmic values */
+    
+    cosmic_importance = 0.0;
+  
+    moyo_data.influence_balance     = 7.0;
+    moyo_data.my_influence_minimum  = 5.0;
+    moyo_data.opp_influence_maximum = 10.0;
+    
+    moyo_restricted_data.influence_balance     = 10.0;
+    moyo_restricted_data.my_influence_minimum  = 10.0;
+    moyo_restricted_data.opp_influence_maximum = 10.0;
+    
+    territory_determination_value = 0.95;
+    
+    min_infl_for_territory.values[0] = 6.0;
+    min_infl_for_territory.values[1] = 15.0;
+    min_infl_for_territory.values[2] = 26.0;
+    min_infl_for_territory.values[3] = 36.0;
+    min_infl_for_territory.values[4] = 45.0;
+    min_infl_for_territory.values[5] = 50.0;
+    min_infl_for_territory.values[6] = 55.0; 
+    
+  }
+  
+  
   if (q != &escape_influence) {
     q->color_to_move = color;
     if (q->is_territorial_influence)
@@ -653,7 +680,7 @@ add_marked_intrusions(struct influence_data *q, int color)
  * B - Intrusion patterns, adding a low intensity influence source.
  * E - Enhance patterns, FIXME: document this one!
  * t - Non-territory patterns, marking vertices as not territory.
- * I - Invasion patterns, adding a low intensity influence source.
+ * I - Invasion patterns, adding a low intensity influence source. 
  * e - Escape bonus. Used together with I to increase the value substantially
  *     if escape influence is being computed.
  *
@@ -803,66 +830,44 @@ influence_callback(int anchor, int color, struct pattern *pattern, int ll,
     pattern->autohelper(ll, pos, color, INFLUENCE_CALLBACK);
   }
   
-#if COSMIC_GNUGO  
 
   /* For I patterns, add a low intensity, both colored, influence
    * source at *.
    */
   if (pattern->class & CLASS_I) {
     int this_color = EMPTY;
-    float t = 0.15 + (1.0 - cosmic_importance);
-    float source_strength;
+    float strength;
+    float attenuation;
 
     if (q->color_to_move == EMPTY || (pattern->class & CLASS_s))
       this_color = BLACK | WHITE;
     else if (q->color_to_move != color)
       this_color = q->color_to_move;
       
-    t = gg_min(1.0, t);
-    t = gg_max(0.0, t);
+    if (cosmic_gnugo) {
+      float t = 0.15 + (1.0 - cosmic_importance);
+      t = gg_min(1.0, t);
+      t = gg_max(0.0, t);
+      strength = t * pattern->value;  
+      attenuation = 1.6;
+    }
+    else {
+      strength = pattern->value;  
+      attenuation = 1.5;
+    }
+
+     /* Increase strength if we're computing escape influence. */
+     if (q == &escape_influence && (pattern->class & CLASS_e))
+       add_influence_source(pos, this_color, 20 * strength, attenuation, q);
+     else
+       add_influence_source(pos, this_color, strength, attenuation, q);
+
+     DEBUG(DEBUG_INFLUENCE,
+	  "  low intensity influence source at %1m, strength %f, color %C\n",
+	  pos, strength, this_color);
+     return;
+  }
     
-    source_strength = t * pattern->value;  
-
-    /* Increase strength if we're computing escape influence. */
-    if (q == &escape_influence && (pattern->class & CLASS_e))
-      add_influence_source(pos, this_color, 20 * source_strength, 1.6, q);
-    else
-      add_influence_source(pos, this_color, source_strength, 1.6, q);
-
-    DEBUG(DEBUG_INFLUENCE,
-	  "  low intensity influence source at %1m, strength %f, color %C\n",
-	  pos, pattern->value, this_color);
-    return;
-  }
-
-#else
-
-  /* For I patterns, add a low intensity, both colored, influence
-   * source at *.
-   */
-  if (pattern->class & CLASS_I) {
-    int this_color = EMPTY;
-    if (q->color_to_move == EMPTY || (pattern->class & CLASS_s))
-      this_color = BLACK | WHITE;
-    else if (q->color_to_move != color)
-      this_color = q->color_to_move;
-
-    /* Increase strength if we're computing escape influence. */
-    if (q == &escape_influence && (pattern->class & CLASS_e))
-      add_influence_source(pos, this_color,
-			   20 * pattern->value, 1.5, q);
-    else
-      add_influence_source(pos, this_color, pattern->value, 1.5, q);
-
-    DEBUG(DEBUG_INFLUENCE,
-	  "  low intensity influence source at %1m, strength %f, color %C\n",
-	  pos, pattern->value, this_color);
-    return;
-  }
-
-
-#endif
-  
   /* For E patterns, add a new influence source of the same color and
    * pattern defined strength at *.
    */
@@ -887,10 +892,7 @@ influence_callback(int anchor, int color, struct pattern *pattern, int ll,
 	    && pattern->patn[k].att == ATT_not)) {
       /* transform pattern real coordinate */
       int ii = AFFINE_TRANSFORM(pattern->patn[k].offset, ll, anchor);
-#if COSMIC_GNUGO
-      float t = 0.15 + (1.0 - cosmic_importance);
-      float source_strength;
-#endif
+      float strength;
 
       /* Territorial connection, making a barrier for opponent influence. */
       if (pattern->class & (CLASS_A | CLASS_D)) {
@@ -917,39 +919,26 @@ influence_callback(int anchor, int color, struct pattern *pattern, int ll,
 	}
       }
 
-#if COSMIC_GNUGO
-      t = gg_min(1.0, t);
-      t = gg_max(0.0, t);
+     if (cosmic_gnugo) {
+        float t = 0.15 + (1.0 - cosmic_importance);
+        t = gg_min(1.0, t);
+        t = gg_max(0.0, t);
+        strength = t * pattern->value;
+      }
+      else 
+        strength = pattern->value;
       
-      source_strength = t * pattern->value;  
-      
-      /* Low intensity influence source for the color in turn to move. */
+      /* Low intensity influence source for the color in turn to move. */  
       if (pattern->class & CLASS_B) {
         if (q->is_territorial_influence)
-          enter_intrusion_source(anchor, ii, source_strength, 
+          enter_intrusion_source(anchor, ii, strength, 
 	  		         TERR_DEFAULT_ATTENUATION, q);
         else
-          add_influence_source(ii, color, source_strength, 
+          add_influence_source(ii, color, strength, 
                                  DEFAULT_ATTENUATION, q); 
 	DEBUG(DEBUG_INFLUENCE, "  intrusion at %1m\n", ii);
       }
-      
-#else
-
-      /* Low intensity influence source for the color in turn to move. */
-      if (pattern->class & CLASS_B) {
-        if (q->is_territorial_influence)
-          enter_intrusion_source(anchor, ii, pattern->value,
-	  		         TERR_DEFAULT_ATTENUATION, q);
-        else
-          add_influence_source(ii, color,
-			       pattern->value, DEFAULT_ATTENUATION, q);
-	DEBUG(DEBUG_INFLUENCE, "  intrusion at %1m\n", ii);
-      }
-
-
-#endif
-      
+            
     }
   }
 }
@@ -1104,6 +1093,7 @@ find_influence_patterns(struct influence_data *q, int color)
  */
 static void
 do_compute_influence(int color, const char safe_stones[BOARDMAX],
+		     const char inhibited_sources[BOARDMAX],
     		     const float strength[BOARDMAX], struct influence_data *q,
 		     int move, const char *trace_message)
 {
@@ -1118,7 +1108,7 @@ do_compute_influence(int color, const char safe_stones[BOARDMAX],
   modify_depth_values(1 - stackp);
   
   for (ii = BOARDMIN; ii < BOARDMAX; ii++)
-    if (ON_BOARD(ii)) {
+    if (ON_BOARD(ii) && !(inhibited_sources && inhibited_sources[ii])) {
       if (q->white_strength[ii] > 0.0)
 	accumulate_influence(q, ii, WHITE);
       if (q->black_strength[ii] > 0.0)
@@ -1160,6 +1150,7 @@ compute_influence(int color, const char safe_stones[BOARDMAX],
 		  int move, const char *trace_message)
 {
   int save_debug = debug;
+  VALGRIND_MAKE_WRITABLE(q, sizeof(*q));
 
   q->is_territorial_influence = 1;
   q->color_to_move = color;
@@ -1175,7 +1166,8 @@ compute_influence(int color, const char safe_stones[BOARDMAX],
   influence_id++;
   q->id = influence_id;
 
-  do_compute_influence(color, safe_stones, strength, q, move, trace_message);
+  do_compute_influence(color, safe_stones, NULL, strength,
+		       q, move, trace_message);
 
   debug = save_debug;
 }
@@ -1192,23 +1184,63 @@ whose_territory(const struct influence_data *q, int pos)
 
   ASSERT_ON_BOARD1(pos);
 
-#if COSMIC_GNUGO
   if (bi > 0.0 && wi == 0.0 && terr < -territory_determination_value)
-    return BLACK;
+     return BLACK;
   if (wi > 0.0 && bi == 0.0 && terr > territory_determination_value)
-    return WHITE;
-
-#else
-
-  if (bi > 0.0 && wi == 0.0 && terr < -0.95)
-    return BLACK;
-  if (wi > 0.0 && bi == 0.0 && terr > 0.95)
-    return WHITE;
-#endif
+     return WHITE;
 
   return EMPTY;
 }
 
+
+/* whose_loose_territory()
+ *
+ * Another function that return the color of the player who has
+ * (almost) territory at pos. If it's territory for neither color, 
+ * EMPTY is returned.
+ * If the global flag use_optimistic_territory is set to 1,
+ * we use a much looser definition of "territory" (essentially,
+ * we use moyo instead of therritory in that case). This will
+ * enable us to use the break-in code to break into moyos instead
+ * of inside hard territories. 
+ */
+int
+whose_loose_territory(const struct influence_data *q, int pos)
+{
+  float bi = q->black_influence[pos];
+  float wi = q->white_influence[pos];
+  int owner;
+
+  ASSERT_ON_BOARD1(pos);
+  
+  /* if the global flag use_optimistic_territory is not 
+     set, be very strict. */
+  if (!use_optimistic_territory)
+    return whose_territory(q, pos);
+  
+  
+  /* If set, we use essentially a somewhat conservative 
+   * version of whose_moyo().  */
+   
+  owner = whose_territory(q, pos);
+  
+  if ((owner == EMPTY) && (bi > 10.0 * wi && bi > 7.0 && wi < 40.0))
+    owner = BLACK;
+  if ((owner == EMPTY) && (wi > 10.0 * bi && wi > 7.0 && bi < 40.0))
+    owner = WHITE;
+
+  if (ON_BOARD(pos) && IS_STONE(board[pos]) && !q->safe[pos]) { 
+    if (0) 
+      TRACE("dead stone : %1m, territory for %s\n", pos,
+                  (owner == WHITE ? "White": 
+                  (owner == BLACK ? "Black": "nobody????")));
+    owner = OTHER_COLOR(board[pos]);
+  }
+    
+  return owner;
+}
+
+
 /* Return the color who has a moyo at (pos). If neither color has a
  * moyo there, EMPTY is returned. The definition of moyo in terms of the
  * influences is totally ad hoc.
@@ -1223,21 +1255,14 @@ whose_moyo(const struct influence_data *q, int pos)
   if (territory_color != EMPTY)
     return territory_color;
     
-#if COSMIC_GNUGO
-  if (bi > moyo_data.influence_balance * wi && 
-      bi > moyo_data.my_influence_minimum && 
-      wi < moyo_data.opp_influence_maximum)
+  if (bi > moyo_data.influence_balance * wi
+      && bi > moyo_data.my_influence_minimum
+      && wi < moyo_data.opp_influence_maximum)
     return BLACK;
-  if (wi > moyo_data.influence_balance * bi && 
-      wi > moyo_data.my_influence_minimum && 
-      bi < moyo_data.opp_influence_maximum)
+  if (wi > moyo_data.influence_balance * bi
+      && wi > moyo_data.my_influence_minimum
+      && bi < moyo_data.opp_influence_maximum)
     return WHITE;
-#else
-  if (bi > 7.0 * wi && bi > 5.0 && wi < 10.0)
-    return BLACK;
-  if (wi > 7.0 * bi && wi > 5.0 && bi < 10.0)
-    return WHITE;
-#endif
   
   return EMPTY;
 }
@@ -1268,17 +1293,18 @@ whose_moyo_restricted(const struct influence_data *q, int pos)
   /* default */
   if (territory_color != EMPTY)
     color = territory_color;
-  else if (bi > 10.0 * wi && bi > 10.0 && wi < 10.0)
+  else if (bi > moyo_restricted_data.influence_balance * wi
+           && bi > moyo_restricted_data.my_influence_minimum
+           && wi < moyo_restricted_data.opp_influence_maximum)
     color = BLACK;
-  else if (wi > 10.0 * bi && wi > 10.0 && bi < 10.0)
-    color = WHITE;
+  else if (wi > moyo_restricted_data.influence_balance * bi
+           && wi > moyo_restricted_data.my_influence_minimum
+           && bi < moyo_restricted_data.opp_influence_maximum)
+    color = WHITE; 
   else
     color = EMPTY;
-  
-  if (color == WHITE && (white_eye[pos].type & INHIBIT_CONNECTION))
-    return EMPTY;
-  
-  if (color == BLACK && (black_eye[pos].type & INHIBIT_CONNECTION))
+
+  if (cut_possible(pos, OTHER_COLOR(color)))
     return EMPTY;
   
   return color;
@@ -1329,7 +1355,7 @@ value_territory(struct influence_data *q)
         float diff = 0.0;
         if (q->white_influence[ii] + q->black_influence[ii] > 0)
           diff = (q->white_influence[ii] - q->black_influence[ii])
-                /(q->white_influence[ii] + q->black_influence[ii]);
+                 / (q->white_influence[ii] + q->black_influence[ii]);
         first_guess[ii] = diff * diff * diff;
 
 	/* If both side have small influence, we have to reduce this value.
@@ -1516,7 +1542,13 @@ segment_influence(struct influence_data *q)
       q->moyo_segmentation[ii] = 0;
       q->area_segmentation[ii] = 0;
     }
-  segment_region(q, whose_territory, IS_TERRITORY, q->territory_segmentation);
+    
+  if (use_optimistic_territory)
+     segment_region(q, whose_loose_territory, 
+                    IS_TERRITORY, q->territory_segmentation);
+  else 
+     segment_region(q, whose_territory, 
+                    IS_TERRITORY, q->territory_segmentation); 
   segment_region(q, whose_moyo_restricted, IS_MOYO, q->moyo_segmentation);
   segment_region(q, whose_area,      IS_AREA,      q->area_segmentation);
 }
@@ -1733,6 +1765,7 @@ compute_followup_influence(const struct influence_data *base,
 
 void
 compute_escape_influence(int color, const char safe_stones[BOARDMAX],
+			 const char goal[BOARDMAX],
     			 const float strength[BOARDMAX],
     			 char escape_value[BOARDMAX])
 {
@@ -1748,33 +1781,37 @@ compute_escape_influence(int color, const char safe_stones[BOARDMAX],
   static char escape_values[BOARDMAX][2];
   static int active_caches[2] = {0, 0};
 
-  /* Encode the values of color and dragons_known into an integer
-   * between 0 and 3.
-   */
   int cache_number = (color == WHITE);
 
-  int board_was_cached = 1;
+  VALGRIND_MAKE_WRITABLE(&escape_influence, sizeof(escape_influence));
 
-  /* Notice that we compare the out of board markers as well, in case
-   * the board size should have changed between calls.
-   */
-  for (ii = BOARDMIN; ii < BOARDMAX; ii++) {
-    if (cached_board[ii] != board[ii]) {
-      cached_board[ii] = board[ii];
-      board_was_cached = 0;
+  if (!goal) {
+    /* Encode the values of color and dragons_known into an integer
+     * between 0 and 3.
+     */
+    int board_was_cached = 1;
+
+    /* Notice that we compare the out of board markers as well, in
+     * case the board size should have changed between calls.
+     */
+    for (ii = BOARDMIN; ii < BOARDMAX; ii++) {
+      if (cached_board[ii] != board[ii]) {
+	cached_board[ii] = board[ii];
+	board_was_cached = 0;
+      }
     }
-  }
 
-  if (!board_was_cached)
-    for (k = 0; k < 2; k++)
-      active_caches[k] = 0;
-  
-  if (active_caches[cache_number]) {
-    for (ii = BOARDMIN; ii < BOARDMAX; ii++)
-      if (ON_BOARD(ii))
-	escape_value[ii] = escape_values[ii][cache_number];
-    
-    return;
+    if (!board_was_cached)
+      for (k = 0; k < 2; k++)
+	active_caches[k] = 0;
+
+    if (active_caches[cache_number]) {
+      for (ii = BOARDMIN; ii < BOARDMAX; ii++)
+	if (ON_BOARD(ii))
+	  escape_value[ii] = escape_values[ii][cache_number];
+
+      return;
+    }
   }
 
   /* Use enhance pattern and higher attenuation for escape influence. */
@@ -1787,7 +1824,7 @@ compute_escape_influence(int color, const char safe_stones[BOARDMAX],
   if (!(debug & DEBUG_ESCAPE))
     debug &= ~DEBUG_INFLUENCE;
 
-  do_compute_influence(OTHER_COLOR(color), safe_stones, strength,
+  do_compute_influence(OTHER_COLOR(color), safe_stones, goal, strength,
       		       &escape_influence, -1, NULL);
 
   debug = save_debug;
@@ -1795,13 +1832,35 @@ compute_escape_influence(int color, const char safe_stones[BOARDMAX],
   for (ii = BOARDMIN; ii < BOARDMAX; ii++)
     if (ON_BOARD(ii)) {
       if (whose_moyo(&escape_influence, ii) == color)
-       escape_value[ii] = 4;
+	escape_value[ii] = 4;
       else if (whose_area(&escape_influence, ii) == color)
-       escape_value[ii] = 2;
-      else if (whose_area(&escape_influence, ii) == EMPTY)
-       escape_value[ii] = 1;
+	escape_value[ii] = 2;
+      else if (whose_area(&escape_influence, ii) == EMPTY) {
+	if (goal) {
+	  escape_value[ii] = 0;
+
+	  if (!goal[ii]) {
+	    int goal_proximity = 0;
+
+	    for (k = 0; k < 8; k++) {
+	      if (ON_BOARD(ii + delta[k])) {
+		goal_proximity += 2 * goal[ii + delta[k]];
+		if (k < 4 && ON_BOARD(ii + 2 * delta[k]))
+		  goal_proximity += goal[ii + delta[k]];
+	      }
+	      else
+		goal_proximity += 1;
+	    }
+
+	    if (goal_proximity < 6)
+	      escape_value[ii] = 1;
+	  }
+	}
+	else
+	  escape_value[ii] = 1;
+      }
       else
-       escape_value[ii] = 0;
+	escape_value[ii] = 0;
     }
 
   if (0 && (debug & DEBUG_ESCAPE) && verbose > 0) {
@@ -1813,17 +1872,20 @@ compute_escape_influence(int color, const char safe_stones[BOARDMAX],
 			    "%3.0f", 3, 1, 1);
   }    
 
-  /* Save the computed values in the cache. */
-  for (ii = BOARDMIN; ii < BOARDMAX; ii++)
-    if (ON_BOARD(ii))
-      escape_values[ii][cache_number] = escape_value[ii];
-  active_caches[cache_number] = 1;
+  if (!goal) {
+    /* Save the computed values in the cache. */
+    for (ii = BOARDMIN; ii < BOARDMAX; ii++)
+      if (ON_BOARD(ii))
+	escape_values[ii][cache_number] = escape_value[ii];
+    active_caches[cache_number] = 1;
+  }
 }
 
 
 /* Cache of delta_territory_values. */
 static float delta_territory_cache[BOARDMAX];
 static float followup_territory_cache[BOARDMAX];
+Hash_data delta_territory_cache_hash[BOARDMAX];
 static int territory_cache_position_number = -1;
 static int territory_cache_influence_id = -1;
 static int territory_cache_color = -1;
@@ -1837,14 +1899,12 @@ static int territory_cache_color = -1;
 int 
 retrieve_delta_territory_cache(int pos, int color, float *move_value,
     			       float *followup_value,
-			       const struct influence_data *base)
+			       const struct influence_data *base,
+			       Hash_data safety_hash)
 {
   ASSERT_ON_BOARD1(pos);
   ASSERT1(IS_STONE(color), pos);
 
-#if 1
-  return 0;
-#endif
   /* We check whether the color, the board position, or the base influence
    * data has changed since the cache entry got entered.
    */
@@ -1852,6 +1912,11 @@ retrieve_delta_territory_cache(int pos, int color, float *move_value,
       && territory_cache_color == color
       && territory_cache_influence_id == base->id
       && delta_territory_cache[pos] != NOT_COMPUTED) {
+    int i;
+    for (i = 0; i < NUM_HASHVALUES; i++)
+      if (delta_territory_cache_hash[pos].hashval[i]
+	  != safety_hash.hashval[i])
+	return 0;
     *move_value = delta_territory_cache[pos];
     *followup_value = followup_territory_cache[pos];
     if (0) 
@@ -1865,8 +1930,11 @@ retrieve_delta_territory_cache(int pos, int color, float *move_value,
 void 
 store_delta_territory_cache(int pos, int color,
 			    float move_value, float followup_value,
-			    const struct influence_data *base)
+			    const struct influence_data *base,
+			    Hash_data safety_hash)
 {
+  int i;
+
   ASSERT_ON_BOARD1(pos);
   ASSERT1(IS_STONE(color), pos);
 
@@ -1884,6 +1952,8 @@ store_delta_territory_cache(int pos, int color,
   }
   delta_territory_cache[pos] = move_value;
   followup_territory_cache[pos] = followup_value;
+  for (i = 0; i < NUM_HASHVALUES; i++)
+    delta_territory_cache_hash[pos].hashval[i] = safety_hash.hashval[i];
   if (0)
     gprintf("%1m: Stored delta territory cache: %f, %f\n", pos, move_value,
 	    followup_value);
@@ -2051,13 +2121,31 @@ void
 get_influence(const struct influence_data *q,
 	      float white_influence[BOARDMAX],
 	      float black_influence[BOARDMAX],
-	      int influence_regions[BOARDMAX])
+	      float white_strength[BOARDMAX],
+	      float black_strength[BOARDMAX],
+	      float white_attenuation[BOARDMAX], 
+	      float black_attenuation[BOARDMAX],
+	      float white_permeability[BOARDMAX],
+	      float black_permeability[BOARDMAX],
+	      float territory_value[BOARDMAX],
+	      int influence_regions[BOARDMAX],
+	      int non_territory[BOARDMAX])
 {
   int ii;
-  for (ii = BOARDMIN; ii < BOARDMAX; ii++)
+  
+  for (ii = BOARDMIN; ii < BOARDMAX; ii++) {
+    white_influence[ii] = q->white_influence[ii];
+    black_influence[ii] = q->black_influence[ii];
+    white_strength[ii] = q->white_strength[ii];
+    black_strength[ii] = q->black_strength[ii];
+    white_attenuation[ii] = q->white_attenuation[ii];
+    black_attenuation[ii] = q->black_attenuation[ii];
+    white_permeability[ii] = q->white_permeability[ii];
+    black_permeability[ii] = q->black_permeability[ii];
+    territory_value[ii] = q->territory_value[ii];
+    non_territory[ii] = q->non_territory[ii];
+
     if (board[ii] == EMPTY) {
-      white_influence[ii] = q->white_influence[ii];
-      black_influence[ii] = q->black_influence[ii];
       if (whose_territory(q, ii) == WHITE)
 	influence_regions[ii] = 3;
       else if (whose_territory(q, ii) == BLACK)
@@ -2073,6 +2161,11 @@ get_influence(const struct influence_data *q,
       else
 	influence_regions[ii] = 0;
     }
+    else if (board[ii] == WHITE)
+      influence_regions[ii] = 4;
+    else if (board[ii] == BLACK)
+      influence_regions[ii] = -4;
+  }
 }
   
 
