@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -28,12 +28,9 @@
 #include "liberty.h"
 
 
-/* STEPHANE NICOLET's modification : playing for influence in the fuseki */
 
-#if COSMIC_GNUGO
-
-
-/* We use much more influence than the defaults attenuation coefficients !
+/* The cosmic style uses more influence than the defaults attenuation 
+ * coefficients !
  * The "TERR_.."-values are used in the influence computations used
  * for territory evaluation. (initial_influence with dragons_known,
  * move_influence)
@@ -50,23 +47,6 @@
 	(cosmic_importance * 2.5 + (1.0 - cosmic_importance) * 1.7)
 
 
-#else
-
-
-/* Default attenuation coefficient.
- * The "TERR_.."-values are used in the influence computations used
- * for territory evaluation. (initial_influence with dragons_known,
- * move_influence)
- */
-#define DEFAULT_ATTENUATION 3.0
-#define TERR_DEFAULT_ATTENUATION 2.4
-
-/* Extra damping coefficient for spreading influence diagonally. */
-#define DIAGONAL_DAMPING 2.0
-#define TERR_DIAGONAL_DAMPING 1.7
-
-#endif
-
 
 
 
@@ -149,7 +129,7 @@ struct influence_data
 };
 
 /* Typedef for pointer to either of the functions whose_territory(),
- * whose_moyo(), and whose_area().
+ * whose_loose_territory(), whose_moyo(), and whose_area().
  */
 typedef int (*owner_function_ptr)(const struct influence_data *q, int pos);
 
@@ -160,8 +140,6 @@ typedef int (*owner_function_ptr)(const struct influence_data *q, int pos);
 
 
 
-#if COSMIC_GNUGO
-
 /* cosmic_importance is a number between 0.0 and 1.0 ;
  * when cosmic_importance is 0.0, the default influence
  * values are used; when cosmic_importance is 1.0, GNU Go
@@ -182,8 +160,6 @@ struct moyo_determination_data
 };
 
 
-#endif
-
 
 
 /*
