@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -39,8 +39,14 @@
 void
 init_gnugo(float memory, unsigned int seed)
 {
-  set_random_seed(seed);
+  /* We need a fixed seed when initializing the Zobrist hashing to get
+   * reproducable results.
+   * FIXME: Test the quality of the seed.
+   */
+  set_random_seed(12345);
   reading_cache_init(memory * 1024 * 1024);
+  set_random_seed(seed);
+  persistent_cache_init();
   clear_board();
 
   transformation_init();
@@ -339,19 +345,11 @@ gnugo_who_wins(int color, FILE *outfile)
 float
 gnugo_estimate_score(float *upper, float *lower)
 {
-  silent_examine_position(WHITE, EXAMINE_DRAGONS);
+  silent_examine_position(EXAMINE_DRAGONS);
   return estimate_score(upper, lower);
 }
 
 
-/* Interface to examine_position(). */
-
-void
-gnugo_examine_position(int color, int how_much)
-{
-  examine_position(color, how_much);
-}
-
 /* Accessor functions for internal board state. */
 
 /* Report the komi. */
@@ -504,7 +502,6 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
   
   int untilm = -1, untiln = -1;
   int until = 9999;
-  int addstone = 0;          /* handicap stone detector */
   
   if (!sgfGetFloatProperty(tree->root, "KM", &komi)) {
     if (gameinfo->handicap == 0)
@@ -524,10 +521,14 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
       DEBUG(DEBUG_LOADSGF, "Loading until move %d\n", until);
     }
     else {
-      untiln = *untilstr - 'A';
-      if (*untilstr >= 'I')
+      if ('A' <= *untilstr && *untilstr <= 'Z')
+	untiln = *untilstr - 'A';
+      else
+	untiln = *untilstr - 'a';
+
+      if (untiln >= 'I' - 'A')
 	--untiln;
-	  
+
       untilm = board_size - atoi(untilstr+1);
       DEBUG(DEBUG_LOADSGF, "Loading until move at %d,%d (%m)\n", 
 	    untilm, untiln, untilm, untiln);
@@ -567,7 +568,6 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
 		  i, j);
 	else
 	  gnugo_add_stone(i, j, BLACK);
-	addstone = 1;
 	break;
 	      
       case SGFAW:
@@ -578,16 +578,9 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
 		  i, j);
 	else
 	  gnugo_add_stone(i, j, WHITE);
-	addstone = 1;
 	break;
 	      
       case SGFPL:
-	/* following really should not be needed for proper sgf file */
-	if (movenum != 0 && !addstone) {
-	  gnugo_sethand(gameinfo->handicap, 0);
-	  sgfOverwritePropertyInt(tree->root, "HA", gameinfo->handicap);
-	}
-
 	/* Due to a bad comment in the SGF FF3 definition (in the
          * "Alphabetical list of properties" section) some
          * applications encode the colors with 1 for black and 2 for
@@ -599,13 +592,20 @@ gameinfo_play_sgftree_rot(Gameinfo *gameinfo, SGFTree *tree,
 	  next = WHITE;
 	else
 	  next = BLACK;
+	/* following really should not be needed for proper sgf file */
+	if (stones_on_board(GRAY) == 0
+	    && next == WHITE) {
+	  gnugo_sethand(gameinfo->handicap, 0);
+	  sgfOverwritePropertyInt(tree->root, "HA", gameinfo->handicap);
+	}
 	break;
 	      
       case SGFW:
       case SGFB:
 	next = prop->name == SGFW ? WHITE : BLACK;
 	/* following really should not be needed for proper sgf file */
-	if (movenum != 0 && !addstone) {
+	if (stones_on_board(GRAY) == 0
+	    && next == WHITE) {
 	  gnugo_sethand(gameinfo->handicap, 0);
 	  sgfOverwritePropertyInt(tree->root, "HA", gameinfo->handicap);
 	}
