@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -23,194 +23,32 @@
 #ifndef _LIBERTY_H_
 #define _LIBERTY_H_
 
+#include "board.h"
+#include "hash.h"
 #include "gnugo.h"
 
-/* local versions of absolute value, min and max */
-
-#define gg_abs(x) ((x) < 0 ? -(x) : (x))
-#define gg_min(a, b) ((a)<(b) ? (a) : (b))
-#define gg_max(a, b) ((a)<(b) ? (b) : (a))
-
-/* Avoid compiler warnings with unused parameters */
-#define UNUSED(x)  (void)x
-
-
 /* ================================================================ */
 /*                           public variables                       */
 /* ================================================================ */
 
 
-/* We need the defintion of type Hash_data here. */
-#include "hash.h"
-
-/* Other modules get read-only access to this variable. */
-extern Hash_data    hashdata;
-
-
-
 /* ================================================================ */
 
 
 #define FALSE_EYE          1
 #define HALF_EYE           2
-#define INHIBIT_CONNECTION 4
 
 
-/* A string with n stones can have at most 2(n+1) liberties. From this
- * follows that an upper bound on the number of liberties of a string
- * on a board of size N^2 is 2/3 (N^2+1).
- */
-#define MAXLIBS   (2*(MAX_BOARD*MAX_BOARD + 1)/3)
-/* This is a smaller, practical number of liberties that we care to keep track of. */
-#define MAX_LIBERTIES 20
-
-
-/* This is an upper bound of the number of strings that can exist on
- * the board simultaneously.  
- * FIXME: This is not sufficiently large;  above stackp==0, the incremental 
- *   board code doesn't necessarily re-use all indices.  This is a problem
- *   only in very pathological cases, and is extremely unlikely to occur in
- *   practice.
- */
-#define MAX_STRINGS (2 * MAX_BOARD * MAX_BOARD / 3)
-
-/* Per gf: Unconditional_life() can get very close to filling the 
- * entire board under certain circumstances. This was discussed in 
- * the list around August 21, 2001, in a thread with the subject 
- * "gnugo bug logs".
- */
-#define MAXSTACK  MAX_BOARD * MAX_BOARD
-#define MAXCHAIN  160
-
-/* 1D board macros.
- * Note that POS(-1, -1) == 0
- * DELTA() is defined so that POS(i+di, j+dj) = POS(i, j) + DELTA(di, dj).
- */
-#define BOARDSIZE     ((MAX_BOARD + 2) * (MAX_BOARD + 1) + 1)
-#define BOARDMIN      (MAX_BOARD + 2)
-#define BOARDMAX      (MAX_BOARD + 1) * (MAX_BOARD + 1)
-#define POS(i, j)     ((MAX_BOARD + 2) + (i) * (MAX_BOARD + 1) + (j))
-#define DELTA(di, dj) ((di) * (MAX_BOARD + 1) + (dj))
-#define I(pos)        ((pos) / (MAX_BOARD + 1) - 1)
-#define J(pos)        ((pos) % (MAX_BOARD + 1) - 1)
-#define PASS_MOVE     0
-#define NO_MOVE       PASS_MOVE
-#define NS            (MAX_BOARD + 1)
-#define WE            1
-#define SOUTH(pos)    ((pos) + NS)
-#define WEST(pos)     ((pos) - 1)
-#define NORTH(pos)    ((pos) - NS)
-#define EAST(pos)     ((pos) + 1)
-#define SW(pos)       ((pos) + NS - 1)
-#define NW(pos)       ((pos) - NS - 1)
-#define NE(pos)       ((pos) - NS + 1)
-#define SE(pos)       ((pos) + NS + 1)
-#define SS(pos)       ((pos) + 2 * NS)
-#define WW(pos)       ((pos) - 2)
-#define NN(pos)       ((pos) - 2 * NS)
-#define EE(pos)       ((pos) + 2)
-
-#define BOARD(i, j)   board[POS(i, j)]
-
 #define REVERSE_RESULT(result)		(WIN - result)
 
-/* Transformation stuff. */
-#define MAX_OFFSET			(2*MAX_BOARD - 1) * (2*MAX_BOARD - 1)
-#define OFFSET(dx, dy)\
-  ((dy + MAX_BOARD - 1) * (2*MAX_BOARD - 1) + (dx + MAX_BOARD - 1))
-#define OFFSET_DELTA(dx, dy)		(OFFSET(dx, dy) - OFFSET(0, 0))
-#define CENTER_OFFSET(offset)		(offset - OFFSET(0, 0))
-#define TRANSFORM(offset, trans)	(transformation[offset][trans])
-#define AFFINE_TRANSFORM(offset, trans, delta)\
-  (transformation[offset][trans] + delta)
-#define TRANSFORM2(x, y, tx, ty, trans)\
-  do {\
-    *tx = transformation2[trans][0][0] * (x) + transformation2[trans][0][1] * (y);\
-    *ty = transformation2[trans][1][0] * (x) + transformation2[trans][1][1] * (y);\
-  } while (0)
-
-
-/* This struct holds the internal board state.
- */
-struct board_state {
-  int board_size;
-
-  Intersection board[BOARDSIZE];
-  int board_ko_pos;
-  int black_captured;
-  int white_captured;
-
-  Intersection initial_board[BOARDSIZE];
-  int initial_board_ko_pos;
-  int initial_white_captured;
-  int initial_black_captured;
-  int move_history_color[MAX_MOVE_HISTORY];
-  int move_history_pos[MAX_MOVE_HISTORY];
-  int move_history_pointer;
-
-  float komi;
-  int move_number;
-};
-
-
-/* board utility functions */
-int find_origin(int str);
-int chainlinks(int str, int adj[MAXCHAIN]);
-int chainlinks2(int str, int adj[MAXCHAIN], int lib);
-int chainlinks3(int str, int adj[MAXCHAIN], int lib);
-int extended_chainlinks(int str, int adj[MAXCHAIN], int both_colors);
-
-
-/* This is increased by one anytime a move is (permanently) played or
- * the board is cleared.
- */
-extern int position_number;
-
-
-/* Detect vertex on edge or corner. */
-int is_edge_vertex(int pos);
-int is_corner_vertex(int pos);
-int edge_distance(int pos);
-
-
-/* Count and/or find liberties at (pos). */
-int countlib(int str);
-int findlib(int str, int maxlib, int *libs);
-int fastlib(int pos, int color, int ignore_captures);
-int approxlib(int pos, int color, int maxlib, int *libs);
-int accuratelib(int pos, int color, int maxlib, int *libs);
-int count_common_libs(int str1, int str2);
-int find_common_libs(int str1, int str2, int maxlib, int *libs);
-int have_common_lib(int str1, int str2, int *lib);
-
 
 void start_timer(int n);
 double time_report(int n, const char *occupation, int move, double mintime);
-
-
-/* Check for self atari. */
-int is_self_atari(int pos, int color);
-
-/* Count the number of stones in a string. */
-int countstones(int str);
-int findstones(int str, int maxstones, int *stones);
-
-/* Exported from incremental_board.c so that reading.c can use it. */
-void incremental_order_moves(int move, int color, int string,
-			     int *number_edges, int *number_same_string,
-			     int *number_own, int *number_opponent,
-			     int *captured_stones, int *threatened_stones,
-			     int *saved_stones, int *number_open);
-
-/* Board caches initialization functions. */
-void clear_approxlib_cache(void);
-void clear_accuratelib_cache(void);
-
+void showstats(void);
+void clearstats(void);
 
 void transformation_init(void);
 
-  
-void dump_stack(void);
 void report_worm(int m, int n);
 void ascii_report_worm(char *string);
 void report_dragon(FILE *outfile, int pos);
@@ -224,41 +62,107 @@ void inv_rotate2(int i, int j, int *ri, int *rj, int rot);
 int rotate1(int pos, int rot);
 int inv_rotate1(int pos, int rot);
 
-int square_dist(int pos1, int pos2);
+/* Routine names used by persistent and non-persistent caching schemes. */
+enum routine_id {
+  OWL_ATTACK,
+  OWL_DEFEND,
+  SEMEAI,
+  FIND_DEFENSE,
+  ATTACK,
+  CONNECT,
+  DISCONNECT,
+  BREAK_IN,
+  BLOCK_OFF,
+  OWL_THREATEN_ATTACK,
+  OWL_THREATEN_DEFENSE,
+  OWL_DOES_DEFEND,
+  OWL_DOES_ATTACK,
+  OWL_CONNECTION_DEFENDS,
+  OWL_SUBSTANTIAL,
+  OWL_CONFIRM_SAFETY,
+  NUM_CACHE_ROUTINES
+};
 
-/* Is this point inside the board? */
-#if 0
-#define ON_BOARD2(i, j) ((i)>=0 && (j)>=0 && (i)<board_size && (j)<board_size)
-#else
-/*
- * For the case when expr can only be slightly negative,
- *    if (expr < 0 || expr > something)
- * is equivalent to
- *    if ((unsigned) expr > something)
+#define ROUTINE_NAMES \
+  "owl_attack", \
+  "owl_defend", \
+  "semeai", \
+  "find_defense", \
+  "attack", \
+  "connect", \
+  "disconnect", \
+  "break_in", \
+  "block_off", \
+  "owl_threaten_attack", \
+  "owl_threatend_defense", \
+  "owl_does_defend", \
+  "owl_does_attack", \
+  "owl_connection_defends", \
+  "owl_substantial", \
+  "owl_confirm_safety"
+
+/* To prioritize between different types of reading, we give a cost
+ * ranking to each of the routines above:
  *
- * (I think gcc knows this trick, but it does no harm to
- *  encode it explicitly since it saves typing !)
+ * 4 semeai
+ * 3 owl
+ * 2 break-in
+ * 1 connection
+ * 0 tactical reading
+ *
+ * -1 is left at the end for a consistency check.
  */
-#define ON_BOARD2(i, j) ((unsigned) (i) < (unsigned) board_size &&\
-		         (unsigned) (j) < (unsigned) board_size)
-#endif
+#define ROUTINE_COSTS \
+  3, 3, 4, 0, 0, 1, 1, 2, 2, 3, 3, 3, 3, 3, 3, 3, -1
+  
 
-#define ASSERT_ON_BOARD2(i, j) ASSERT2(ON_BOARD2((i), (j)), (i), (j))
+const char *routine_id_to_string(enum routine_id routine);
 
-#define ON_BOARD1(pos) (((unsigned) (pos) < BOARDSIZE) && board[pos] != GRAY)
-#define ON_BOARD(pos) (board[pos] != GRAY)
-#define ASSERT_ON_BOARD1(pos) ASSERT1(ON_BOARD1(pos), (pos))
 
-/* Coordinates for the eight directions, ordered
- * south, west, north, east, southwest, northwest, northeast, southeast.
- * Defined in utils.c.
+/* This is used for both the dragon status and safety fields.
+ * Also used for unconditional status in struct worm_data and for the
+ * final status computed by the aftermath code.
  */
-extern int deltai[8]; /* = { 1,  0, -1,  0,  1, -1, -1, 1}; */
-extern int deltaj[8]; /* = { 0, -1,  0,  1, -1, -1,  1, 1}; */
-extern int delta[8];  /* = { NS, -1, -NS, 1, NS-1, -NS-1, -NS+1, NS+1}; */
+enum dragon_status {
+  DEAD,
+  ALIVE,
+  CRITICAL,
+  UNKNOWN,
+  UNCHECKED,
+  CAN_THREATEN_ATTACK,
+  CAN_THREATEN_DEFENSE, 
+  INESSENTIAL,
+  TACTICALLY_DEAD,
+  ALIVE_IN_SEKI,
+  STRONGLY_ALIVE,
+  INVINCIBLE,
+  INSUBSTANTIAL,
+  WHITE_TERRITORY,
+  BLACK_TERRITORY,
+  DAME,
+  NUM_DRAGON_STATUS
+};
+
+#define DRAGON_STATUS_NAMES \
+  "dead", \
+  "alive", \
+  "critical", \
+  "unknown", \
+  "unchecked", \
+  "can threaten attack", \
+  "can threaten defense", \
+  "inessential", \
+  "tactically dead", \
+  "alive in seki", \
+  "strongly alive", \
+  "invincible", \
+  "insubstantial", \
+  "white_territory", \
+  "black_territory", \
+  "dame"
+
+const char *status_to_string(enum dragon_status status);
 
-void store_board(struct board_state *state);
-void restore_board(struct board_state *state);
 
 /* Forward struct declarations. */
 struct pattern;
@@ -316,9 +220,9 @@ int defend_both(int astr, int bstr);
 int break_through(int apos, int bpos, int cpos);
 int attack_threats(int pos, int max_points, int moves[], int codes[]);
 
-int restricted_defend1(int str, int *move, int komaster, int kom_pos,
+int restricted_defend1(int str, int *move,
 		       int num_forbidden_moves, int *forbidden_moves);
-int restricted_attack2(int str, int *move, int komaster, int kom_pos,
+int restricted_attack2(int str, int *move,
 		       int num_forbidden_moves, int *forbidden_moves);
 
 int simple_ladder(int str, int *move);
@@ -327,37 +231,39 @@ void tune_move_ordering(int params[MOVE_ORDERING_PARAMETERS]);
 void draw_reading_shadow(void);
 
 /* persistent.c */
-void purge_persistent_reading_cache(void);
-void clear_persistent_reading_cache(void);
-int search_persistent_reading_cache(int routine, int str, int *result,
-				    int *move);
-void store_persistent_reading_cache(int routine, int str, int result,
-				    int move, int nodes);
-void delete_persistent_reading_cache_entry(int routine, int str);
+void persistent_cache_init(void);
+void purge_persistent_caches(void);
+void clear_persistent_caches(void);
+
+int search_persistent_reading_cache(enum routine_id routine, int str,
+				    int *result, int *move);
+void store_persistent_reading_cache(enum routine_id routine, int str,
+				    int result, int move, int nodes);
 void reading_hotspots(float values[BOARDMAX]);
-void purge_persistent_connection_cache(void);
-void clear_persistent_connection_cache(void);
-int search_persistent_connection_cache(int routine, int str1, int str2,
+int search_persistent_connection_cache(enum routine_id routine,
+				       int str1, int str2,
 				       int *result, int *move);
-void store_persistent_connection_cache(int routine, int str1, int str2,
+void store_persistent_connection_cache(enum routine_id routine,
+				       int str1, int str2,
 				       int result, int move,
 				       int tactical_nodes,
 				       char connection_shadow[BOARDMAX]);
-int search_persistent_breakin_cache(int routine, int str, Hash_data goal_hash,
+int search_persistent_breakin_cache(enum routine_id routine,
+				    int str, Hash_data *goal_hash,
+				    int breakin_node_limit,
 				    int *result, int *move);
-void store_persistent_breakin_cache(int routine, int str, Hash_data goal_hash,
+void store_persistent_breakin_cache(enum routine_id routine,
+				    int str, Hash_data *goal_hash,
 				    int result, int move,
 				    int tactical_nodes,
+				    int breakin_node_limit,
 				    char breakin_shadow[BOARDMAX]);
-void purge_persistent_breakin_cache(void);
-void clear_persistent_breakin_cache(void);
-void print_persistent_breakin_cache(void);
-void purge_persistent_owl_cache(void);
-void clear_persistent_owl_cache(void);
-int search_persistent_owl_cache(int routine, int apos, int bpos, int cpos,
+int search_persistent_owl_cache(enum routine_id routine,
+				int apos, int bpos, int cpos,
 				int *result, int *move, int *move2,
 				int *certain);
-void store_persistent_owl_cache(int routine, int apos, int bpos, int cpos,
+void store_persistent_owl_cache(enum routine_id routine,
+				int apos, int bpos, int cpos,
 				int result, int move, int move2, int certain,
 				int tactical_nodes, char goal[BOARDMAX],
 				int goal_color);
@@ -371,26 +277,6 @@ int non_transitivity(int str1, int str2, int str3, int *move);
 int break_in(int str, const char goal[BOARDMAX], int *move);
 int block_off(int str1, const char goal[BOARDMAX], int *move);
 
-/* board.c */
-int liberty_of_string(int pos, int str);
-int second_order_liberty_of_string(int pos, int str);
-int neighbor_of_string(int pos, int str);
-int has_neighbor(int pos, int color);
-int same_string(int str1, int str2);
-int adjacent_strings(int str1, int str2);
-int is_ko(int pos, int color, int *ko_pos);
-int is_ko_point(int pos);
-int komaster_trymove(int pos, int color,
-		     const char *message, int str,
-		     int komaster, int kom_pos,
-		     int *new_komaster, int *new_kom_pos,
-		     int *is_conditional_ko, int consider_conditional_ko);
-int does_capture_something(int pos, int color);
-void mark_string(int str, char mx[BOARDMAX], char mark);
-int move_in_stack(int pos, int cutoff);
-void get_move_from_stack(int k, int *move, int *color);
-int stones_on_board(int color);
-
 int obvious_false_eye(int pos, int color);
 void estimate_lunch_eye_value(int lunch, int *min, int *probable, int *max,
 			      int appreciate_one_two_lunches);
@@ -399,6 +285,7 @@ int vital_chain(int pos);
 int confirm_safety(int move, int color, int *defense_point,
 		   char safe_stones[BOARDMAX]);
 int dragon_weak(int pos);
+int size_of_biggest_critical_dragon(void);
 float blunder_size(int move, int color, int *defense_point,
 		   char safe_stones[BOARDMAX]);
 void set_depth_values(int level, int report_levels);
@@ -412,12 +299,16 @@ void restore_depth_values(void);
 int safe_move(int move, int color);
 int does_secure(int color, int move, int pos);
 
+void compute_new_dragons(int dragon_origins[BOARDMAX]);
 void join_dragons(int d1, int d2);
 int dragon_escape(char goal[BOARDMAX], int color, char escape_value[BOARDMAX]);
 void compute_refined_dragon_weaknesses(void);
+
 struct eyevalue;
+void compute_dragon_genus(int d, struct eyevalue *genus, int eye_to_exclude);
 float crude_dragon_weakness(int safety, struct eyevalue *genus, int has_lunch,
 			    float moyo_value, float escape_route);
+
 int is_same_dragon(int d1, int d2);
 int are_neighbor_dragons(int d1, int d2);
 void mark_dragon(int pos, char mx[BOARDMAX], char mark);
@@ -442,7 +333,7 @@ int movelist_move_known(int move, int max_points, int points[], int codes[]);
 void movelist_change_point(int move, int code, int max_points, 
 			   int points[], int codes[]);
 
-/* safety.c */
+/* surround.c */
 int compute_surroundings(int pos, int apos, int showboard,
 			 int *surround_size);
 int is_surrounded(int pos);
@@ -466,13 +357,15 @@ void add_antisuji_move(int pos);
 void add_semeai_move(int pos, int dr);
 void add_semeai_threat(int pos, int dr);
 
-void add_owl_attack_move(int pos, int dr, int code);
+void add_owl_attack_move(int pos, int dr, int kworm, int code);
 void add_owl_defense_move(int pos, int dr, int code);
 void add_owl_attack_threat_move(int pos, int dr, int code);
 void add_owl_defense_threat_move(int pos, int dr, int code);
 void add_owl_prevent_threat_move(int pos, int dr);
 void add_owl_uncertain_defense_move(int pos, int dr);
 void add_owl_uncertain_attack_move(int pos, int dr);
+void add_gain_move(int pos, int target1, int target2);
+void add_loss_move(int pos, int target1, int target2);
 
 void add_my_atari_atari_move(int pos, int size);
 void add_your_atari_atari_move(int pos, int size);
@@ -484,19 +377,6 @@ void add_strategical_attack_move(int pos, int dr);
 void add_strategical_defense_move(int pos, int dr);
 void add_worthwhile_threat_move(int pos);
 void add_replacement_move(int from, int to);
-int  set_minimum_move_value(int pos, float value);
-void set_maximum_move_value(int pos, float value);
-void set_minimum_territorial_value(int pos, float value);
-void set_maximum_territorial_value(int pos, float value);
-void add_shape_value(int pos, float value);
-void add_followup_value(int pos, float value);
-void add_reverse_followup_value(int pos, float value);
-void record_top_move(int move, float val);
-void remove_top_move(int move);
-void scale_randomness(int pos, float scaling);
-
-void register_good_attack_threat(int move, int target);
-int is_known_good_attack_threat(int move, int target);
 
 /* Parameters to add_either_move and add_all_move */
 #define ATTACK_STRING  1
@@ -506,9 +386,22 @@ void add_either_move(int pos, int reason1, int target1,
 void add_all_move(int pos, int reason1, int target1,
 		  int reason2, int target2);
 
-void add_gain_move(int pos, int target1, int target2);
-void add_loss_move(int pos, int target1, int target2);
+int set_minimum_move_value(int pos, float value);
+void set_maximum_move_value(int pos, float value);
+void set_minimum_territorial_value(int pos, float value);
+void set_maximum_territorial_value(int pos, float value);
+void add_shape_value(int pos, float value);
+void add_followup_value(int pos, float value);
+void add_reverse_followup_value(int pos, float value);
+int list_move_reasons(FILE *out, int pos);
+void print_all_move_values(void);
+void record_top_move(int move, float val);
+void remove_top_move(int move);
+void scale_randomness(int pos, float scaling);
+void compute_move_probabilities(float probabilities[BOARDMAX]);
 
+void register_good_attack_threat(int move, int target);
+int is_known_good_attack_threat(int move, int target);
 
 int get_attack_threats(int pos, int max_strings, int strings[]);
 int get_defense_threats(int pos, int max_strings, int strings[]);
@@ -529,6 +422,7 @@ int owl_mineye(int pos);
 int owl_maxeye(int pos);
 int owl_proper_eye(int pos);
 int owl_eye_size(int pos);
+int owl_lunch(int str);
 int owl_strong_dragon(int pos);
 void owl_reasons(int color);
 
@@ -555,9 +449,7 @@ int free_handicap_total_stones(void);
 
 /* Various different strategies for finding a move */
 void fuseki(int color);
-void semeai(int color);
-void new_semeai(void);
-void small_semeai(int save_verbose);
+void semeai(void);
 void semeai_move_reasons(int color);
 void shapes(int color);
 void endgame(int color);
@@ -570,16 +462,18 @@ int atari_atari_confirm_safety(int color, int tpos, int *move, int minsize,
 			       const char saved_dragons[BOARDMAX],
 			       const char saved_worms[BOARDMAX]);
 
-int atari_atari_blunder_size(int color, int tpos, int *move,
+int atari_atari_blunder_size(int color, int tpos, char defense_moves[BOARDMAX],
 			     const char safe_stones[BOARDMAX]);
 
 int review_move_reasons(int *move, float *val, int color,
-			float pure_threat_value, float lower_bound,
+			float pure_threat_value, float our_score,
 			int allowed_moves[BOARDMAX]);
+void prepare_move_influence_debugging(int pos, int color);
 int fill_liberty(int *move, int color);
 int aftermath_genmove(int *aftermath_move, int color,
 		      int under_control[BOARDMAX],
 		      int do_capture_dead_stones);
+enum dragon_status aftermath_final_status(int color, int pos);
 
 int owl_attack(int target, int *attack_point, int *certain, int *kworm);
 int owl_defend(int target, int *defense_point, int *certain, int *kworm);
@@ -596,7 +490,8 @@ void owl_analyze_semeai(int apos, int bpos,
 void owl_analyze_semeai_after_move(int move, int color, int apos, int bpos,
 				   int *resulta, int *resultb,
 				   int *semeai_move, int owl,
-				   int *semeai_result_certain);
+				   int *semeai_result_certain,
+				   int recompute_dragons);
 
 void set_search_diamond(int pos);
 void reset_search_mask(void);
@@ -622,6 +517,8 @@ int attack_threat_move_known(int move, int str);
 int defense_threat_move_known(int move, int str);
 void worm_reasons(int color);
 
+int semeai_move_reason_known(int move, int dr);
+
 int does_attack(int move, int str);
 int does_defend(int move, int str);
 int double_atari(int move, int color, float *value,
@@ -634,6 +531,7 @@ int cut_possible(int pos, int color);
 int defend_against(int move, int color, int apos);
 int somewhere(int color, int check_alive, int num_moves, ...);
 int visible_along_edge(int color, int apos, int bpos);
+int test_symmetry_after_move(int move, int color, int strict);
 
 /* Printmoyo values, specified by -m flag. */
 #define PRINTMOYO_TERRITORY         0x01
@@ -699,6 +597,7 @@ void compute_followup_influence(const struct influence_data *base,
 			        struct influence_data *q, 
 		                int move, const char *trace_message);
 void compute_escape_influence(int color, const char safe_stones[BOARDMAX],
+			      const char goal[BOARDMAX],
 			      const float strength[BOARDMAX],
                               char escape_value[BOARDMAX]);
 
@@ -707,12 +606,15 @@ float influence_delta_territory(const struct influence_data *base,
 				int move);
 int retrieve_delta_territory_cache(int pos, int color, float *move_value,
 			           float *followup_value,
-				   const struct influence_data *base);
+				   const struct influence_data *base,
+				   Hash_data safety_hash);
 void store_delta_territory_cache(int pos, int color, float move_value,
 				 float followup_value,
-				 const struct influence_data *base);
+				 const struct influence_data *base,
+			         Hash_data safety_hash);
 
 int whose_territory(const struct influence_data *q, int pos);
+int whose_loose_territory(const struct influence_data *q, int pos);
 int whose_moyo(const struct influence_data *q, int pos);
 int whose_area(const struct influence_data *q, int pos);
 float influence_territory(const struct influence_data *q, int pos, int color);
@@ -726,7 +628,15 @@ void influence_get_moyo_data(const struct influence_data *q,
 void get_influence(const struct influence_data *q,
 		   float white_influence[BOARDMAX],
 		   float black_influence[BOARDMAX],
-		   int regions[BOARDMAX]);
+		   float white_strength[BOARDMAX],
+		   float black_strength[BOARDMAX],
+		   float white_attenuation[BOARDMAX], 
+		   float black_attenuation[BOARDMAX],
+		   float white_permeability[BOARDMAX],
+		   float black_permeability[BOARDMAX],
+		   float territory_value[BOARDMAX],
+		   int influence_regions[BOARDMAX],
+		   int non_territory[BOARDMAX]);
 float influence_score(const struct influence_data *q);
 float game_status(int color);
 void resegment_initial_influence(void);
@@ -740,6 +650,7 @@ void clear_break_in_list(void);
 void break_in_move_reasons(int color);
 
 float estimate_score(float *upper, float *lower);
+void choose_strategy(int color, float our_score, float game_status);
 
 /* Eye space functions. */
 int is_eye_space(int pos);
@@ -756,44 +667,29 @@ void goaldump(const char goal[BOARDMAX]);
 void move_considered(int move, float value);
 
 
-/* SGF routines for debugging purposes in sgffile.c */
-void sgffile_begindump(struct SGFTree_t *tree);
-void sgffile_enddump(const char *filename);
-
+/* Transformation stuff. */
+#define MAX_OFFSET			(2*MAX_BOARD - 1) * (2*MAX_BOARD - 1)
+#define OFFSET(dx, dy)\
+  ((dy + MAX_BOARD - 1) * (2*MAX_BOARD - 1) + (dx + MAX_BOARD - 1))
+#define OFFSET_DELTA(dx, dy)		(OFFSET(dx, dy) - OFFSET(0, 0))
+#define CENTER_OFFSET(offset)		(offset - OFFSET(0, 0))
+#define TRANSFORM(offset, trans)	(transformation[offset][trans])
+#define AFFINE_TRANSFORM(offset, trans, delta)\
+  (transformation[offset][trans] + delta)
+#define TRANSFORM2(x, y, tx, ty, trans)\
+  do {\
+    *tx = transformation2[trans][0][0] * (x) + transformation2[trans][0][1] * (y);\
+    *ty = transformation2[trans][1][0] * (x) + transformation2[trans][1][1] * (y);\
+  } while (0)
 
 
 /* ================================================================ */
 /*                         global variables                         */
 /* ================================================================ */
 
-/* The board and the other parameters deciding the current position. */
-extern int          board_size;             /* board size (usually 19) */
-extern Intersection board[BOARDSIZE];       /* go board */
-extern int          board_ko_pos;
-extern int          black_captured;   /* num. of black stones captured */
-extern int          white_captured;
-
-extern Intersection initial_board[BOARDSIZE];
-extern int          initial_board_ko_pos;
-extern int          initial_white_captured;
-extern int          initial_black_captured;
-extern int          move_history_color[MAX_MOVE_HISTORY];
-extern int          move_history_pos[MAX_MOVE_HISTORY];
-extern int          move_history_pointer;
-
-extern float        komi;
-extern int          movenum;      /* movenumber - used for debug output */
-		    
-extern Intersection shadow[BOARDMAX];      /* reading tree shadow */
-
-		    
-extern int          disable_threat_computation;
-extern int          disable_endgame_patterns;
-extern int          doing_scoring;
-
-/* Transformation arrays */
-extern int	    transformation[MAX_OFFSET][8];
-extern const int    transformation2[8][2][2];
+extern int disable_threat_computation;
+extern int disable_endgame_patterns;
+extern int doing_scoring;
 
 /* Reading parameters */
 extern int depth;               /* deep reading cutoff */
@@ -805,6 +701,7 @@ extern int branch_depth;        /* deep reading cutoff */
 extern int fourlib_depth;       /* deep reading cutoff */
 extern int ko_depth;            /* deep ko reading cutoff */
 extern int aa_depth;            /* deep global reading cutoff */
+extern int depth_offset;        /* keeps track of temporary depth changes */
 extern int owl_distrust_depth;  /* below this owl trusts the optics code */
 extern int owl_branch_depth;    /* below this owl tries only one variation */
 extern int owl_reading_depth;   /* owl does not read below this depth */
@@ -824,24 +721,25 @@ extern int semeai_variations;   /* max variations considered reading semeai */
 extern float best_move_values[10];
 extern int best_moves[10];
 
-extern int chinese_rules;
 extern int experimental_owl_ext;     /* use experimental owl (GAIN/LOSS) */
 extern int experimental_semeai;      /* use experimental semeai module */
 extern int experimental_connections; /* use experimental connection module */
 extern int alternate_connections;    /* use alternate connection module */
 extern int owl_threats;              /* compute owl threats */
 extern int experimental_break_in;    /* use experimental module breakin.c */
+extern int cosmic_gnugo;             /* use center oriented influence */
+extern int large_scale;              /* seek large scale captures */
 
-extern int thrashing_dragon; /* Dead opponent's dragon trying to live */
-extern char thrashing_stone[BOARDMAX]; /* All thrashing stones. */
+extern int thrashing_dragon;        /* Dead opponent's dragon trying to live */
+extern char thrashing_stone[BOARDMAX];        /* All thrashing stones. */
 
 /* Experimental reading */
 extern char *rgoal;
 extern int goallib;
 
-extern int stackp;                /* stack pointer */
-extern int count_variations;      /* count (decidestring) */
-extern SGFTree *sgf_dumptree;
+extern int transformation[MAX_OFFSET][8];
+extern const int transformation2[8][2][2];
+
 
 /* Arrays pointing out the closest worms from each vertex.  The first
  * one is the closest worms of either color, the last two ones ignore
@@ -865,20 +763,9 @@ extern int number_close_white_worms[BOARDMAX];
 extern int false_eye_territory[BOARDMAX];
 extern int forced_backfilling_moves[BOARDMAX];
 
-struct stats_data {
-  int nodes;                     /* Number of visited nodes while reading */
-  int position_entered;          /* Number of Positions entered. */
-  int position_hits;             /* Number of hits of Positions. */
-  int read_result_entered;       /* Number of Read_results entered. */
-  int read_result_hits;          /* Number of hits of Read_results. */
-  int hash_collisions;           /* Number of hash collisions. */
-};
-
-extern struct stats_data stats;
-
 extern double slowest_time;      /* Timing statistics */
-extern int    slowest_move;
-extern int    slowest_movenum;
+extern int slowest_move;
+extern int slowest_movenum;
 extern double total_time;
 
 
@@ -930,7 +817,8 @@ struct worm_data {
   int genus;         /* number of connected components of the complement, less one */
   int inessential;   /* 1=inessential worm */
   int invincible;    /* 1=strongly unconditionally non-capturable */
-  int unconditional_status; /* ALIVE, DEAD, WHITE_BORDER, BLACK_BORDER, UNKNOWN */
+  enum dragon_status unconditional_status; /* ALIVE, DEAD, WHITE_TERRITORY,
+					      BLACK_TERRITORY, UNKNOWN */
 
   /* The following arrays keeps track of up to MAX_TACTICAL_POINTS
    * different attack, defense, attack threat, and defense threat
@@ -974,8 +862,8 @@ struct dragon_data {
                 /* dragon iff they have same origin.                         */
   int size;     /* size of the dragon                                        */
   float effective_size; /* stones and surrounding spaces                     */
-  int crude_status;     /* (ALIVE, DEAD, UNKNOWN, CRITICAL)                  */
-  int status;           /* best trusted status                               */
+  enum dragon_status crude_status; /* (ALIVE, DEAD, UNKNOWN, CRITICAL)       */
+  enum dragon_status status;       /* best trusted status                    */
 };
 
 extern struct dragon_data dragon[BOARDMAX];
@@ -994,7 +882,7 @@ struct dragon_data2 {
 
   int moyo_size;		      /* size of surrounding influence moyo, */
   float moyo_territorial_value;       /* ...and its territorial value        */
-  int safety;                         /* a more detailed status estimate     */
+  enum dragon_status safety;          /* a more detailed status estimate     */
   float weakness; /* A new (3.4) continuous estimate of the dragon's safety  */
   float weakness_pre_owl;     /* Dragon safety based on pre-owl computations */
   int escape_route; /* a measurement of likelihood of escape                 */
@@ -1005,14 +893,15 @@ struct dragon_data2 {
   int surround_status;         /* Is it surrounded?                          */
   int surround_size;           /* Size of the surrounding area               */
 
-  int semeai;              /* true if a dragon is part of a semeai           */
-  int semeai_margin_of_safety; /* if small, the semeai is close              */
+  int semeais;         /* number of semeais in which the dragon is involved  */
   int semeai_defense_point;/* Move found by semeai code to rescue dragon     */
   int semeai_defense_certain;
+  int semeai_defense_target; /* The opponent dragon involved in the semeai   */
   int semeai_attack_point; /* Move found by semeai code to kill dragon       */
   int semeai_attack_certain;
-  int owl_threat_status;   /* CAN_THREATEN_ATTACK or CAN_THREATEN_DEFENSE    */
-  int owl_status;          /* (ALIVE, DEAD, UNKNOWN, CRITICAL, UNCHECKED)    */
+  int semeai_attack_target; /* The opponent dragon involved in the semeai    */
+  enum dragon_status owl_threat_status; /* CAN_THREATEN_ATTACK/DEFENSE       */
+  enum dragon_status owl_status; /* (ALIVE, DEAD, UNKNOWN, CRITICAL, UNCHECKED)    */
   int owl_attack_point;    /* vital point for attack                         */
   int owl_attack_code;     /* ko result code                                 */
   int owl_attack_certain;  /* 0 if owl reading node limit is reached         */
@@ -1041,6 +930,19 @@ struct dragon_data2 * dragon2_func(int pos);
 
 #define DRAGON(d) dragon[dragon2[d].origin]
 
+
+/* Global variables to tune strategy. */
+
+extern int use_optimistic_territory;
+
+extern float minimum_value_weight;
+extern float maximum_value_weight;
+extern float invasion_malus_weight;
+extern float strategical_weight;
+extern float territorial_weight;
+extern float attack_dragon_weight;
+extern float followup_weight;
+
 struct aftermath_data {
   int white_captured;
   int black_captured;
@@ -1052,17 +954,19 @@ struct aftermath_data {
   int black_area;
   int white_control[BOARDMAX];
   int black_control[BOARDMAX];
-  int final_status[BOARDMAX];
+  enum dragon_status final_status[BOARDMAX];
 };
 
+#define MAX_EYE_ATTACKS 3
+
 struct eye_data {
   int color;/* BLACK, WHITE, BLACK_BORDERED, WHITE_BORDERED or GRAY_BORDERED */
   int esize;         /* size of the eyespace                                 */
   int msize;         /* number of marginal vertices                          */
   int origin;        /* The origin                                           */
   struct eyevalue value; /* Number of eyes.                                  */
-  int attack_point;  /* vital point for attack                               */
-  int defense_point; /* vital point for defense                              */
+  int attack_point;  /* vital point for attack */
+  int defense_point; /* vital point for defense	*/
 
   /* The above fields are constant on the whole eyespace. */
   /* ---------------------------------------------------------------- */
@@ -1072,12 +976,25 @@ struct eye_data {
   char type;                 /* Various characteristics of the eyespace    */
   char neighbors;            /* number of neighbors in eyespace            */
   char marginal_neighbors;   /* number of marginal neighbors               */
-  char cut;                  /* Opponent can cut at vertex.                */
 };
 
+struct vital_eye_points {
+  int attack_points[MAX_EYE_ATTACKS];
+  int defense_points[MAX_EYE_ATTACKS];
+};
+
+extern struct vital_eye_points black_vital_points[BOARDMAX];
+extern struct vital_eye_points white_vital_points[BOARDMAX];
+
 extern struct eye_data white_eye[BOARDMAX];
 extern struct eye_data black_eye[BOARDMAX];
 
+/* Array with the information which was previously stored in the cut
+ * field and in the INHIBIT_CONNECTION bit of the type field in struct
+ * eye_data.
+ */
+extern int cutting_points[BOARDMAX];
+
 /* The following declarations have to be postponed until after the
  * definition of struct eye_data or struct half_eye_data.
  */
@@ -1086,7 +1003,7 @@ void compute_eyes(int pos, struct eyevalue *value,
                   int *attack_point, int *defense_point,
                   struct eye_data eye[BOARDMAX],
                   struct half_eye_data heye[BOARDMAX],
-                  int add_moves, int color);
+		  int add_moves);
 void compute_eyes_pessimistic(int pos, struct eyevalue *value,
                               int *pessimistic_min,
                               int *attack_point, int *defense_point,
@@ -1106,7 +1023,7 @@ void make_domains(struct eye_data b_eye[BOARDMAX],
 void partition_eyespaces(struct eye_data eye[BOARDMAX], int color);
 void find_half_and_false_eyes(int color, struct eye_data eye[BOARDMAX],
 			      struct half_eye_data heye[BOARDMAX],
-			      char find_mask[BOARDMAX]);
+			      int find_mask[BOARDMAX]);
 
 void set_eyevalue(struct eyevalue *e, int a, int b, int c, int d);
 int min_eye_threat(struct eyevalue *e);
@@ -1121,23 +1038,6 @@ char *eyevalue_to_string(struct eyevalue *e);
 int is_halfeye(struct half_eye_data heye[BOARDMAX], int pos);
 int is_false_eye(struct half_eye_data heye[BOARDMAX], int pos);
 
-/* Our own abort() which prints board state on the way out.
- * (pos) is a "relevant" board position for info.
- */
-void abortgo(const char *file, int line, const char *msg, int pos);
-
-#if GG_TURN_OFF_ASSERTS
-#define ASSERT2(x, i, j)
-#define ASSERT1(x, pos)
-#else
-/* avoid dangling else */
-/* FIXME: Should probably re-write these using do {...} while (0) idiom. */
-#define ASSERT2(x, i, j) if (x) ; else abortgo(__FILE__, __LINE__, #x, POS(i, j))
-#define ASSERT1(x, pos) if (x) ; else abortgo(__FILE__, __LINE__, #x, pos)
-#endif
-
-#define gg_assert(x) ASSERT1(x, NO_MOVE);
-
 #endif  /* _LIBERTY_H_ */
 
 
