@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -206,7 +206,7 @@ static const int val_mask[2][8] = {
  * we care about each time.
  */
   
-static unsigned int class_mask[MAX_DRAGON_STATUS][3];
+static unsigned int class_mask[NUM_DRAGON_STATUS][3];
 
 
 /* In the current implementation, the edge constraints depend on
@@ -622,8 +622,8 @@ tree_matchpat_loop(matchpat_callback_fn_ptr callback,
 
   for (i = 0; i != board_size; i++)
     for (j = 0; j != board_size; j++)
-      if (BOARD(i,j) == anchor
-          && (!anchor_in_goal || goal[POS(i,j)] != 0))
+      if (BOARD(i, j) == anchor
+          && (!anchor_in_goal || goal[POS(i, j)] != 0))
 	tree_do_matchpat(i, j, callback, color, 
 			 pdb, callback_data, goal, anchor_in_goal);
 
@@ -656,13 +656,13 @@ do_tree_matchpat_rec(int color, int m, int n, int goal_found,
     struct tree_node *node = &(tnl->node);
     int x = m + node->x;
     int y = n + node->y;
-    if (ON_BOARD2(x,y)) {
+    if (ON_BOARD2(x, y)) {
       int att = node->att;
-      int point_color = BOARD(x,y);
+      int point_color = BOARD(x, y);
       if ((att == EMPTY && point_color == EMPTY)
           || (att == ATT_X && point_color == OTHER_COLOR(color))
           || (att == ATT_O && point_color == color)) {
-        goal_found = goal_found || goal[POS(x,y)];
+        goal_found = goal_found || goal[POS(x, y)];
         if (node->matches) {
           struct match_node *match = node->matches->next;
           while (match) {
@@ -775,14 +775,16 @@ tree_initialize_pointers(struct tree_node_list *tnl,
 /* Set this to show the dfa board in action */
 /* #define DFA_TRACE 1 */
 
-/* data */
-extern int board_size;
+/* Data. */
 static int dfa_board_size = -1;
-extern int dfa_p[DFA_MAX_BOARD * 4 * DFA_MAX_BOARD * 4];
-extern int spiral[MAX_ORDER][8];
+static int dfa_p[DFA_BASE * DFA_BASE];
 
 /* This is used by the EXPECTED_COLOR macro. */
-extern const int convert[3][4];
+static const int convert[3][4] = {
+  {-1, -1, -1, -1},		/* not used */
+  {EMPTY, WHITE, BLACK, OUT_BOARD},	/* WHITE */
+  {EMPTY, BLACK, WHITE, OUT_BOARD}	/* BLACK */
+};
 
 /* Forward declarations. */
 static void dfa_prepare_for_match(int color);
@@ -813,7 +815,7 @@ static void dfa_matchpat_loop(matchpat_callback_fn_ptr callback,
 void
 dfa_match_init(void)
 {
-  buildSpiralOrder(spiral);
+  build_spiral_order();
 
   if (owl_vital_apat_db.pdfa != NULL)
     DEBUG(DEBUG_MATCHER, "owl_vital_apat --> using dfa\n");
@@ -850,47 +852,48 @@ static void
 dfa_prepare_for_match(int color)
 {
   int i, j;
-  int ii;
+  int pos;
     
   if (dfa_board_size != board_size) {
     dfa_board_size = board_size;
     /* clean up the board */
-    for (ii = 0; ii < 4 * DFA_MAX_BOARD * 4 * DFA_MAX_BOARD; ii++)
-      dfa_p[ii] = OUT_BOARD;
+    for (pos = 0; pos < DFA_BASE * DFA_BASE; pos++)
+      dfa_p[pos] = OUT_BOARD;
   }
 
   /* copy the board */
   for (i = 0; i < dfa_board_size; i++)
     for (j = 0; j < dfa_board_size; j++)
-      dfa_p[DFA_POS(i, j) + DFA_OFFSET] = EXPECTED_COLOR(color, BOARD(i, j));
+      dfa_p[DFA_POS(i, j)] = EXPECTED_COLOR(color, BOARD(i, j));
 
   prepare_for_match(color);
 }
 
 #if 0
-/* debug function */
+/* Debug function. */
 static void
 dump_dfa_board(int m, int n)
 {
   int i, j;
 
-  for (i = DFA_MAX_BOARD / 2; i < DFA_MAX_BOARD*1.5 ; i++) {
-    for (j = DFA_MAX_BOARD / 2 ; j < DFA_MAX_BOARD*1.5 ; j++)
-      if (i != (m+DFA_MAX_BOARD) || j != (n+DFA_MAX_BOARD))
+  for (i = 0; i < dfa_board_size; i++) {
+    for (j = 0; j < dfa_board_size; j++) {
+      if (i != m || j != n)
 	fprintf(stderr, "%1d", dfa_p[DFA_POS(i, j)]);
       else
 	fprintf(stderr, "*");
+    }
+
     fprintf(stderr, "\n");
   }
 }
 #endif
 
 
-/* 
- * Scan the board with a dfa to get 
- * all patterns matching at (m, n) with transformation l.
- * Store patterns indexes + transformation in pat_list.
- * Return the number of patterns found.
+/*
+ * Scan the board with a DFA to get all patterns matching at
+ * `dfa_pos' with transformation l.  Store patterns indexes
+ * `pat_list'.  Return the number of patterns found.
  */
 static int
 scan_for_patterns(dfa_rt_t *pdfa, int l, int *dfa_pos, int *pat_list)
@@ -898,8 +901,8 @@ scan_for_patterns(dfa_rt_t *pdfa, int l, int *dfa_pos, int *pat_list)
   int delta;
   int state = 1; /* initial state */
   int row = 0; /* initial row */
-  int id = 0; /* position in id_list */ 
-  
+  int id = 0; /* position in id_list */
+
   do {
     /* collect patterns indexes */
     int att = pdfa->states[state].att;
@@ -908,63 +911,61 @@ scan_for_patterns(dfa_rt_t *pdfa, int l, int *dfa_pos, int *pat_list)
       id++;
       att = pdfa->indexes[att].next;
     }
-      
+
     /* go to next state */
     delta = pdfa->states[state].next[dfa_pos[spiral[row][l]]];
     state += delta;
     row++;
   } while (delta != 0); /* while not on error state */
 
-  gg_assert(row < MAX_ORDER);
+  gg_assert(row < DFA_MAX_ORDER);
   return id;
 }
 
 
-/* Perform pattern matching with dfa filtering. */
-static void 
+/* Perform pattern matching with DFA filtering. */
+static void
 do_dfa_matchpat(dfa_rt_t *pdfa,
 		int anchor, matchpat_callback_fn_ptr callback,
 		int color, struct pattern *database,
 		void *callback_data, char goal[BOARDMAX],
-                int anchor_in_goal)
+		int anchor_in_goal)
 {
-  int k = 0;
+  int k;
   int ll;      /* Iterate over transformations (rotations or reflections)  */
-  int patterns[DFA_MAX_MATCHED];
-  int *ll_patterns = patterns;
+  int patterns[DFA_MAX_MATCHED + 8];
   int num_matched = 0;
-  int num_transformations = (pdfa->pre_rotated ? 1 : 8);
-  int transformation_end[8];
-  int *dfa_pos = dfa_p + DFA_POS(I(anchor), J(anchor)) + DFA_OFFSET;
+  int *dfa_pos = dfa_p + DFA_POS(I(anchor), J(anchor));
 
   /* Basic sanity checks. */
   ASSERT_ON_BOARD1(anchor);
 
   /* One scan by transformation */
-  for (ll = 0; ll < num_transformations; ll++) {
-    int ll_matched = scan_for_patterns(pdfa, ll, dfa_pos,
-				       patterns + num_matched);
-    
-    ll_patterns += ll_matched;
-    num_matched += ll_matched;
-    transformation_end[ll] = num_matched;
+  for (ll = 0; ll < 8; ll++) {
+    num_matched += scan_for_patterns(pdfa, ll, dfa_pos,
+				     patterns + num_matched);
+    patterns[num_matched++] = -1;
   }
 
-  ASSERT1(num_matched <= DFA_MAX_MATCHED, anchor);
+  ASSERT1(num_matched <= DFA_MAX_MATCHED + 8, anchor);
 
   /* Constraints and other tests. */
-  for (ll = 0; ll < num_transformations; ll++) {
-    while (k < transformation_end[ll]) {
-      int matched = patterns[k];
+  for (ll = 0, k = 0; ll < 8; k++) {
+    int matched;
+
+    if (patterns[k] == -1) {
+      ll++;
+      continue;
+    }
+
+    matched = patterns[k];
 
 #if PROFILE_PATTERNS
-      database[matched].dfa_hits++;
+    database[matched].dfa_hits++;
 #endif
-    
-      check_pattern_light(anchor, callback, color, database + matched,
-			  ll, callback_data, goal, anchor_in_goal);
-      k++;
-    }
+
+    check_pattern_light(anchor, callback, color, database + matched,
+			ll, callback_data, goal, anchor_in_goal);
   }
 }
 
