@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -63,6 +63,9 @@ int next_lunch;
 /* Point redistribution */
 int replacement_map[BOARDMAX];
 
+/* The color for which we are evaluating moves. */
+int current_color;
+
 /* Attack threats that are known to be sente locally. */
 static int known_good_attack_threats[BOARDMAX][MAX_ATTACK_THREATS];
 
@@ -139,7 +142,7 @@ clear_move_reasons(void)
  * Find the index of a connection in the list of connections.
  * If necessary, add a new entry.
  */
-static int
+int
 find_connection(int worm1, int worm2)
 {
   int k;
@@ -240,27 +243,6 @@ find_pair_data(int what1, int what2)
   return next_either - 1;
 }
 
-/*
- * Find the index of an eye space in the list of eye spaces.
- * If necessary, add a new entry.
- */
-static int
-find_eye(int eye, int color)
-{
-  int k;
-  ASSERT_ON_BOARD1(eye);
-  
-  for (k = 0; k < next_eye; k++)
-    if (eyes[k] == eye && eyecolor[k] == color)
-      return k;
-  
-  /* Add a new entry. */
-  gg_assert(next_eye < MAX_EYES);
-  eyes[next_eye] = eye;
-  eyecolor[next_eye] = color;
-  next_eye++;
-  return next_eye - 1;
-}
 
 /* Interprets the object of a reason and returns its position.
  * If the object is a pair (of worms or dragons), the position of the first
@@ -285,7 +267,6 @@ get_pos(int reason, int what)
 
   case SEMEAI_MOVE:
   case SEMEAI_THREAT:
-  case VITAL_EYE_MOVE:
   case STRATEGIC_ATTACK_MOVE:
   case STRATEGIC_DEFEND_MOVE:
   case OWL_ATTACK_MOVE:
@@ -571,6 +552,12 @@ owl_move_vs_worm_known(int pos, int what)
   return owl_move_reason_known(pos, dragon[what].origin);
 }
 
+int
+semeai_move_reason_known(int pos, int what)
+{
+  return move_reason_known(pos, SEMEAI_MOVE, what);
+}
+
 /* Check whether a worm is inessential */
 static int
 concerns_inessential_worm(int pos, int what)
@@ -592,7 +579,8 @@ static int
 move_is_marked_unsafe(int pos, int what)
 {
   UNUSED(what);
-  return !move[pos].move_safety;
+  return (!move[pos].move_safety
+	  && !adjacent_to_nondead_stone(pos, current_color));
 }
 
 /* Check whether a dragon is non-critical. */
@@ -863,7 +851,6 @@ add_antisuji_move(int pos)
 void
 add_semeai_move(int pos, int dr)
 {
-  
   ASSERT_ON_BOARD1(dr);
   add_move_reason(pos, SEMEAI_MOVE, dragon[dr].origin);
 }
@@ -882,23 +869,6 @@ add_semeai_threat(int pos, int dr)
   add_move_reason(pos, SEMEAI_THREAT, dragon[dr].origin);
 }
 
-/*
- * Add to the reasons for the move at (pos) that it's the vital
- * point for the eye space at (eyespace) of color.
- */
-void
-add_vital_eye_move(int pos, int eyespace, int color)
-{
-  int eye;
-
-  ASSERT_ON_BOARD1(eyespace);
-  if (color == WHITE)
-    eye = find_eye(white_eye[eyespace].origin, color);
-  else
-    eye = find_eye(black_eye[eyespace].origin, color);
-  add_move_reason(pos, VITAL_EYE_MOVE, eye);
-}
-
 /*
  * Add to the reasons for the move at (pos) that it will accomplish
  * one of two things: either (reason1) on (target1) or (reason2) on 
@@ -1025,16 +995,6 @@ add_all_move(int pos, int reason1, int target1, int reason2, int target2)
 }
 
 
-void
-add_gain_move(int pos, int target1, int target2)
-{
-  int what1 = dragon[target1].origin;
-  int what2 = worm[target2].origin;
-  int index = find_pair_data(what1, what2);
-  ASSERT1(target2 != NO_MOVE, pos);
-  add_move_reason(pos, OWL_ATTACK_MOVE_GAIN, index);
-}
-
 void
 add_loss_move(int pos, int target1, int target2)
 {
@@ -1138,7 +1098,7 @@ add_strategical_defense_move(int pos, int dr)
  * code reports an attack on the dragon (dr).
  */
 void
-add_owl_attack_move(int pos, int dr, int code)
+add_owl_attack_move(int pos, int dr, int kworm, int code)
 {
   dr = dragon[dr].origin;
 
@@ -1149,6 +1109,10 @@ add_owl_attack_move(int pos, int dr, int code)
     add_move_reason(pos, OWL_ATTACK_MOVE_GOOD_KO, dr);
   else if (code == KO_B)
     add_move_reason(pos, OWL_ATTACK_MOVE_BAD_KO, dr);
+  else if (code == GAIN) {
+    ASSERT_ON_BOARD1(kworm);
+    add_move_reason(pos, OWL_ATTACK_MOVE_GAIN, find_pair_data(dr, kworm));
+  }
 }
 
 /*
@@ -1514,13 +1478,12 @@ mark_changed_dragon(int pos, int color, int affected, int affected2,
 	if (worm[ii].attack_codes[0] == NO_MOVE
 	    || defense_move_reason_known(pos, ii))
 	  worm_is_safe = 1;
-	else if (trymove(pos, color, "mark-changed-dragon", ii,
-			 EMPTY, NO_MOVE)) {
+	else if (trymove(pos, color, "mark-changed-dragon", ii)) {
 	    if (REVERSE_RESULT(attack(ii, NULL)) >= result_to_beat)
 	      worm_is_safe = 1;
 	    popgo();
 	}
-	if (worm_is_safe) {
+	if (worm_is_safe || move_reason_type == SEMEAI_MOVE) {
 	  /* This string can now be considered safe. Hence we mark the
 	   * whole string as such:
 	   */
@@ -1629,9 +1592,11 @@ mark_safe_stones(int color, int move_pos, const char saved_dragons[BOARDMAX],
 }
 
 
-/* List the move reasons for (color). */
-void
-list_move_reasons(int color)
+/* List the move reasons for (color)'s move at (pos). Return the
+ * number of move reasons.
+ */
+int
+list_move_reasons(FILE *out, int move_pos)
 {
   int m;
   int n;
@@ -1643,48 +1608,53 @@ list_move_reasons(int color)
   int bb = NO_MOVE;
   int worm1 = -1;
   int worm2 = -1;
-  int ecolor = 0;
-  
+  int num_move_reasons = 0;
+
   gprintf("\nMove reasons:\n");
   
   for (n = 0; n < board_size; n++)
     for (m = board_size-1; m >= 0; m--) {
       pos = POS(m, n);
 
+      if (move_pos != NO_MOVE && move_pos != pos)
+	continue;
+      
       for (k = 0; k < MAX_REASONS; k++) {
 	int r = move[pos].reason[k];
 
 	if (r < 0)
 	  break;
-	
+
+	num_move_reasons++;
+
 	switch (move_reasons[r].type) {
 	case ATTACK_MOVE:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m attacks %1m%s\n", pos, aa,
-		  (worm[aa].defense_codes[0] == 0) ? " (defenseless)" : "");
+	  gfprintf(out, "Move at %1m attacks %1m%s\n", pos, aa,
+		   (worm[aa].defense_codes[0] == 0) ? " (defenseless)" : "");
 	  break;
 	case ATTACK_MOVE_GOOD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m attacks %1m%s with good ko\n", pos, aa,
-		  (worm[aa].defense_codes[0] == 0) ? " (defenseless)" : "");
+	  gfprintf(out, "Move at %1m attacks %1m%s with good ko\n", pos, aa,
+		   (worm[aa].defense_codes[0] == 0) ? " (defenseless)" : "");
 	  break;
 	case ATTACK_MOVE_BAD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m attacks %1m%s with bad ko\n", pos, aa,
-		  (worm[aa].defense_codes[0] == 0) ? " (defenseless)" : "");
+	  gfprintf(out, "Move at %1m attacks %1m%s with bad ko\n", pos, aa,
+		   (worm[aa].defense_codes[0] == 0) ? " (defenseless)" : "");
 	  break;
 	  
 	case DEFEND_MOVE:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m defends %1m\n", pos, aa);
+	  gfprintf(out, "Move at %1m defends %1m\n", pos, aa);
 	  break;
 	case DEFEND_MOVE_GOOD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m defends %1m with good ko\n", pos, aa);
+	  gfprintf(out, "Move at %1m defends %1m with good ko\n", pos, aa);
 	  break;
 	case DEFEND_MOVE_BAD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m defends %1m with bad ko\n", pos, aa);
+	  gfprintf(out, "Move at %1m defends %1m with bad ko\n", pos, aa);
 	  break;
 	  
 	case ATTACK_THREAT:
@@ -1692,19 +1662,19 @@ list_move_reasons(int color)
 	  aa = move_reasons[r].what;
 	  
 	  if (move_reasons[r].type == ATTACK_THREAT)
-	    gprintf("Move at %1m threatens to attack %1m\n", pos, aa);
+	    gfprintf(out, "Move at %1m threatens to attack %1m\n", pos, aa);
 	  else if (move_reasons[r].type == DEFEND_THREAT)
-	    gprintf("Move at %1m threatens to defend %1m\n", pos, aa);
+	    gfprintf(out, "Move at %1m threatens to defend %1m\n", pos, aa);
 	  break;
 
 	case UNCERTAIN_OWL_DEFENSE:
 	  aa = move_reasons[r].what;
-	  if (board[aa] == color)
-	    gprintf("%1m found alive but not certainly, %1m defends it again\n",
-		    aa, pos);
+	  if (board[aa] == current_color)
+	    gfprintf(out, "%1m found alive but not certainly, %1m defends it again\n",
+		     aa, pos);
 	  else
-	    gprintf("%1m found dead but not certainly, %1m attacks it again\n",
-		    aa, pos);
+	    gfprintf(out, "%1m found dead but not certainly, %1m attacks it again\n",
+		     aa, pos);
 	  break;	  
 
 	case CONNECT_MOVE:
@@ -1712,32 +1682,25 @@ list_move_reasons(int color)
 	  worm1 = conn_worm1[move_reasons[r].what];
 	  worm2 = conn_worm2[move_reasons[r].what];
 	  if (move_reasons[r].type == CONNECT_MOVE)
-	    gprintf("Move at %1m connects %1m and %1m\n", pos, worm1, worm2);
+	    gfprintf(out, "Move at %1m connects %1m and %1m\n",
+		     pos, worm1, worm2);
 	  else
-	    gprintf("Move at %1m cuts %1m and %1m\n", pos, worm1, worm2);
+	    gfprintf(out, "Move at %1m cuts %1m and %1m\n", pos, worm1, worm2);
 	  break;
 	  
 	case ANTISUJI_MOVE:
-	  gprintf("Move at %1m is an antisuji\n", pos);
+	  gfprintf(out, "Move at %1m is an antisuji\n", pos);
 	  break;
 	  
 	case SEMEAI_MOVE:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m wins semeai for %1m\n", pos, aa);
+	  gfprintf(out, "Move at %1m wins semeai for %1m\n", pos, aa);
 	  break;
 	  
 	case SEMEAI_THREAT:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m threatens to win semeai for %1m\n", pos, aa);
-	  break;
-	  
-	case VITAL_EYE_MOVE:
-	  aa = eyes[move_reasons[r].what];
-	  ecolor = eyecolor[move_reasons[r].what];
-	  if (ecolor == WHITE)
-	    gprintf("Move at %1m vital eye point for eye %1m\n", pos, aa);
-	  else
-	    gprintf("Move at %1m vital eye point for eye %1m\n", pos, aa);
+	  gfprintf(out, "Move at %1m threatens to win semeai for %1m\n",
+		   pos, aa);
 	  break;
 	  
 	case EITHER_MOVE:
@@ -1745,9 +1708,9 @@ list_move_reasons(int color)
 	  reason2 = either_data[move_reasons[r].what].reason2;
 	  worm1 = either_data[move_reasons[r].what].what1;
 	  worm2 = either_data[move_reasons[r].what].what2;
-	  gprintf("Move at %1m either %s %1m or %s %1m\n", pos, 
-		  reason1 == ATTACK_STRING ? "attacks" : "defends", worm1, 
-		  reason2 == ATTACK_STRING ? "attacks" : "defends", worm2);
+	  gfprintf(out, "Move at %1m either %s %1m or %s %1m\n", pos, 
+		   reason1 == ATTACK_STRING ? "attacks" : "defends", worm1, 
+		   reason2 == ATTACK_STRING ? "attacks" : "defends", worm2);
 	  break;
 
 	case ALL_MOVE:
@@ -1755,73 +1718,75 @@ list_move_reasons(int color)
 	  reason2 = all_data[move_reasons[r].what].reason2;
 	  worm1 = all_data[move_reasons[r].what].what1;
 	  worm2 = all_data[move_reasons[r].what].what2;
-	  gprintf("Move at %1m both %s %1m and %s %1m\n", pos, 
-		  reason1 == ATTACK_STRING ? "attacks" : "defends", worm1, 
-		  reason2 == ATTACK_STRING ? "attacks" : "defends", worm2);
+	  gfprintf(out, "Move at %1m both %s %1m and %s %1m\n", pos, 
+		   reason1 == ATTACK_STRING ? "attacks" : "defends", worm1, 
+		   reason2 == ATTACK_STRING ? "attacks" : "defends", worm2);
 	  break;
 
 	case OWL_ATTACK_MOVE:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-attacks %1m\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-attacks %1m\n", pos, aa);
 	  break;
 	case OWL_ATTACK_MOVE_GOOD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-attacks %1m with good ko\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-attacks %1m with good ko\n", pos, aa);
 	  break;
 	case OWL_ATTACK_MOVE_BAD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-attacks %1m with bad ko\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-attacks %1m with bad ko\n", pos, aa);
 	  break;
 	case OWL_ATTACK_MOVE_GAIN:
 	  aa = either_data[move_reasons[r].what].what1;
 	  bb = either_data[move_reasons[r].what].what2;
-	  gprintf("Move at %1m owl-attacks %1m (captures %1m)\n", pos, aa, bb);
+	  gfprintf(out, "Move at %1m owl-attacks %1m (captures %1m)\n",
+		   pos, aa, bb);
 	  break;
 	  
 	case OWL_DEFEND_MOVE:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-defends %1m\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-defends %1m\n", pos, aa);
 	  break;
 	case OWL_DEFEND_MOVE_GOOD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-defends %1m with good ko\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-defends %1m with good ko\n", pos, aa);
 	  break;
 	case OWL_DEFEND_MOVE_BAD_KO:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-defends %1m with bad ko\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-defends %1m with bad ko\n", pos, aa);
 	  break;
 	case OWL_DEFEND_MOVE_LOSS:
 	  aa = either_data[move_reasons[r].what].what1;
 	  bb = either_data[move_reasons[r].what].what2;
-	  gprintf("Move at %1m owl-defends %1m (loses %1m)\n", pos, aa, bb);
+	  gfprintf(out, "Move at %1m owl-defends %1m (loses %1m)\n",
+		   pos, aa, bb);
 	  break;
 	  
 	case OWL_ATTACK_THREAT:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-threatens to attack %1m\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-threatens to attack %1m\n", pos, aa);
 	  break;
 	  
 	case OWL_DEFEND_THREAT:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-threatens to defend %1m\n", pos, aa);
+	  gfprintf(out, "Move at %1m owl-threatens to defend %1m\n", pos, aa);
 	  break;
 	  
 	case OWL_PREVENT_THREAT:
 	  aa = move_reasons[r].what;
-	  gprintf("Move at %1m owl-prevents a threat to attack or defend %1m\n", 
-		  pos, aa);
+	  gfprintf(out, "Move at %1m owl-prevents a threat to attack or defend %1m\n", 
+		   pos, aa);
 	  break;
 
 	case EXPAND_TERRITORY_MOVE:
-	  gprintf("Move at %1m expands territory\n", pos);
+	  gfprintf(out, "Move at %1m expands territory\n", pos);
 	  break;
 	  
 	case EXPAND_MOYO_MOVE:
-	  gprintf("Move at %1m expands moyo\n", pos);
+	  gfprintf(out, "Move at %1m expands moyo\n", pos);
 	  break;
 	  
 	case INVASION_MOVE:
-	  gprintf("Move at %1m is an invasion\n", pos);
+	  gfprintf(out, "Move at %1m is an invasion\n", pos);
 	  break;
 	  
 	case STRATEGIC_ATTACK_MOVE:
@@ -1829,21 +1794,24 @@ list_move_reasons(int color)
 	  aa = move_reasons[r].what;
 	  
 	  if (move_reasons[r].type == STRATEGIC_ATTACK_MOVE)
-	    gprintf("Move at %1m strategically attacks %1m\n", pos, aa);
+	    gfprintf(out, "Move at %1m strategically attacks %1m\n", pos, aa);
 	  else
-	    gprintf("Move at %1m strategically defends %1m\n", pos, aa);
+	    gfprintf(out, "Move at %1m strategically defends %1m\n", pos, aa);
 	  break;
 	  
 	case MY_ATARI_ATARI_MOVE:
-	  gprintf("Move at %1m captures something\n", pos);
+	  gfprintf(out, "Move at %1m captures something\n", pos);
 
 	case YOUR_ATARI_ATARI_MOVE:
-	  gprintf("Move at %1m defends against combination attack\n", pos);
+	  gfprintf(out, "Move at %1m defends against combination attack\n",
+		   pos);
 	}
       }
       if (k > 0 && move[pos].move_safety == 0)
-	gprintf("Move at %1m strategically or tactically unsafe\n", pos);
+	gfprintf(out, "Move at %1m strategically or tactically unsafe\n", pos);
     }
+  
+  return num_move_reasons;
 }
 
 
@@ -1870,10 +1838,10 @@ static struct discard_rule discard_rules[] =
     "  %1m: 0.0 - (threat of) attack/defense of %1m (owl attack/defense as well)\n" },
   { { SEMEAI_MOVE, SEMEAI_THREAT, -1 },
     owl_move_reason_known, REDUNDANT,
-    "  %1m: 0.0 - (threat to) win semai involving %1m (owl move as well)\n"},
+    "  %1m: 0.0 - (threat to) win semeai involving %1m (owl move as well)\n"},
   { { SEMEAI_MOVE, SEMEAI_THREAT, -1 },
     tactical_move_vs_whole_dragon_known, REDUNDANT,
-    "  %1m: 0.0 - (threat to) win semai involving %1m (tactical move as well)\n"},
+    "  %1m: 0.0 - (threat to) win semeai involving %1m (tactical move as well)\n"},
   { { EITHER_MOVE, -1 },
     either_worm_attackable, REDUNDANT,
     "  %1m: 0.0 - 'attack either' is redundant at %1m (direct att./def. as well)\n"},
