@@ -1,8 +1,9 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -19,7 +20,9 @@
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
-#include "gnugo.h"
+#include "board.h"
+#include "hash.h"
+#include "gg_utils.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -27,11 +30,6 @@
 #include <stdarg.h>
 #include <ctype.h>
 
-#include "liberty.h"
-#include "gnugo.h"
-#include "gg_utils.h"
-#include "cache.h"
-
 /*
  * This function underpins all the TRACE and DEBUG stuff.
  * It accepts %c, %d, %f, %s, and %x as usual. But it
@@ -39,19 +37,18 @@
  * Other accepted formats are
  * %H: Print a hashvalue.
  * %C: Print a color as a string.
- * %M: Like %m but three characters wide for all moves (pads with spaces).
  * Nasty bodge: %o at the start means outdent, i.e. cancel indent.
  */
 
-static void 
-vgprintf(FILE* outputfile, const char *fmt, va_list ap)
+void 
+vgprintf(FILE *outputfile, const char *fmt, va_list ap)
 {
   if (fmt[0] == '%' && fmt[1] == 'o')
     fmt += 2;  /* cancel indent */
   else if (stackp > 0)
     fprintf(outputfile, "%.*s", stackp*2, "                                ");
 
-  for (; *fmt ; ++fmt) {
+  for (; *fmt; ++fmt) {
     if (*fmt == '%') {
       switch (*++fmt) {
       case 'c':
@@ -69,7 +66,7 @@ vgprintf(FILE* outputfile, const char *fmt, va_list ap)
       }
       case 'x':
       {
-	int d = va_arg(ap, int);
+	unsigned int d = va_arg(ap, unsigned int);
 	fprintf(outputfile, "%x", d);
 	break;
       }
@@ -81,10 +78,17 @@ vgprintf(FILE* outputfile, const char *fmt, va_list ap)
       }
       case 's':
       {
-	char *s = va_arg(ap, char*);
+	char *s = va_arg(ap, char *);
 	fputs(s, outputfile);
 	break;
       }
+      case '2':
+        fmt++;
+        if (*fmt != 'm' && *fmt != 'M') {
+	  fprintf(outputfile, "\n\nUnknown format string '2%c'\n", *fmt);
+	  break;
+        }
+        /* else fall through - 2 modifier on %m is default. */
       case 'm':
       case 'M':
       {
@@ -140,7 +144,7 @@ vgprintf(FILE* outputfile, const char *fmt, va_list ap)
       }
       case 'H':
       {
-	long h = va_arg(ap, int);
+	unsigned long h = va_arg(ap, unsigned long);
 	fprintf(outputfile, "%lx", h);
 	break;
       }
@@ -205,63 +209,83 @@ mprintf(const char *fmt, ...)
   va_end(ap);
 }
 
-#ifndef HAVE_VARIADIC_DEFINE
-
-/* See gnugo.h for related TRACE family macro definitions */
-
-int trace_dummy;
-
-/* Always returns 1 to allow use in short-circuit logical expressions. */
-int 
-DEBUG_func(int flag, const char *fmt, ...)
+/* This writes the move history information in sgf format to stderr.
+ * This is only intended as a stand-alone debug tool for use in
+ * abortgo(). Anywhere else you should use the normal sgf library.
+ */
+static void
+dump_board_sgf(void)
 {
-  va_list ap;
+  int pos;
+  int initial_colors_found = EMPTY;
+  int color;
+  int k;
 
-  if (debug & flag) {
-    va_start(ap, fmt);
-    vgprintf(stderr, fmt, ap);
-    va_end(ap);
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++)
+    if (ON_BOARD(pos))
+      initial_colors_found |= initial_board[pos];
+  
+  fprintf(stderr, "(;GM[1]FF[4]SZ[%d]KM[%.1f]GN[GNU Go %s stepped on a bug]\n",
+	  board_size, komi, gg_version());
+
+  for (color = WHITE; color <= BLACK; color++) {
+    if (initial_colors_found & color) {
+      fprintf(stderr, "A%s", color == WHITE ? "W" : "B");
+      for (k = 0, pos = BOARDMIN; pos < BOARDMAX; pos++) {
+	if (ON_BOARD(pos) && initial_board[pos] == color) {
+	  fprintf(stderr, "[%c%c]", 'a' + J(pos), 'a' + I(pos));
+	  k++;
+	  if (k % 16 == 0)
+	    fprintf(stderr, "\n");
+	}
+      }
+      if (k % 16 != 0)
+	fprintf(stderr, "\n");
+    }
   }
 
-  return 1;
+  if (move_history_pointer > 0) {
+    for (k = 0; k < move_history_pointer; k++) {
+      fprintf(stderr, ";%s", move_history_color[k] == WHITE ? "W" : "B");
+      if (move_history_pos[k] == PASS_MOVE)
+	fprintf(stderr, "[]");
+      else
+	fprintf(stderr, "[%c%c]", 'a' + J(move_history_pos[k]),
+		'a' + I(move_history_pos[k]));
+      
+      if (k % 12 == 11)
+	fprintf(stderr, "\n");
+    }
+    if (k % 12 != 0)
+      fprintf(stderr, "\n");
+  }
+  fprintf(stderr, ")\n");
 }
 
-#endif /*HAVE_VARIADIC_DEFINE*/
-
-
 /*
  * A wrapper around abort() which shows the state variables at the time
- * of the problem.  (i, j) are typically a related move, or -1, -1.
+ * of the problem. (pos) is typically a related move, or NO_MOVE.
  */
 
 void 
-abortgo(const char *file, int line, const char *msg, int x, int y)
+abortgo(const char *file, int line, const char *msg, int pos)
 {
-  verbose = 4;
-  TRACE("%o\n\n***assertion failure:\n%s:%d - %s near %m***\n\n",
-	file, line, msg, x, y);
+  gprintf("%o\n\n***assertion failure:\n%s:%d - %s near %1m***\n\n",
+	  file, line, msg, pos);
   dump_stack();
 
-  /* Dump the stack as board images. */
-  showboard(0);
-  while (stackp > 0) {
-    popgo();
-    showboard(0);
-  }
+  /* Print the board at the top of the stack. */
+  simple_showboard(stderr);
+  fprintf(stderr, "\n");
 
-#if 0
-  if (sgf_root) {
-    sgf_write_header(sgf_root, 1, random_seed, komi, level);
-    writesgf(sgf_root, "abortgo.sgf");
-  }
-#endif
+  dump_board_sgf();
 
-  fprintf(stderr, "\ngnugo %s (seed %d): You stepped on a bug.\n",
-          gg_version(), random_seed);
+  fprintf(stderr, "gnugo %s (seed %d): You stepped on a bug.\n",
+          gg_version(), get_random_seed());
   if (board_size >= 9 && board_size <= 19) {
     fprintf(stderr, "\
-Please save this game as an sgf file and mail it to gnugo@gnu.org\n\
-If you can, please also include the debug output above this message.\n");
+Please mail this message, including the debug output above, \
+to gnugo@gnu.org\n");
   }
   fprintf(stderr, "\n");
 
@@ -271,25 +295,16 @@ If you can, please also include the debug output above this message.\n");
   abort();  /* cause core dump */
 }
 
+static const char *color_names[] = {
+  COLOR_NAMES
+};
 
 /* Convert a color value to a string. */
 const char *
 color_to_string(int color)
 {
-  if (color == EMPTY)
-    return "empty";
-  else if (color == WHITE)
-    return "white";
-  else if (color == BLACK)
-    return "black";
-  else if (color == GRAY)
-    return "gray";
-  else if (color == WHITE_BORDER)
-    return "white border";
-  else if (color == BLACK_BORDER)
-    return "black border";
-  else
-    return "purple?";
+  gg_assert(color < NUM_KOMASTER_STATES);
+  return color_names[color];
 }
 
 /* Convert a location to a string. */
@@ -335,120 +350,6 @@ location_to_buffer(int pos, char *buf)
   *bufp = 0;
 }
 
-/* Convert a status value to a string. */
-const char *
-status_to_string(int status)
-{
-  if (status == DEAD)
-    return "DEAD";
-  else if (status == ALIVE)
-    return "ALIVE";
-  else if (status == CRITICAL)
-    return "CRITICAL";
-  else if (status == UNKNOWN)
-    return "UNKNOWN";
-  else if (status == UNCHECKED)
-    return "UNCHECKED";
-  else if (status == CAN_THREATEN_ATTACK)
-    return "CAN_THREATEN_ATTACK";
-  else if (status == CAN_THREATEN_DEFENSE)
-    return "CAN_THREATEN_DEFENSE";
-  else
-    return "ERROR";
-}
-
-
-/* Convert a safety value to a string. */
-const char *
-safety_to_string(int status)
-{
-  if (status == DEAD)
-    return "DEAD";
-  else if (status == ALIVE)
-    return "ALIVE";
-  else if (status == CRITICAL)
-    return "CRITICAL";
-  else if (status == INESSENTIAL)
-    return "INESSENTIAL";
-  else if (status == TACTICALLY_DEAD)
-    return "TACTICALLY_DEAD";
-  else if (status == INESSENTIAL)
-    return "INESSENTIAL";
-  else if (status == TACTICALLY_DEAD)
-    return "TACTICALLY_DEAD";
-  else if (status == WEAK)
-    return "WEAK";
-  else if (status == WEAKLY_ALIVE)
-    return "WEAKLY_ALIVE";
-  else if (status == ALIVE_IN_SEKI)
-    return "ALIVE_IN_SEKI";
-  else if (status == STRONGLY_ALIVE)
-    return "STRONGLY_ALIVE";
-  else if (status == INVINCIBLE)
-    return "INVINCIBLE";
-  else if (status == INSUBSTANTIAL)
-    return "INSUBSTANTIAL";
-  else if (status == CAN_THREATEN_ATTACK)
-    return "CAN_THREATEN_ATTACK";
-  else if (status == CAN_THREATEN_DEFENSE)
-    return "CAN_THREATEN_DEFENSE";
-  else
-    return "ERROR";
-}
-
-/* Convert a routine to a string. */
-const char *
-routine_to_string(int routine)
-{
-  if (routine == FIND_DEFENSE)
-    return "FIND_DEFENSE";
-  else if (routine == DEFEND1)
-    return "DEFEND1";
-  else if (routine == DEFEND2)
-    return "DEFEND2";
-  else if (routine == DEFEND3)
-    return "DEFEND3";
-  else if (routine == DEFEND4)
-    return "DEFEND4";
-  else if (routine == ATTACK)
-    return "ATTACK";
-  else if (routine == ATTACK2)
-    return "ATTACK2";
-  else if (routine == ATTACK3)
-    return "ATTACK3";
-  else if (routine == OWL_ATTACK)
-    return "OWL_ATTACK";
-  else if (routine == OWL_DEFEND)
-    return "OWL_DEFEND";
-  else if (routine == SEMEAI)
-    return "SEMEAI";
-  else if (routine == CONNECT)
-    return "CONNECT";
-  else if (routine == DISCONNECT)
-    return "DISCONNECT";
-  else
-    return "ERROR";
-}
-
-/* Convert a read result to a string */
-const char *
-result_to_string(int result)
-{
-  if (result == 0)
-    return "0";
-  else if (result == KO_A)
-    return "KO_A";
-  else if (result == KO_B)
-    return "KO_B";
-  else if (result == WIN)
-    return "WIN";
-  else if (result == ALIVE_IN_SEKI)
-    return "SEKI";
-  else
-    return "ERROR";
-}
-
-
 
 /*
  * Get the (m, n) coordinates in the standard GNU Go coordinate system
@@ -483,6 +384,132 @@ string_to_location(int boardsize, char *str, int *m, int *n)
 }
 
 
+/* Some simple functions to draw an ASCII board. */
+
+/* True if the coordinate is a hoshi point. */
+int
+is_hoshi_point(int m, int n)
+{
+  int hoshi;
+  int middle;
+
+  /* No hoshi points on these boards. */
+  if (board_size == 2 || board_size == 4)
+    return 0;
+
+  /* In the middle of a 3x3 board. */
+  if (board_size == 3) {
+    if (m == 1 && n == 1)
+      return 1;
+
+    return 0;
+  }
+
+  if (board_size == 5) {
+    if (m == 1 && (n == 1 || n == 3))
+      return 1;
+    if (m == 2 && n == 2)
+      return 1;
+    if (m == 3 && (n == 1 || n == 3))
+      return 1;
+
+    return 0;
+  }
+
+  /* 3-3 points are hoshi on sizes 7--11, 4-4 on larger. */
+  if (board_size <= 11)
+    hoshi = 2;
+  else
+    hoshi = 3;
+
+  /* Coordinate for midpoint. */
+  middle = board_size/2;
+    
+  /* Normalize the coordinates by mirroring to the lower numbers. */
+  if (m >= middle)
+    m = board_size - 1 - m;
+  if (n >= middle)
+    n = board_size - 1 - n;
+  
+  /* Is this a corner hoshi? */
+  if (m == hoshi && n == hoshi)
+    return 1;
+
+  /* If even sized board, only hoshi points in the corner. */
+  if (board_size%2 == 0)
+    return 0;
+
+  /* Less than 12 in board size only middle point. */
+  if (board_size < 12) {
+    if (m == middle && n == middle)
+      return 1;
+
+    return 0;
+  }
+
+  /* Is this a midpoint hoshi? */
+  if ((m == hoshi || m == middle)
+      && (n == hoshi || n == middle))
+    return 1;
+
+  /* No more chances. */
+  return 0;
+}
+
+
+/* Print a line with coordinate letters above the board. */
+void
+draw_letter_coordinates(FILE *outfile)
+{
+  int i;
+  int ch;
+  
+  fprintf(outfile, "  ");
+  for (i = 0, ch = 'A'; i < board_size; i++, ch++) {
+    if (ch == 'I')
+      ch++;
+    fprintf(outfile, " %c", ch);
+  }
+}
+
+
+/* Bare bones version of showboard(0). No fancy options, no hint of
+ * color, and you can choose where to write it.
+ */
+void
+simple_showboard(FILE *outfile)
+{
+  int i, j;
+
+  draw_letter_coordinates(outfile);
+  
+  for (i = 0; i < board_size; i++) {
+    fprintf(outfile, "\n%2d", board_size - i);
+    
+    for (j = 0; j < board_size; j++) {
+      if (BOARD(i, j) == EMPTY)
+	fprintf(outfile, " %c", is_hoshi_point(i, j) ? '+' : '.');
+      else
+	fprintf(outfile, " %c", BOARD(i, j) == BLACK ? 'X' : 'O');
+    }
+
+    fprintf(outfile, " %d", board_size - i);
+    
+    if ((board_size < 10 && i == board_size-2)
+	|| (board_size >= 10 && i == 8))
+      fprintf(outfile, "     WHITE (O) has captured %d stones", black_captured);
+    
+    if ((board_size < 10 && i == board_size-1)
+	|| (board_size >= 10 && i == 9))
+      fprintf(outfile, "     BLACK (X) has captured %d stones", white_captured);
+  }
+  
+  fprintf(outfile, "\n");
+  draw_letter_coordinates(outfile);
+}
+
+
+
 /*
  * Local Variables:
  * tab-width: 8
