@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -68,8 +68,11 @@ dilate_erode(int dilations, int erosions, int gb[BOARDMAX], int color)
     if (!ON_BOARD(ii))
       continue;
 
-    if (board[ii] && dragon[ii].status == CRITICAL)
+    if (board[ii] && dragon[ii].status == CRITICAL
+	&& DRAGON2(ii).safety != INESSENTIAL) {
       critical_found = 1;
+      DEBUG(DEBUG_SCORING, "critical dragon found at %1m\n", ii);
+    }
     if (board[ii] == WHITE && !captured_territory(ii, color))
       gb[ii] = 128;
     else if (board[ii] == BLACK && !captured_territory(ii, color))      
@@ -493,7 +496,7 @@ estimate_score(float *upper, float *lower)
       l = white_area - black_area + komi;
     else {
       DEBUG(DEBUG_SCORING,
-	  "black captured: %d\nwhite captured: %d\nkomi: %f\n",
+	    "black captured: %d\nwhite captured: %d\nkomi: %f\n",
 	    black_captured, white_captured, komi);
       l = white_territory 
 	+ black_captured - black_territory - white_captured + komi;
@@ -549,10 +552,11 @@ captured_territory(int pos, int color)
   for (d = 0; d < DRAGON2(pos).neighbors; d++)
     if (DRAGON(DRAGON2(pos).adjacent[d]).color == OTHER_COLOR(board[pos])
 	&& (DRAGON(DRAGON2(pos).adjacent[d]).status == ALIVE
-	|| (board[pos] != color
-	    && DRAGON(DRAGON2(pos).adjacent[d]).status == CRITICAL)))
+	    || (DRAGON(DRAGON2(pos).adjacent[d]).status == UNKNOWN &&
+		dragon2[DRAGON2(pos).adjacent[d]].weakness < .1)
+	    || (board[pos] != color
+		&& DRAGON(DRAGON2(pos).adjacent[d]).status == CRITICAL)))
       return 1;
-
   return 0;
 }
 
