@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -44,11 +44,11 @@ decide_string(int pos)
 {
   int aa, dd;
   int acode, dcode;
-  SGFTree  tree;
+  SGFTree tree;
   
   if (board[pos] == EMPTY) {
     fprintf(stderr, "gnugo: --decide-string called on an empty vertex\n");
-    return ;
+    return;
   }
 
   if (*outfilename)
@@ -205,13 +205,13 @@ decide_owl(int pos)
 
   if (board[pos] == EMPTY) {
     fprintf(stderr, "gnugo: --decide-dragon called on an empty vertex\n");
-    return ;
+    return;
   }
 
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
-  silent_examine_position(board[pos], EXAMINE_DRAGONS_WITHOUT_OWL);
+  silent_examine_position(EXAMINE_DRAGONS_WITHOUT_OWL);
   gprintf("finished examine_position\n");
 
   /* We want to see the reading performed, not just a result picked
@@ -295,10 +295,10 @@ decide_dragon_data(int pos)
 {
   if (board[pos] == EMPTY) {
     fprintf(stderr, "gnugo: --decide-dragon-data called on an empty vertex\n");
-    return ;
+    return;
   }
   reset_engine();
-  silent_examine_position(board[pos], FULL_EXAMINE_DRAGONS);
+  silent_examine_position(FULL_EXAMINE_DRAGONS);
 
   gprintf("Dragon at %1m:\n", pos);
   report_dragon(stderr, pos);
@@ -324,7 +324,7 @@ decide_semeai(int apos, int bpos)
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
-  silent_examine_position(board[apos], EXAMINE_DRAGONS_WITHOUT_OWL);
+  silent_examine_position(EXAMINE_DRAGONS_WITHOUT_OWL);
   gprintf("finished examine_position\n");
   count_variations = 1;
 
@@ -377,7 +377,7 @@ decide_tactical_semeai(int apos, int bpos)
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
-  silent_examine_position(board[apos], EXAMINE_DRAGONS_WITHOUT_OWL);
+  silent_examine_position(EXAMINE_DRAGONS_WITHOUT_OWL);
   gprintf("finished examine_position\n");
   count_variations = 1;
 
@@ -390,17 +390,17 @@ decide_tactical_semeai(int apos, int bpos)
 
   owl_analyze_semeai(apos, bpos, &resulta, &resultb, &move, 0, &dummy);
   gprintf("After %s at %1m, %1m is %s, %1m is %s (%d nodes)\n",
-	  color == BLACK ? "black" : "white",
+	  color_to_string(color),
 	  move,
-	  apos, safety_to_string(resulta),
-  	  bpos, safety_to_string(resultb),
+	  apos, status_to_string(resulta),
+  	  bpos, status_to_string(resultb),
 	  count_variations);
   owl_analyze_semeai(bpos, apos, &resultb, &resulta, &move, 0, &dummy);
   gprintf("After %s at %1m, %1m is %s, %1m is %s (%d nodes)\n",
-	  color == BLACK ? "white" : "black",
+	  color_to_string(color),
 	  move,
-	  apos, safety_to_string(resulta),
-  	  bpos, safety_to_string(resultb),
+	  apos, status_to_string(resulta),
+  	  bpos, status_to_string(resultb),
 	  count_variations);
 
   sgffile_enddump(outfilename);
@@ -414,7 +414,7 @@ decide_tactical_semeai(int apos, int bpos)
  */
 
 void
-decide_position(int color)
+decide_position()
 {
   int pos;
   int move = NO_MOVE;
@@ -426,7 +426,7 @@ decide_position(int color)
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
-  silent_examine_position(color, EXAMINE_DRAGONS_WITHOUT_OWL);
+  silent_examine_position(EXAMINE_DRAGONS_WITHOUT_OWL);
 
   /* We want to see the reading performed, not just a result picked
    * from the cache. Thus we clear the cache here. */
@@ -525,13 +525,10 @@ decide_eye(int pos)
   SGFTree tree;
 
   reset_engine();
-  silent_examine_position(BLACK, EXAMINE_DRAGONS_WITHOUT_OWL);
+  silent_examine_position(EXAMINE_DRAGONS_WITHOUT_OWL);
   
-  if (black_eye[pos].color == BLACK_BORDER) 
-    color = BLACK;
-  else if (white_eye[pos].color == WHITE_BORDER) 
-    color = WHITE;
-  else {
+  color = black_eye[pos].color;
+  if (!IS_STONE(color)) {
     gprintf("The eye at %1m is not of a single color.\n", pos);
     return;
   }
@@ -544,10 +541,10 @@ decide_eye(int pos)
     sgffile_begindump(&tree);
   count_variations = 1;
   
-  if (black_eye[pos].color == BLACK_BORDER) {
+  if (black_eye[pos].color == BLACK) {
     eyepos = black_eye[pos].origin;
     compute_eyes(eyepos, &value, &attack_point, &defense_point,
-		 black_eye, half_eye, 0, EMPTY);
+		 black_eye, half_eye, 0);
     gprintf("Black eyespace at %1m: %s\n", eyepos, eyevalue_to_string(&value));
     if (eye_move_urgency(&value) > 0) {
       gprintf("  vital points: %1m (attack) %1m (defense)\n", attack_point,
@@ -555,10 +552,10 @@ decide_eye(int pos)
     }
   }
   
-  if (white_eye[pos].color == WHITE_BORDER) {
+  if (white_eye[pos].color == WHITE) {
     eyepos = white_eye[pos].origin;
     compute_eyes(eyepos, &value, &attack_point, &defense_point,
-		 white_eye, half_eye, 0, EMPTY);
+		 white_eye, half_eye, 0);
     gprintf("White eyespace at %1m: %s\n", eyepos, eyevalue_to_string(&value));
     if (eye_move_urgency(&value) > 0) {
       gprintf("  vital points: %1m (attack) %1m (defense)\n", attack_point,
@@ -589,7 +586,7 @@ decide_combination(int color)
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
-  silent_examine_position(color, EXAMINE_ALL);
+  silent_examine_position(EXAMINE_ALL);
 
   if (*outfilename)
     sgffile_begindump(&tree);
@@ -630,7 +627,7 @@ decide_surrounded(int pos)
   /* Prepare pattern matcher and reading code. */
   reset_engine();
 
-  silent_examine_position(board[pos], EXAMINE_ALL);
+  silent_examine_position(EXAMINE_ALL);
   surround_status = compute_surroundings(pos, NO_MOVE, 1, NULL);
   if (surround_status == 1)
     gprintf("the dragon at %1m is SURROUNDED!\n", pos);
@@ -641,29 +638,6 @@ decide_surrounded(int pos)
 }  
 
 
-#if ORACLE
-
-void
-decide_oracle(Gameinfo *gameinfo, char *infilename, char *untilstring)
-{
-  SGFTree tree;
-
-  reset_engine();
-  if (*outfilename)
-    sgffile_begindump(&tree);
-
-  count_variations = 1;
-  summon_oracle();
-  oracle_loadsgf(infilename, untilstring);
-  consult_oracle(gameinfo->to_move);
-  sgffile_enddump(outfilename);
-  dismiss_oracle();
-  count_variations = 0;
-}
-
-#endif
-
-
 /*
  * Local Variables:
  * tab-width: 8
