@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -59,7 +59,7 @@ change_dragon_status(int dr, int status)
 int
 defend_against(int move, int color, int apos)
 {
-  if (trymove(move, color, "defend_against", NO_MOVE, EMPTY, NO_MOVE)) {
+  if (trymove(move, color, "defend_against", NO_MOVE)) {
     if (safe_move(apos, OTHER_COLOR(color)) == 0) {
       popgo();
       return 1;
@@ -79,12 +79,7 @@ defend_against(int move, int color, int apos)
 int
 cut_possible(int pos, int color)
 {
-  if (color == WHITE)
-    return (black_eye[pos].cut
-	    || (black_eye[pos].type & INHIBIT_CONNECTION));
-  else
-    return (white_eye[pos].cut
-	    || (white_eye[pos].type & INHIBIT_CONNECTION));
+  return (cutting_points[pos] & OTHER_COLOR(color)) != 0;
 }
 
 
@@ -116,12 +111,11 @@ does_attack(int move, int str)
       return 0;
   }
   
-  if (trymove(move, other, "does_attack-A", str, EMPTY, NO_MOVE)) {
+  if (trymove(move, other, "does_attack-A", str)) {
     if (!board[str] || !find_defense(str, NULL)) {
       result = WIN;
       increase_depth_values();
-      if (spos != NO_MOVE && trymove(spos, color, "does_attack-B", str,
-				     EMPTY, NO_MOVE)) {
+      if (spos != NO_MOVE && trymove(spos, color, "does_attack-B", str)) {
 	if (board[str] && !attack(str, NULL))
 	  result = 0;
 	popgo();
@@ -160,11 +154,11 @@ does_defend(int move, int str)
 
   gg_assert(spos != NO_MOVE);
   
-  if (trymove(move, color, "does_defend-A", str, EMPTY, NO_MOVE)) {
+  if (trymove(move, color, "does_defend-A", str)) {
     if (!attack(str, NULL)) {
       result = 1;
       increase_depth_values();
-      if (trymove(spos, other, "does_defend-B", str, EMPTY, NO_MOVE)) {
+      if (trymove(spos, other, "does_defend-B", str)) {
 	if (!board[str] || !find_defense(str, NULL))
 	  result = 0;
 	popgo();
@@ -273,6 +267,48 @@ visible_along_edge(int color, int apos, int bpos)
   return 0;
 }
 
+/* Is the board symmetric (or rather antisymmetric) with respect to
+ * mirroring in tengen after a specific move has been played? If the
+ * move is PASS_MOVE, check the current board.
+ *
+ * If strict is set we require that each stone is matched by a stone
+ * of the opposite color at the mirrored vertex. Otherwise we only
+ * require that each stone is matched by a stone of either color.
+ */
+int
+test_symmetry_after_move(int move, int color, int strict)
+{
+  int pos;
+  int result = 1;
+
+  if (move != PASS_MOVE) {
+    if (board[move] != EMPTY)
+      return 0;
+    if (!trymove(move, color, "find_mirror_move", NO_MOVE))
+      return 0;
+  }
+  
+  for (pos = BOARDMIN; pos < MIRROR_MOVE(pos); pos++) {
+    int sum;
+    if (!ON_BOARD(pos))
+      continue;
+    
+    sum = board[pos] + board[MIRROR_MOVE(pos)];
+    if (sum != EMPTY + EMPTY && sum != BLACK + WHITE) {
+      if (strict || sum == EMPTY + WHITE || sum == EMPTY + BLACK) {
+      result = 0;
+      break;
+      }
+    }
+  }
+
+  if (move != PASS_MOVE)
+    popgo();
+  
+  return result;
+}
+
+
 /* The function play_break_through_n() plays a sequence of moves,
  * alternating between the players and starting with color. After
  * having played through the sequence, the three last coordinate pairs
@@ -311,9 +347,8 @@ play_break_through_n(int color, int num_moves, ...)
     apos = va_arg(ap, int);
 
     if (apos != NO_MOVE
-	&& (trymove(apos, mcolor, "play_break_through_n", NO_MOVE,
-		    EMPTY, NO_MOVE)
-	    || tryko(apos, mcolor, "play_break_through_n", EMPTY, NO_MOVE)))
+	&& (trymove(apos, mcolor, "play_break_through_n", NO_MOVE)
+	    || tryko(apos, mcolor, "play_break_through_n")))
       played_moves++;
     mcolor = OTHER_COLOR(mcolor);
   }
@@ -382,9 +417,8 @@ play_attack_defend_n(int color, int do_attack, int num_moves, ...)
     apos = va_arg(ap, int);
 
     if (apos != NO_MOVE
-	&& (trymove(apos, mcolor, "play_attack_defend_n", NO_MOVE,
-		    EMPTY, NO_MOVE)
-	    || tryko(apos, mcolor, "play_attack_defend_n", EMPTY, NO_MOVE)))
+	&& (trymove(apos, mcolor, "play_attack_defend_n", NO_MOVE)
+	    || tryko(apos, mcolor, "play_attack_defend_n")))
       played_moves++;
     mcolor = OTHER_COLOR(mcolor);
   }
@@ -468,9 +502,8 @@ play_attack_defend2_n(int color, int do_attack, int num_moves, ...)
     apos = va_arg(ap, int);
 
     if (apos != NO_MOVE
-	&& (trymove(apos, mcolor, "play_attack_defend_n", NO_MOVE,
-		    EMPTY, NO_MOVE)
-	    || tryko(apos, mcolor, "play_attack_defend_n", EMPTY, NO_MOVE)))
+	&& (trymove(apos, mcolor, "play_attack_defend_n", NO_MOVE)
+	    || tryko(apos, mcolor, "play_attack_defend_n")))
       played_moves++;
     mcolor = OTHER_COLOR(mcolor);
   }
@@ -544,9 +577,8 @@ play_connect_n(int color, int do_connect, int num_moves, ...)
     apos = va_arg(ap, int);
 
     if (apos != NO_MOVE
-	&& (trymove(apos, mcolor, "play_connect_n", NO_MOVE,
-		    EMPTY, NO_MOVE)
-	    || tryko(apos, mcolor, "play_connect_n", EMPTY, NO_MOVE)))
+	&& (trymove(apos, mcolor, "play_connect_n", NO_MOVE)
+	    || tryko(apos, mcolor, "play_connect_n")))
       played_moves++;
     mcolor = OTHER_COLOR(mcolor);
   }
@@ -656,7 +688,6 @@ set_depth_values(int level, int report_levels)
    * Other policies depending on level:
    * aftermath.c:   >=  8: call estimate_score().
    * dragon.c:      >=  8: compute owl threats (currently disabled)
-   *                >=  8: call owl analysis of semeai (currently disabled)
    * genmove.c:     >=  8: call estimate_score().
    * owl.c:         >=  9: use vital attack pattern database
    *                >=  8: increase depth values in owl_substantial
@@ -689,8 +720,9 @@ set_depth_values(int level, int report_levels)
   break_chain_depth    = gg_max(2, BREAK_CHAIN_DEPTH + depth_level);
   if (level >= 8)
     owl_distrust_depth = gg_max(1, (2 * OWL_DISTRUST_DEPTH + depth_level) / 2);
-  else owl_distrust_depth = 
-      gg_max(1, (2 * OWL_DISTRUST_DEPTH - 1 + depth_level) / 2);
+  else
+    owl_distrust_depth = gg_max(1, (2 * OWL_DISTRUST_DEPTH - 1
+				    + depth_level) / 2);
   owl_branch_depth     = gg_max(2, (2 * OWL_BRANCH_DEPTH   + depth_level) / 2);
   owl_reading_depth    = gg_max(5, (2 * OWL_READING_DEPTH  + depth_level) / 2);
 
@@ -700,7 +732,7 @@ set_depth_values(int level, int report_levels)
   else if (level == 9)
     aa_depth = gg_max(0, AA_DEPTH);
   else if (level >= 7)
-    aa_depth = gg_max(0, AA_DEPTH -1);
+    aa_depth = gg_max(0, AA_DEPTH - 1);
   else
     aa_depth = gg_max(0, AA_DEPTH - (8 - level));
 
@@ -774,6 +806,8 @@ set_depth_values(int level, int report_levels)
   if (mandated_owl_node_limit != -1)
     owl_node_limit = mandated_owl_node_limit;
 
+  depth_offset = 0;
+  
   if (report_levels) {
     fprintf(stderr, "at level %d:\n\n\
 depth: %d\n\
@@ -828,6 +862,8 @@ modify_depth_values(int n)
   branch_depth       += n;
   fourlib_depth      += n;
   ko_depth           += n;
+  breakin_depth	     += n;
+  depth_offset       += n;
 }
 
 void
@@ -942,11 +978,11 @@ blunder_size(int move, int color, int *defense_point,
 {
   int libs[5];
   int liberties = accuratelib(move, color, 5, libs);
-  int apos;
   int trouble = 0;
   int save_verbose = verbose;
   float return_value = 0.0;
   int atari;
+  char defense_moves[BOARDMAX];
   
   if (defense_point)
     *defense_point = NO_MOVE;
@@ -982,13 +1018,20 @@ blunder_size(int move, int color, int *defense_point,
    * set up some combination attack that didn't exist before. We do
    * this last to avoid duplicate blunder reports.
    */
-  atari = atari_atari_blunder_size(color, move, &apos, safe_stones);
+  atari = atari_atari_blunder_size(color, move, defense_moves, safe_stones);
   if (atari) {
-    ASSERT_ON_BOARD1(apos);
-    if (defense_point)
-      *defense_point = apos;
+    if (defense_point) {
+      /* FIXME: Choose defense point more systematically. */
+      int pos;
+      *defense_point = NO_MOVE;
+      for (pos = BOARDMIN; pos < BOARDMAX; pos++)
+	if (ON_BOARD(pos) && defense_moves[pos]) {
+	  *defense_point = pos;
+	  break;
+	}
+    }
     verbose = save_verbose;
-    TRACE("Combination attack appears at %1m.\n", apos);
+    TRACE("Combination attack appears.\n");
     return_value += (float) atari;
   }
 
@@ -1058,7 +1101,7 @@ detect_owl_blunder(int move, int color, int *defense_point,
 	  if (board[neighbor] == color)
 	    continue;
 	  owl_analyze_semeai_after_move(move, color, neighbor, bpos,
-					NULL, &resultb, NULL, 1, NULL);
+					NULL, &resultb, NULL, 1, NULL, 0);
 	  if (resultb == 0)
 	    acode = WIN;
 	}
@@ -1116,9 +1159,9 @@ detect_tactical_blunder(int move, int color, int *defense_point,
   int other = OTHER_COLOR(color);
   int pos;
   int ii;
-  int current_verbose = save_verbose;
+  int current_verbose = verbose;
 
-  if (!trymove(move, color, NULL, NO_MOVE, EMPTY, NO_MOVE))
+  if (!trymove(move, color, NULL, NO_MOVE))
     return;
   
   /* Need to increase the depth values during this reading to avoid
@@ -1166,41 +1209,62 @@ detect_tactical_blunder(int move, int color, int *defense_point,
       decrease_depth_values();
       owl_attacks = owl_does_attack(move, pos, NULL);
       if (owl_attacks != WIN) {
-	*return_value += worm[pos].effective_size;
+	*return_value += 2 * worm[pos].effective_size;
 	defense_effective = 1;
 	verbose = save_verbose;
-	TRACE("After %1m worm at %1m becomes defendable.\n", move, pos);
+	TRACE("After %1m worm at %1m becomes defendable - A.\n", move, pos);
 	verbose = current_verbose;
       }
-      else {
+      else if (dragon[pos].status != ALIVE) {
 	/* Before redoing the trymove we also check whether the worm now
 	 * has a semeai defense. See blunder:26 for an example.
+	 *
+	 * If the worm already was alive in seki, it is generally okay
+	 * that it also becomes tactically safe when the outer
+	 * liberties are filled, see e.g. blunder:32,34. Thus the
+	 * check above.
 	 */
 	int k;
-	for (k = 0; k < DRAGON2(pos).neighbors; k++) {
-	  int neighbor = dragon2[DRAGON2(pos).adjacent[k]].origin;
+	int adj[MAXCHAIN];
+	int num_adj;
+	num_adj = extended_chainlinks(pos, adj, 0);
+	for (k = 0; k < num_adj; k++) {
+	  int neighbor = adj[k];
 	  int resulta;
-	  if (board[neighbor] != color)
-	    continue;
 	  owl_analyze_semeai_after_move(move, color, pos, neighbor,
-					&resulta, NULL, NULL, 1, NULL);
+					&resulta, NULL, NULL, 1, NULL, 1);
 	  if (resulta != 0) {
-	    *return_value += worm[pos].effective_size;
+	    *return_value += 2 * worm[pos].effective_size;
 	    defense_effective = 1;
 	    verbose = save_verbose;
-	    TRACE("After %1m worm at %1m becomes defendable.\n", move, pos);
+	    TRACE("After %1m worm at %1m becomes defendable - B.\n",
+		  move, pos);
 	    verbose = current_verbose;
+	    break;
 	  }
 	}
       }
       
-      trymove(move, color, NULL, NO_MOVE, EMPTY, NO_MOVE);
+      trymove(move, color, NULL, NO_MOVE);
       increase_depth_values();
       
       if (defense_effective && defense_point) {
 	int dpos;
-	if (attack(pos, &dpos))
+	if (attack(pos, &dpos)) {
 	  *defense_point = dpos;
+	  /* Check that this move is legal and effective also on the
+           * original board, otherwise find a tactical attack there
+           * instead.
+	   */
+	  popgo();
+	  
+	  if (!is_legal(dpos, color)
+	      || play_attack_defend_n(color, 0, 1, dpos, pos))
+	    attack(pos, defense_point);
+
+	  /* Redo the move, we know that it won't fail. */
+	  trymove(move, color, NULL, NO_MOVE);
+	}
 	else {
 	  verbose = save_verbose;
 	  TRACE("No attack found (unexpectedly) on %1m after move at %1m.\n",
@@ -1288,7 +1352,7 @@ double_atari(int move, int color, float *value, char safe_stones[BOARDMAX])
 	&& BOARD(m+dm, n) == other
 	&& (!safe_stones
 	    || (safe_stones[POS(m, n+dn)] && safe_stones[POS(m+dm, n)]))
-	&& trymove(move, color, "double_atari", NO_MOVE, EMPTY, NO_MOVE)) {
+	&& trymove(move, color, "double_atari", NO_MOVE)) {
       if (countlib(move) > 1
 	  && (BOARD(m, n+dn) == EMPTY || BOARD(m+dm, n) == EMPTY 
 	      || !defend_both(POS(m, n+dn), POS(m+dm, n)))) {
@@ -1316,60 +1380,17 @@ double_atari(int move, int color, float *value, char safe_stones[BOARDMAX])
 }
     
 
-/* Find those worms of the given color that can never be captured,
- * even if the opponent is allowed an arbitrary number of consecutive
- * moves. The coordinates of the origins of these worms are written to
- * the worm arrays and the number of non-capturable worms is
- * returned.
- *
- * The algorithm is to cycle through the worms until none remains or
- * no more can be captured. A worm is removed when it is found to be
- * capturable, by letting the opponent try to play on all its
- * liberties. If the attack fails, the moves are undone. When no more
- * worm can be removed in this way, the remaining ones are
- * unconditionally alive.
- *
- * After this, unconditionally dead opponent worms and unconditional
- * territory are identified. To find these, we continue from the
- * position obtained at the end of the previous operation (only
- * unconditionally alive strings remain for color) with the following
- * steps:
- *
- * 1. Play opponent stones on all liberties of the unconditionally
- *    alive strings except where illegal. (That the move order may
- *    determine exactly which liberties can be played legally is not
- *    important. Just pick an arbitrary order).
- * 2. Recursively extend opponent strings in atari, except where this
- *    would be suicide.
- * 3. Play an opponent stone anywhere it can get two empty
- *    neighbors. (I.e. split big eyes into small ones).
- * 4. Play an opponent stone anywhere it can get one empty
- *    neighbor. (I.e. reduce two space eyes to one space eyes.)
- *
- * Remaining opponent strings in atari and remaining liberties of the
- * unconditionally alive strings constitute the unconditional
- * territory.
- *
- * Opponent strings from the initial position placed on
- * unconditional territory are unconditionally dead.
- *
- * On return, unconditional_territory[][] is 1 where color has
- * unconditionally alive stones, 2 where it has unconditional
- * territory, and 0 otherwise.
- */
-
-void
-unconditional_life(int unconditional_territory[BOARDMAX], int color)
+static int
+capture_non_invincible_strings(int color, int exceptions[BOARDMAX])
 {
+  int other = OTHER_COLOR(color);
   int something_captured = 1; /* To get into the first turn of the loop. */
-  int found_one;
   int moves_played = 0;
   int save_moves;
-  int pos;
-  int k;
   int libs[MAXLIBS];
   int liberties;
-  int other = OTHER_COLOR(color);
+  int pos;
+  int k;
   
   while (something_captured) {
     /* Nothing captured so far in this turn of the loop. */
@@ -1377,14 +1398,17 @@ unconditional_life(int unconditional_territory[BOARDMAX], int color)
 
     /* Visit all friendly strings on the board. */
     for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-      if (board[pos] != color || !is_worm_origin(pos, pos))
+      if (board[pos] != color || find_origin(pos) != pos)
 	continue;
-	
-      /* Try to capture the worm at (m, n). */
+
+      if (exceptions && exceptions[pos])
+	continue;
+      
+      /* Try to capture the string at pos. */
       liberties = findlib(pos, MAXLIBS, libs);
       save_moves = moves_played;
       for (k = 0; k < liberties; k++) {
-	if (trymove(libs[k], other, "unconditional_life", pos, EMPTY, 0))
+	if (trymove(libs[k], other, "unconditional_life", pos))
 	  moves_played++;
       }
       
@@ -1405,10 +1429,10 @@ unconditional_life(int unconditional_territory[BOARDMAX], int color)
 	 * |.XO.O|
 	 * +-----+
 	 */
-	int success = tryko(libs[0], other, "unconditional_life", EMPTY, 0);
+	int success = tryko(libs[0], other, "unconditional_life");
 	gg_assert(success);
 	moves_played++;
-	something_captured++;
+	something_captured = 1;
       }
       else
 	while (moves_played > save_moves) {
@@ -1417,21 +1441,268 @@ unconditional_life(int unconditional_territory[BOARDMAX], int color)
 	}
     }
   }
+  return moves_played;
+}
+
+/* Find those worms of the given color that can never be captured,
+ * even if the opponent is allowed an arbitrary number of consecutive
+ * moves. The coordinates of the origins of these worms are written to
+ * the worm arrays and the number of non-capturable worms is
+ * returned.
+ *
+ * The algorithm is to cycle through the worms until none remains or
+ * no more can be captured. A worm is removed when it is found to be
+ * capturable, by letting the opponent try to play on all its
+ * liberties. If the attack fails, the moves are undone. When no more
+ * worm can be removed in this way, the remaining ones are
+ * unconditionally alive.
+ *
+ * After this, unconditionally dead opponent worms and unconditional
+ * territory are identified. This is almost, but only almost,
+ * straightforward. We first present a simple but only almost correct
+ * solution, then show how to patch up its deficiencies.
+ *
+ * - - - - - - -
+ *
+ * Algorithm 1, simple but slightly incorrect.
+ *
+ * To find unconditionally dead opponent worms and unconditional
+ * territory, we continue from the position obtained at the end of the
+ * previous operation (only unconditionally alive strings remain for
+ * color) with the following steps:
+ *
+ * 1. Play opponent stones on all liberties of the unconditionally
+ *    alive strings except where illegal. (That the move order may
+ *    determine exactly which liberties can be played legally is not
+ *    important. Just pick an arbitrary order).
+ * 2. Recursively extend opponent strings in atari, except where this
+ *    would be suicide.
+ * 3. Play an opponent stone anywhere it can get two empty
+ *    neighbors. (I.e. split big eyes into small ones).
+ * 4. Play an opponent stone anywhere it can get one empty
+ *    neighbor. (I.e. reduce two space eyes to one space eyes.)
+ *
+ * Remaining opponent strings in atari and remaining liberties of the
+ * unconditionally alive strings constitute the unconditional
+ * territory.
+ *
+ * Opponent strings from the initial position placed on
+ * unconditional territory are unconditionally dead.
+ *
+ * - - - - - - -
+ *
+ * The deficiency with this algorithm is that a certain class of sekis
+ * are considered as dead, e.g. this position:
+ *
+ * .OOOOO.
+ * OOXXXOO
+ * OXX.XXO
+ * OX.O.XO
+ * OX.O.XO
+ * OXX.XXO
+ * OOXXXOO
+ * .OOOOO.
+ *
+ * The problem is that while removing the two O stones, X is reduced
+ * to a single small eye. Still O cannot capture these stones under
+ * alternating play since the eyespace is too big.
+ *
+ * Before discussing this seki further we make a preliminary
+ * modification of the algorithm.
+ *
+ * - - - - - - -
+ *
+ * Algorithm 2. More complex but still slightly incorrect algorithm:
+ *
+ * 1. Run algorithm 1.
+ * 2. Return to the original position.
+ * 3. Capture all capturable O strings which according to algorithm 1
+ *    do not belong to unconditional territory.
+ * 4. Play opponent stones on all liberties of the unconditionally
+ *    alive strings except where illegal. (That the move order may
+ *    determine exactly which liberties can be played legally is not
+ *    important. Just pick an arbitrary order).
+ * 5. Recursively extend opponent strings in atari, except where this
+ *    would be suicide.
+ * 6. Capture all remaining capturable O strings.
+ * 7. Repeat 4 and 5 once.
+ * 8. Play an opponent stone anywhere it can get two empty
+ *    neighbors. (I.e. split big eyes into small ones).
+ * 9. Play an opponent stone anywhere it can get one empty
+ *    neighbor. (I.e. reduce two space eyes to one space eyes.)
+ *
+ * Remaining opponent strings in atari and remaining liberties of the
+ * unconditionally alive strings constitute the unconditional
+ * territory.
+ *
+ * Opponent strings from the initial position placed on
+ * unconditional territory are unconditionally dead.
+ *
+ * - - - - - - -
+ *
+ * We can observe that, after step 5, an X group with at least two
+ * distinct eyespaces would not risk being reduced to a single small
+ * eye. Similarly an X group with a capturable O string of size at
+ * least three would allow the formation of two distinct small eyes
+ * after being captured. Thus it is easy to see that the only X groups
+ * which would live in seki but could not be transformed into
+ * unconditionally alive groups would have a single eyespace with a
+ * capturable O string of size at most 2. Furthermore the eyespace
+ * would not be possible to subdivide. Then if the capturable string
+ * would be of size 1 it would in all cases form a nakade and we would
+ * not have a seki. The plausible seki positions would all be
+ * reducable to the following eyeshape:
+ *
+ * .OOOOO.
+ * OOXXXO.
+ * OXX.XOO
+ * OX.OXXO
+ * OXXO.XO
+ * OOX.XXO
+ * .OXXXOO
+ * .OOOOO.
+ *
+ * The remaining question is what effects cutting points in the X
+ * group would have. For example these X groups are dead:
+ *
+ * .OOOOO.   .OOOOO.   .OOOOO.   .OOOOO.   ..OOOO.   ..OOOO.
+ * .OXXXO.   .OXXXO.   .OXXXO.	 .OXXXO.   OOOXXO.   OOOXXO.
+ * OOX.XO.   OOX.XOO   OOX.XOO	 OOX.XOO   OXX.XO.   OXX.XOO
+ * OX.OXOO   OX.OXXO   OX.OXXO	 OX.OXXO   OX.OXOO   OX.OXXO
+ * OXXO.XO   OXXO.XO   OXXO.XO	 OXXO.XO   OXXO.XO   OXXO.XO
+ * OOX.XXO   OOX.XOO   OOX.XXO	 OOX.XXO   OOX.XXO   OOX.XXO
+ * .OXXXOO   .OXXXO.   .OXXOOO	 .OOXXOO   .OXXXOO   .OXXOOO
+ * .OOOOO.   .OOOOO.   .OOOO..	 ..OOOO.   .OOOOO.   .OOOO..
+ *
+ * while these are alive in seki
+ *
+ * ..OOOO.   .OOOO..   .OOOO..   ..OOOO.   ..OOOO.
+ * OOOXXO.   .OXXOO.   OOXXOO.	 .OOXXO.   OOOXXO.
+ * OXX.XOO   OOX.XOO   OXX.XOO	 OOX.XOO   OXX.XOO
+ * OX.OXXO   OX.OXXO   OX.OXXO	 OX.OXXO   OX.OXXO
+ * OXXO.XO   OXXO.XO   OOXO.XO	 OXXO.XO   OOXO.XO
+ * OOX.XXO   OOX.XXO   .OX.XXO	 OOX.XXO   .OX.XXO
+ * .OXXXOO   .OXXXOO   .OXXOOO	 .OXXXOO   .OXXXOO
+ * .OOOOO.   .OOOOO.   .OOOO..	 ..OOOO.   .OOOOO.
+ *
+ * The critical distinction between the dead ones and the seki ones is
+ * that the stones marked a and b below,
+ *
+ * .OOOOO.
+ * OOXXXO.
+ * OXX.XOO
+ * OX.ObXO
+ * OXaO.XO
+ * OOX.XXO
+ * .OXXXOO
+ * .OOOOO.
+ *
+ * belong to different strings for the dead groups and to the same
+ * string for the seki groups.
+ *
+ * The trick to avoid misclassifying areas where the opponent can form
+ * a seki group but not an invincible group as unconditional territory
+ * is thus to detect the formation above and add a third stone to the
+ * O group before the capturing in step 6 above.
+ *
+ * This leads to the final algorithm.
+ *
+ * - - - - - - -
+ *
+ * Algorithm 3. Final and correct algorithm:
+ *
+ * 1. Run algorithm 1.
+ * 2. Return to the original position.
+ * 3. Capture all capturable O strings which according to algorithm 1
+ *    do not belong to unconditional territory.
+ * 4. Play opponent stones on all liberties of the unconditionally
+ *    alive strings except where illegal. (That the move order may
+ *    determine exactly which liberties can be played legally is not
+ *    important. Just pick an arbitrary order).
+ * 5. Recursively extend opponent strings in atari, except where this
+ *    would be suicide.
+ * 6. Identify eyespaces of the kind described above and extend any
+ *    matching two-stone string with a third stone.
+ * 7. Capture all remaining capturable O strings.
+ * 8. Repeat 4 and 5 once.
+ * 9. Play an opponent stone anywhere it can get two empty
+ *    neighbors. (I.e. split big eyes into small ones).
+ * 10. Play an opponent stone anywhere it can get one empty
+ *    neighbor. (I.e. reduce two space eyes to one space eyes.)
+ *
+ * Remaining opponent strings in atari and remaining liberties of the
+ * unconditionally alive strings constitute the unconditional
+ * territory.
+ *
+ * Opponent strings from the initial position placed on
+ * unconditional territory are unconditionally dead.
+ *
+ * - - - - - - -
+ *
+ * On return, unconditional_territory[][] is 1 where color has
+ * unconditionally alive stones, 2 where it has unconditional
+ * territory, and 0 otherwise.
+ */
+
+void
+unconditional_life(int unconditional_territory[BOARDMAX], int color)
+{
+  int found_one;
+  int other = OTHER_COLOR(color);
+  int libs[MAXLIBS];
+  int liberties;
+  int pos;
+  int k, r;
+  int moves_played;
+  int potential_sekis[BOARDMAX];
+
+  /* Find isolated two-stone strings which might be involved in the
+   * kind of seki described in the comments.
+   */
+  memset(potential_sekis, 0, sizeof(potential_sekis));
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    int isolated = 1;
+    int stones[2];
+    int pos2;
+    
+    if (board[pos] != color
+	|| find_origin(pos) != pos
+	|| countstones(pos) != 2)
+      continue;
+    
+    findstones(pos, 2, stones);
+    for (k = 0; k < 2 && isolated; k++) {
+      for (r = 0; r < 8 && isolated; r++) {
+	pos2 = stones[k] + delta[r];
+	if (!ON_BOARD(pos2)
+	    || (board[pos2] == color
+		&& !same_string(pos, pos2)))
+	  isolated = 0;
+      }
+    }
+
+    if (isolated) {
+      potential_sekis[stones[0]] = 1;
+      potential_sekis[stones[1]] = 1;
+    }
+  }
   
-  /* The strings still remaining are uncapturable. Now see which
-   * opponent strings can survive.
+  moves_played = capture_non_invincible_strings(color, potential_sekis);
+  /* The strings still remaining except those marked in
+   * potential_sekis[] are uncapturable. Now see which opponent
+   * strings can survive.
    *
    * 1. Play opponent stones on all liberties of the unconditionally
    *    alive strings except where illegal.
    */
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    if (board[pos] != color || !is_worm_origin(pos, pos))
+    if (board[pos] != color || potential_sekis[pos] || find_origin(pos) != pos)
       continue;
       
     /* Play as many liberties as we can. */
     liberties = findlib(pos, MAXLIBS, libs);
     for (k = 0; k < liberties; k++) {
-      if (trymove(libs[k], other, "unconditional_life", pos, EMPTY, 0))
+      if (trymove(libs[k], other, "unconditional_life", pos))
 	moves_played++;
     }
   }
@@ -1450,13 +1721,76 @@ unconditional_life(int unconditional_territory[BOARDMAX], int color)
 	
       /* Try to extend the string at (m, n). */
       findlib(pos, 1, libs);
-      if (trymove(libs[0], other, "unconditional_life", pos, EMPTY, 0)) {
+      if (trymove(libs[0], other, "unconditional_life", pos)) {
 	moves_played++;
 	found_one = 1;
       }
     }
   }
 
+  /* Now see whether there are any significant sekis on the board. */
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (!potential_sekis[pos]
+	|| board[pos] == EMPTY
+	|| find_origin(pos) != pos)
+      continue;
+    for (r = 0; r < 4; r++) {
+      int up = delta[r];
+      int right = delta[(r + 1) % 4];
+      int locally_played_moves = 0;
+      if (board[pos + up] != color
+	  || board[pos + up + up] != EMPTY
+	  || board[pos - up] != EMPTY)
+	continue;
+      for (k = 0; k < 2; k++) {
+	if (k == 1)
+	  right = -right;
+	if (board[pos + right] != EMPTY || board[pos + up - right] != EMPTY)
+	  continue;
+	if (board[pos - right] == EMPTY
+	    && trymove(pos - right, other, "unconditional_life", pos))
+	  locally_played_moves++;
+	if (board[pos + up + right] == EMPTY
+	    && trymove(pos + up + right, other, "unconditional_life", pos))
+	  locally_played_moves++;
+	if (board[pos - right] == other && board[pos + up + right] == other
+	    && same_string(pos - right, pos + up + right)) {
+	  /* This is a critical seki. Extend the string with one stone
+           * in an arbitrary direction to break the seki.
+	   */
+	  while (locally_played_moves > 0) {
+	    popgo();
+	    locally_played_moves--;
+	  }
+	  trymove(pos - up, color, "unconditional_life", pos);
+	  moves_played++;
+	  break;
+	}
+	else {
+	  while (locally_played_moves > 0) {
+	    popgo();
+	    locally_played_moves--;
+	  }
+	}
+      }
+      if (countstones(pos) > 2)
+	break;
+    }
+  }
+
+  /* Capture the strings involved in potential sekis. */
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (!potential_sekis[pos] || board[pos] == EMPTY)
+      continue;
+    /* Play as many liberties as we can. */
+    liberties = findlib(pos, MAXLIBS, libs);
+    for (k = 0; k < liberties; k++) {
+      if (trymove(libs[k], other, "unconditional_life", pos))
+	moves_played++;
+    }
+  }
+    
+
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
     int apos;
     int bpos;
@@ -1489,11 +1823,11 @@ unconditional_life(int unconditional_territory[BOARDMAX], int color)
     blib  = approxlib(bpos, other, 4, NULL);
     
     if (aopen > bopen || (aopen == bopen && alib >= blib)) {
-      trymove(apos, other, "unconditional_life", pos, EMPTY, 0);
+      trymove(apos, other, "unconditional_life", pos);
       moves_played++;
     }
     else {
-      trymove(bpos, other, "unconditional_life", pos, EMPTY, 0);
+      trymove(bpos, other, "unconditional_life", pos);
       moves_played++;
     }
   }
@@ -1501,9 +1835,9 @@ unconditional_life(int unconditional_territory[BOARDMAX], int color)
   /* Identify unconditionally alive stones and unconditional territory. */
   memset(unconditional_territory, 0, sizeof(int) * BOARDMAX);
   for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    if (board[pos] == color) {
+    if (board[pos] == color && !potential_sekis[pos]) {
       unconditional_territory[pos] = 1;
-      if (is_worm_origin(pos, pos)) {
+      if (find_origin(pos) == pos) {
 	liberties = findlib(pos, MAXLIBS, libs);
 	for (k = 0; k < liberties; k++)
 	  unconditional_territory[libs[k]] = 2;
@@ -1531,7 +1865,7 @@ who_wins(int color, FILE *outfile)
 {
   float result;
 
-  silent_examine_position(WHITE, EXAMINE_DRAGONS);
+  silent_examine_position(EXAMINE_DRAGONS);
 
 #if 0
   float white_score;
@@ -2037,6 +2371,24 @@ time_report(int n, const char *occupation, int move, double mintime)
   return dt;
 }
 
+void
+clearstats()
+{
+  stats.nodes                    = 0;
+  stats.read_result_entered      = 0;
+  stats.read_result_hits         = 0;
+  stats.trusted_read_result_hits = 0;
+}
+  
+void
+showstats()
+{
+  gprintf("Nodes:                    %d\n", stats.nodes);
+  gprintf("Read results entered:     %d\n", stats.read_result_entered);
+  gprintf("Read result hits:         %d\n", stats.read_result_hits);
+  gprintf("Trusted read result hits: %d\n", stats.trusted_read_result_hits);
+}
+
 
 /*
  * Local Variables:
