@@ -1,5 +1,5 @@
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
- * This is GNU GO, a Go program. Contact gnugo@gnu.org, or see       *
+ * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
  * Copyright 1999, 2000, 2001, 2002 by the Free Software Foundation. *
@@ -54,7 +54,6 @@
 #include "gnugo.h"
 #include "sgftree.h"
 #include "interface.h"
-#include "random.h"
 
 #include "display.h"
 #include "gui.h"
@@ -71,7 +70,7 @@ Tab tabs[] = {
 
 
 Gameinfo  gameinfo;
-SGFNode  *sgf_root;
+SGFTree   sgftree;
 
 int       current_color = WHITE;
 
@@ -100,7 +99,8 @@ main(int argc, char *argv[])
 
   /* Try to open the infile. */
   infilename = argv[1];
-  if ((sgf_root = readsgffile(infilename)) == NULL) {
+  sgftree_clear(&sgftree);
+  if (!sgftree_readfile(&sgftree, infilename)) {
     fprintf(stderr, "Cannot open or parse '%s'\n", infilename);
     exit(1);
   }
@@ -112,24 +112,21 @@ main(int argc, char *argv[])
   /* FIXME: Want to do this somewhere more structured. */
   debug = 0;
   level = 10;
-  life  = 0;
-  life_eyesize = LIFE_EYESIZE;
   fusekidb = 1;
   josekidb = 1;
   
-  gameinfo_clear(&gameinfo, 19, 5.5);
-  next = gameinfo_play_sgftree(&gameinfo, sgf_root, until);
-
-
   /* Check if there is enough screen space. */
   if (!display_init(MINWIDTH, MINHEIGHT, board_size))
     exit(1);
 
   /* Initialize the GNU Go engine. */
-  gg_srand(time(0));
-  init_gnugo(8);
+  init_gnugo(8, time(0));
+
+  gameinfo_clear(&gameinfo, 19, 5.5);
+  next = gameinfo_play_sgftree(&gameinfo, &sgftree, until);
+
+
   reset_engine();
-  life = 1;
 
   display_message("Hit 'h' for help!");
 
