@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -61,7 +61,7 @@ static void show_debug_help(void);
 static void show_debug_flags(void);
 
 /* long options which have no short form */
-enum {OPT_BOARDSIZE=127,
+enum {OPT_BOARDSIZE = 127,
       OPT_HANDICAPSTONES,
       OPT_COLOR,
       OPT_KOMI,
@@ -74,6 +74,7 @@ enum {OPT_BOARDSIZE=127,
       OPT_OUTFILE, 
       OPT_QUIET,
       OPT_GTP_INPUT,
+      OPT_GTP_DUMP_COMMANDS,
       OPT_GTP_INITIAL_ORIENTATION,
       OPT_GTP_VERSION,
       OPT_SHOWCOPYRIGHT,
@@ -85,15 +86,17 @@ enum {OPT_BOARDSIZE=127,
       OPT_DECIDE_SEMEAI,
       OPT_DECIDE_SURROUNDED,
       OPT_DECIDE_TACTICAL_SEMEAI,
-      OPT_DECIDE_ORACLE,
       OPT_EXPERIMENTAL_SEMEAI,
-      OPT_EXPERIMENTAL_OWL_EXT,
       OPT_SEMEAI_NODE_LIMIT,
       OPT_EXPERIMENTAL_CONNECTIONS,
       OPT_EXPERIMENTAL_INFLUENCE,
       OPT_ALTERNATE_CONNECTIONS,
       OPT_WITH_BREAK_IN,
       OPT_WITHOUT_BREAK_IN,
+      OPT_COSMIC_GNUGO,
+      OPT_NO_COSMIC_GNUGO,
+      OPT_LARGE_SCALE,
+      OPT_NO_LARGE_SCALE,
       OPT_OPTIONS,
       OPT_STANDARD_SEMEAI,
       OPT_STANDARD_CONNECTIONS,
@@ -144,7 +147,7 @@ enum {OPT_BOARDSIZE=127,
 /* names of playing modes */
 
 enum mode {
-  MODE_UNKNOWN=0,
+  MODE_UNKNOWN = 0,
   MODE_ASCII,
   MODE_ASCII_EMACS,
   MODE_GTP,
@@ -165,8 +168,7 @@ enum mode {
   MODE_DECIDE_POSITION,
   MODE_DECIDE_EYE,
   MODE_DECIDE_COMBINATION,
-  MODE_DECIDE_SURROUNDED,
-  MODE_DECIDE_ORACLE
+  MODE_DECIDE_SURROUNDED
 };
 
 
@@ -184,6 +186,7 @@ static struct gg_option const long_options[] =
   {"quiet",          no_argument,       0, OPT_QUIET},
   {"silent",         no_argument,       0, OPT_QUIET},
   {"gtp-input",      required_argument, 0, OPT_GTP_INPUT},
+  {"gtp-dump-commands", required_argument, 0, OPT_GTP_DUMP_COMMANDS},
   {"orientation",    required_argument, 0, OPT_GTP_INITIAL_ORIENTATION},
   {"gtp-initial-orientation",
   		     required_argument, 0, OPT_GTP_INITIAL_ORIENTATION},
@@ -229,16 +232,17 @@ static struct gg_option const long_options[] =
   {"chinese-rules",  no_argument,       0, OPT_CHINESE_RULES},
   {"japanese-rules", no_argument,       0, OPT_JAPANESE_RULES},
   {"experimental-semeai",  no_argument, 0, OPT_EXPERIMENTAL_SEMEAI},
-  {"experimental-owl-ext",  no_argument, 0, OPT_EXPERIMENTAL_OWL_EXT},
   {"semeai-node-limit",   required_argument, 0, OPT_SEMEAI_NODE_LIMIT},
   {"experimental-connections",  no_argument, 0, OPT_EXPERIMENTAL_CONNECTIONS},
-  {"owl-threats",     no_argument,      0, OPT_OWL_THREATS},
-  {"no-owl-threats",  no_argument,      0, OPT_NO_OWL_THREATS},
   {"standard-connections",  no_argument, 0, OPT_STANDARD_CONNECTIONS},
   {"standard-semeai", no_argument,      0, OPT_STANDARD_SEMEAI},
   {"alternate-connections",  no_argument, 0, OPT_ALTERNATE_CONNECTIONS},
   {"with-break-in",  	no_argument, 0, OPT_WITH_BREAK_IN},
   {"without-break-in",  no_argument, 0, OPT_WITHOUT_BREAK_IN},
+  {"cosmic-gnugo",   no_argument, 0, OPT_COSMIC_GNUGO},
+  {"no-cosmic-gnugo",   no_argument, 0, OPT_NO_COSMIC_GNUGO},
+  {"large-scale",    no_argument, 0, OPT_LARGE_SCALE},
+  {"no-large-scale",    no_argument, 0, OPT_NO_LARGE_SCALE},
   {"options",        no_argument, 0, OPT_OPTIONS},
   {"allow-suicide",  no_argument,       0, OPT_ALLOW_SUICIDE},
   {"capture-all-dead",   no_argument,   0, OPT_CAPTURE_ALL_DEAD},
@@ -262,7 +266,6 @@ static struct gg_option const long_options[] =
   {"decide-surrounded",  required_argument, 0, OPT_DECIDE_SURROUNDED},
   {"decide-eye",     required_argument, 0, OPT_DECIDE_EYE},
   {"decide-combination", no_argument,   0, OPT_DECIDE_COMBINATION},
-  {"decide-oracle",  no_argument,       0, OPT_DECIDE_ORACLE},
   {"nofusekidb",     no_argument,       0, OPT_NOFUSEKIDB},
   {"nofuseki",       no_argument,       0, OPT_NOFUSEKI},
   {"nojosekidb",     no_argument,       0, OPT_NOJOSEKIDB},
@@ -300,6 +303,7 @@ main(int argc, char *argv[])
   char *outfile = NULL;
   char *outflags = NULL;
   char *gtpfile = NULL;
+  char *gtp_dump_commands_file = NULL;
   
   char *printsgffile = NULL;
   
@@ -308,7 +312,9 @@ main(int argc, char *argv[])
   char debuginfluence_move[4] = "\0";
   
   int benchmark = 0;  /* benchmarking mode (-b) */
-  FILE *gtp_input_FILE, *output_check;
+  FILE *gtp_input_FILE;
+  FILE *gtp_dump_commands_FILE = NULL;
+  FILE *output_check;
   int orientation = 0;
 
   float memory = (float) DEFAULT_MEMORY; /* Megabytes used for hash table. */
@@ -352,15 +358,15 @@ main(int argc, char *argv[])
     owl_threats = 1;
   else
     owl_threats = 0;
-  experimental_owl_ext = EXPERIMENTAL_OWL_EXT;
-  experimental_semeai = EXPERIMENTAL_SEMEAI;
   experimental_connections = EXPERIMENTAL_CONNECTIONS;
+  cosmic_gnugo = COSMIC_GNUGO;
+  large_scale = LARGE_SCALE;
+  resign_allowed = RESIGNATION_ALLOWED;
 
   allow_suicide = 0;
   capture_all_dead = 0;
   play_out_aftermath = 0;
   limit_search = 0;
-  resign_allowed = RESIGNATION_ALLOWED;
 
   /* Default parameters for clock and auto level systems. */
   clock_init(3600, 0, 0);      /* One hour sudden death. */
@@ -413,7 +419,7 @@ main(int argc, char *argv[])
 	outflags = gg_optarg;
 	output_flags = 0;
 	if (outflags)
-	  while (*outflags){
+	  while (*outflags) {
 	    switch (*outflags) {
 	    case 'd':
 	      output_flags |= OUTPUT_MARKDRAGONS;
@@ -434,6 +440,10 @@ main(int argc, char *argv[])
 	gtpfile = gg_optarg;
 	break;
 	
+      case OPT_GTP_DUMP_COMMANDS:
+	gtp_dump_commands_file = gg_optarg;
+	break;
+	
       case OPT_GTP_INITIAL_ORIENTATION:
 	orientation = atoi(gg_optarg);
 	if (orientation < 0 || orientation > 7) {
@@ -449,35 +459,41 @@ main(int argc, char *argv[])
 	
       case OPT_OPTIONS:
 	if (USE_BREAK_IN)
-	  fprintf(stderr,
+	  fprintf(stdout,
 		  "configure option enabled: experimental break-ins\n");
 	if (COSMIC_GNUGO)
-	  fprintf(stderr,
+	  fprintf(stdout,
 		  "configure option enabled: cosmic GNU Go \n");
+	if (LARGE_SCALE)
+	  fprintf(stdout,
+		  "configure option enabled: large scale captures \n");
 	if (EXPERIMENTAL_CONNECTIONS)
-	  fprintf(stderr,
+	  fprintf(stdout,
 		  "configure option enabled: experimental connections\n");
 	if (ALTERNATE_CONNECTIONS)
-	  fprintf(stderr,
+	  fprintf(stdout,
 		  "configure option enabled: alternate connections\n");
-	if (EXPERIMENTAL_SEMEAI)
-	  fprintf(stderr,
-		  "configure option enabled: experimental semeai\n");
 	if (EXPERIMENTAL_OWL_EXT)
-	  fprintf(stderr,
+	  fprintf(stdout,
 		  "configure option enabled: experimental GAIN/LOSS codes\n");
 	if (EXPERIMENTAL_READING)
-	  fprintf(stderr,
+	  fprintf(stdout,
 		  "configure option enabled: experimental reading\n");
-	if (HASHING_SCHEME != 2)
-	  fprintf(stderr, "hash scheme %d\n", HASHING_SCHEME);
 	if (OWL_THREATS)
-	  fprintf(stderr,
+	  fprintf(stdout,
 		  "configure option enabled: owl threats\n");
-	fprintf(stderr,
+	if (RESIGNATION_ALLOWED)
+	  fprintf(stdout,
+		  "configure option enabled: resignation allowed\n");
+	if (ORACLE)
+	  fprintf(stdout,
+		  "configure option enabled: metamachine\n");
+	fprintf(stdout,
 		"Owl node limit: %d\n", OWL_NODE_LIMIT);
-	fprintf(stderr,
+	fprintf(stdout,
 		"Semeai node limit: %d\n", SEMEAI_NODE_LIMIT);
+	fprintf(stdout,
+		"Cache size: %d MB\n", DEFAULT_MEMORY);
 
 	return EXIT_SUCCESS;
 	break;
@@ -544,22 +560,10 @@ main(int argc, char *argv[])
 	chinese_rules = 0;
 	break;
 
-      case OPT_EXPERIMENTAL_OWL_EXT:
-	experimental_owl_ext = 1;
-	break;
-
-      case OPT_EXPERIMENTAL_SEMEAI:
-	experimental_semeai = 1;
-	break;
-
       case OPT_SEMEAI_NODE_LIMIT:
 	semeai_node_limit = atoi(gg_optarg);
 	break;
 
-      case OPT_STANDARD_SEMEAI: 
-	experimental_semeai = 0;
-	break;
-
       case OPT_EXPERIMENTAL_CONNECTIONS:
 	experimental_connections = 1;
 	break;
@@ -580,6 +584,22 @@ main(int argc, char *argv[])
 	experimental_break_in = 0;
 	break;
 
+      case OPT_COSMIC_GNUGO:
+	cosmic_gnugo = 1;
+	break;
+
+      case OPT_NO_COSMIC_GNUGO:
+	cosmic_gnugo = 0;
+	break;
+
+      case OPT_LARGE_SCALE:
+	large_scale = 1;
+	break;
+
+      case OPT_NO_LARGE_SCALE:
+	large_scale = 0;
+	break;
+
       case OPT_ALLOW_SUICIDE:
 	allow_suicide = 1;
 	break;
@@ -601,7 +621,7 @@ main(int argc, char *argv[])
 	break;
 
       case OPT_MODE: 
-	if (strcmp(gg_optarg, "ascii") == 0) 
+	if (strcmp(gg_optarg, "ascii") == 0)
 	  playmode = MODE_ASCII;
 	else if (strcmp(gg_optarg, "emacs") == 0)
 	  playmode = MODE_ASCII_EMACS;
@@ -725,10 +745,6 @@ main(int argc, char *argv[])
 	playmode = MODE_DECIDE_SURROUNDED;
 	break;
 	
-      case OPT_DECIDE_ORACLE:
-	playmode = MODE_DECIDE_ORACLE;
-	break;
-	
       case OPT_BRANCH_DEPTH:
 	mandated_branch_depth = atoi(gg_optarg);
 	break;
@@ -779,6 +795,10 @@ main(int argc, char *argv[])
 	
       case OPT_LEVEL:
 	level = atoi(gg_optarg);
+	if (level > max_level)
+	  max_level = level;
+	if (level < min_level)
+	  min_level = level;
 	break;
 
       case OPT_MIN_LEVEL:
@@ -902,6 +922,7 @@ main(int argc, char *argv[])
 
       case 'v':
 	show_version();
+	show_copyright();
 	return EXIT_SUCCESS;
 	break;
 	
@@ -932,17 +953,17 @@ main(int argc, char *argv[])
 	return EXIT_SUCCESS;
 	break;
 
-	/* NOTE: getopt returns '?' if an illegal option is supplied. */
-
       case OPT_PRINT_LEVELS:
 	{
 	  int lev;
-	  for(lev = 10; lev > 0; lev--)
+	  for (lev = 10; lev > 0; lev--)
 	    set_depth_values(lev, 1);
 	}
 	return EXIT_SUCCESS;
 	break;
 
+      /* NOTE: getopt returns '?' if an illegal option is supplied. */
+
       case '?':
       default:
 	fprintf(stderr, "Try `gnugo --help' for more information.\n");
@@ -950,14 +971,6 @@ main(int argc, char *argv[])
       }
     }
 
-
-  /* Display copyright message unless --quiet option used. */
-  if (!quiet) {
-    fprintf(stderr, "\n");
-    show_version();
-    show_copyright();
-  }
-  
   /* Start random number seed. */
   if (!seed_specified)
     seed = time(0);
@@ -1243,7 +1256,7 @@ main(int argc, char *argv[])
       color = gameinfo.to_move;
       if (mandated_color != EMPTY)
 	color = mandated_color;
-      decide_position(color);
+      decide_position();
     }
     break;
     
@@ -1295,22 +1308,6 @@ main(int argc, char *argv[])
       break;
     }
 
-#if ORACLE
-  case MODE_DECIDE_ORACLE:
-    {
-      if (mandated_color != EMPTY)
-	gameinfo.to_move = mandated_color;
-      
-      if (!infilename) {
-	fprintf(stderr, "You must use -l infile with load and analyze mode.\n");
-	exit(EXIT_FAILURE);
-      }
-
-      decide_oracle(&gameinfo, infilename, untilstring);
-      break;
-    }
-#endif
-
   case MODE_GTP:  
     if (gtpfile != NULL) {
       gtp_input_FILE = fopen(gtpfile, "r");
@@ -1322,7 +1319,18 @@ main(int argc, char *argv[])
     else
       gtp_input_FILE = stdin;
 
-    play_gtp(gtp_input_FILE, orientation);
+    if (gtp_dump_commands_file != NULL) {
+      gtp_dump_commands_FILE = fopen(gtp_dump_commands_file, "w");
+      if (gtp_dump_commands_FILE == NULL) {
+	fprintf(stderr, "gnugo: Cannot open file %s\n",
+		gtp_dump_commands_file);
+	return EXIT_FAILURE;
+      } 
+    }
+
+    play_gtp(gtp_input_FILE, gtp_dump_commands_FILE, orientation);
+    if (gtp_dump_commands_FILE)
+      fclose(gtp_dump_commands_FILE);
     break;
 
   case MODE_ASCII_EMACS:  
@@ -1337,14 +1345,37 @@ main(int argc, char *argv[])
     if (mandated_color != EMPTY)
       gameinfo.computer_player = OTHER_COLOR(mandated_color);
 
+  /* Display copyright message in ASCII mode unless --quiet option used. */
+    if (!quiet) {
+      show_version();
+      show_copyright();
+    }
+  
+#if ORACLE
+    if (metamachine) {
+      summon_oracle();
+      oracle_loadsgf(infilename, untilstring);
+    }
+#endif
+
     play_ascii(&sgftree, &gameinfo, infilename, untilstring);
+
+#if ORACLE
+    dismiss_oracle();
+#endif
+
     break;
   }
   
+#if ORACLE
+  dismiss_oracle();
+#endif
+
   if (profile_patterns)
     report_pattern_profiling();
 
   clock_report_autolevel(NULL, gameinfo.computer_player);
+  sgfFreeNode(gameinfo.game_record.root);
  
   return 0;
 }  /* end main */
@@ -1354,7 +1385,7 @@ main(int argc, char *argv[])
 static void
 show_version(void)
 {
-  fprintf(stderr, "GNU Go Version %s\n", VERSION);
+  printf("GNU Go %s\n", VERSION);
 }
 
 
@@ -1373,6 +1404,13 @@ show_version(void)
 
 #define USAGE "\n\
 Usage: gnugo [-opts]\n\
+Examples:\
+  gnugo --mode gtp --level 5\n\
+         To play against gnugo in level 5 from a GTP client\n\
+  gnugo --mode ascii -l game.sgf -L 123\n\
+         Resume game at move 123 in ASCII mode\n\
+  gnugo --score estimate -l game.sgf\n\
+         Give a rough score estimate of the end position in game.sgf\n\
 \n\
 Main Options:\n\
        --mode <mode>     Force the playing mode ('ascii', 'gmp', 'sgmp',\n\
@@ -1380,29 +1418,15 @@ Main Options:\n\
                          If no terminal is detected GMP (Go Modem Protocol)\n\
                          will be assumed.\n\
        --quiet           Don't print copyright and informational messages\n\
-       --level <amount>  strength (default %d, up to 10 supported)\n\
+       --level <amount>  strength (default %d)\n\
+       --never-resign    Forbid GNU Go to resign\n\
+       --resign-allowed	 Allow resignation (default)\n\
        --gtp-input <file>Read gtp commands from file instead of stdin\n\
    -l, --infile <file>   Load name sgf file\n\
    -L, --until <move>    Stop loading just before move is played. <move>\n\
                          can be the move number or location (eg L10).\n\
    -o, --outfile <file>  Write sgf output to file\n\
    --printsgf <file>     Write position as a diagram to file (use with -l)\n\
-\n\
-Other options affecting strength (higher = stronger, slower):\n\
-(Unless you are working on the program --level <n> is all you need.)\n\
-   -D, --depth <depth>          deep reading cutoff (default %d)\n\
-   -B, --backfill-depth <depth> deep reading cutoff (default %d)\n\
-   -F, --fourlib-depth <depth>  deep reading cutoff (default %d)\n\
-   -K, --ko-depth <depth>       deep reading cutoff (default %d)\n\
-   --branch-depth <depth>       deep reading cutoff (default %d)\n\
-   --backfill2-depth <depth>    deep reading cutoff (default %d)\n\
-   --break_chain-depth <depth>  deep reading cutoff (default %d)\n\
-   --superstring-depth <depth>  deep reading cutoff (default %d)\n\
-   --aa-depth <depth>           deep reading cutoff (default %d)\n\
-   --owl-distrust <depth>       owl distrust depth (default %d)\n\
-   --owl-branch <depth>         owl branching depth (default %d)\n\
-   --owl-reading <depth>        owl reading depth (default %d)\n\
-   --owl-node-limit <limit>     max nodes for owl reading (default %d)\n\
 "
 
 #define USAGE1 "\
@@ -1413,6 +1437,10 @@ Other options affecting strength (higher = stronger, slower):\n\
 Experimental options:\n\
    --with-break-in         use the break-in code (on at level 10 by default)\n\
    --without-break-in      do not use the break-in code\n\
+   --cosmic-gnugo          use center oriented influence\n\
+   --no-cosmic-gnugo       don't use center oriented influence (default)\n\
+   --large-scale           look for large scale captures\n\
+   --no-large-scale        don't seek large scale captures (default)\n\
    --nofusekidb            turn off fuseki database\n\
    --nofuseki              turn off fuseki moves entirely\n\
    --nojosekidb            turn off joseki database\n\
@@ -1427,6 +1455,9 @@ Scoring:\n\
 Cache size (higher=more memory usage, faster unless swapping occurs):\n\
    -M, --cache-size <megabytes>  RAM cache for hashing (default %4.1f Mb)\n\
 \n\
+"
+
+#define USAGE2 "\
 Game Options: (--mode ascii)\n\
        --boardsize num   Set the board size to use (%d--%d)\n\
        --color <color>   Choose your color ('black' or 'white')\n\
@@ -1437,7 +1468,7 @@ Game Options: (--mode ascii)\n\
        --byo-period <stones>  Initialize the byo-yomi period.\n\
 \n\
 Informative Output:\n\
-   -v, --version         Display the version of GNU Go\n\
+   -v, --version         Display the version and copyright of GNU Go\n\
    --options             Display configure options\n\
    -h, --help            Display this help message\n\
        --help debug      Display help about debugging options\n\
@@ -1446,24 +1477,14 @@ Informative Output:\n\
 "
 
 
-#define COPYRIGHT "\n\n\
-This is GNU Go, a Go program. Contact gnugo@gnu.org, or see\n\
-http://www.gnu.org/software/gnugo/ for more information.\n\n\
-Copyright 1999, 2000, 2001, 2002 and 2003 by\n\
-the Free Software Foundation.\n\n\
-This program is free software; you can redistribute it and/or\n\
-modify it under the terms of the GNU General Public License\n\
-as published by the Free Software Foundation - version 2.\n\n\
-This program is distributed in the hope that it will be\n\
-useful, but WITHOUT ANY WARRANTY; without even the implied\n\
-warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR\n\
-PURPOSE.  See the GNU General Public License in file COPYING\n\
-for more details.\n\n\
-You should have received a copy of the GNU General Public\n\
-License along with this program; if not, write to the Free\n\
-Software Foundation, Inc., 59 Temple Place - Suite 330,\n\
-Boston, MA 02111, USA.\n\n\
-"
+#define COPYRIGHT \
+"Copyright 1999, 2000, 2001, 2002, 2003 and 2004 by the Free Software\n\
+Foundation, Inc. See http://www.gnu.org/software/gnugo/ or contact\n\
+gnugo@gnu.org for information about GNU Go. GNU Go comes with NO WARRANTY to\n\
+the extent permitted by law. This program is free software; you can\n\
+redistribute it and/or modify it under the terms of the GNU General Public\n\
+License as published by the Free Software Foundation - version 2.  For more\n\
+information about these matters, see the files named COPYING.\n"
 
 /* USAGE_DEBUG Split in half because of VC limit on constant string 
  * length of 2048 characters!*/
@@ -1479,38 +1500,51 @@ Debugging Options:\n\
    -T, --printboard              colored display of dragons\n\
    -E, --printeyes               colored display of eye spaces\n\
    -d, --debug <flags>           debugging output (see next item for bits)\n\
-   --debug-flags                 print the debug flags for previous item\n\
+       --debug-flags             print the debug flags for previous item\n\
    -H, --hash <level>            hash (see gnugo.h for bits)\n\
    -w, --worms                   print worm and dragon data and move reasons\n\
    -m, --moyo <level>            moyo debugging, show moyo board\n\
+       --debuginfluence <move>   print influence map after making a move\n\
    -b, --benchmark num           benchmarking mode - can be used with -l\n\
    -S, --statistics              print statistics (for debugging purposes)\n\n\
+       --profile-patterns        print statistics for pattern usage\n\
+       --showtime                print timing diagnostic\n\
    -t, --trace                   verbose tracing\n\
    -O, --output-flags <flags>    optional output (use with -o)\n\
                     d: mark dead and critical dragons\n\
                     v: show values of considered moves\n\
                     specify either 'd', 'v' or 'dv' (nothing by default)\n\
-   --showtime                    print timing diagnostic\n\
-   --replay <color>              replay game. Use with -o.\n\
-   --showscore                   print estimated score\n\
+       --showscore               print estimated score\n\
    -r, --seed number             set random number seed\n\
+       --gtp-dump-commands <file>dump commands received in GTP mode\n\
 "
+
 #define USAGE_DEBUG2 "\
-       --decide-string <string>  can this string live? (try with -o)\n\
-       --decide-connection <str/str> can these strings connect? (try with -o)\n\
-       --decide-dragon <dragon>  can this dragon live? (try with -o or -t)\n\
-       --decide-position         evaluate all dragons (try with -o or -t)\n\
-       --decide-eye <string>     evaluate the eye\n\
-       --decide-combination      search for combination attack (try with -o)\n\
-       --genmove <color>         generate a move for color\n\
-       --nofusekidb              turn off fuseki database\n\
-       --nofuseki                turn off fuseki moves entirely\n\
-       --nojosekidb              turn off joseki database\n\
-       --debuginfluence <move>   print influence map after making a move\n\
-       --score <end|last|move>   count or estimate territory\n\
-       --profile-patterns        print statistics for pattern usage\n\
-       --attack-by-pattern       use pattern-based tactical reading for attack\n\
-       --defend-by-pattern       use pattern-based tactical reading for defense\n\
+Options affecting depth settings and playing strength:\n\
+   -D, --depth <depth>          deep reading cutoff (default %d)\n\
+   -B, --backfill-depth <depth> deep reading cutoff (default %d)\n\
+   -F, --fourlib-depth <depth>  deep reading cutoff (default %d)\n\
+   -K, --ko-depth <depth>       deep reading cutoff (default %d)\n\
+   --branch-depth <depth>       deep reading cutoff (default %d)\n\
+   --backfill2-depth <depth>    deep reading cutoff (default %d)\n\
+   --break_chain-depth <depth>  deep reading cutoff (default %d)\n\
+   --superstring-depth <depth>  deep reading cutoff (default %d)\n\
+   --aa-depth <depth>           deep reading cutoff (default %d)\n\
+   --owl-distrust <depth>       owl distrust depth (default %d)\n\
+   --owl-branch <depth>         owl branching depth (default %d)\n\
+   --owl-reading <depth>        owl reading depth (default %d)\n\
+   --owl-node-limit <limit>     max nodes for owl reading (default %d)\n\
+Options providing detailed reading results etc.:\n\
+   --decide-string <string>     can this string live? (try with -o)\n\
+   --decide-connection <str/str> can these strings connect? (try with -o)\n\
+   --decide-dragon <dragon>     can this dragon live? (try with -o or -t)\n\
+   --decide-position            evaluate all dragons (try with -o or -t)\n\
+   --decide-eye <string>        evaluate the eye\n\
+   --decide-combination         search for combination attack (try with -o)\n\
+   --genmove <color>            generate a move for color\n\
+Other options:\n\
+   --attack-by-pattern          use pattern-based tactical reading for attack\n\
+   --defend-by-pattern          use pattern-based tactical reading for defense\n\
 "
 
 #define DEBUG_FLAGS "\
@@ -1539,6 +1573,7 @@ DEBUG_OWL_PERSISTENT_CACHE  0X200000\n\
 DEBUG_TOP_MOVES             0x400000\n\
 DEBUG_MISCELLANEOUS         0x800000\n\
 DEBUG_ORACLE_STREAM         0x1000000\n\
+DEBUG_LARGE_SCALE           0x1000000\n\
 "
 
 /*
@@ -1549,37 +1584,35 @@ static void
 show_help(void)
 {
   set_depth_values(DEFAULT_LEVEL, 0);
-  fprintf(stderr, USAGE, level,
-	  depth, backfill_depth, fourlib_depth, ko_depth, branch_depth,
-	  backfill2_depth, break_chain_depth, superstring_depth, aa_depth, 
-	  owl_distrust_depth, owl_branch_depth,
-	  owl_reading_depth, owl_node_limit);
-  fprintf(stderr, USAGE1,
-	  (float) DEFAULT_MEMORY, MIN_BOARD, MAX_BOARD, MAX_HANDICAP);
+  printf(USAGE, level);
+  printf(USAGE1, (float) DEFAULT_MEMORY);
+  printf(USAGE2, MIN_BOARD, MAX_BOARD, MAX_HANDICAP);
 }
 
 
 static void
 show_debug_help(void)
 {
-  fprintf(stderr, USAGE_DEBUG USAGE_DEBUG2);
+  printf(USAGE_DEBUG USAGE_DEBUG2, 
+	 depth, backfill_depth, fourlib_depth, ko_depth, branch_depth,
+	 backfill2_depth, break_chain_depth, superstring_depth, aa_depth, 
+	 owl_distrust_depth, owl_branch_depth,
+	 owl_reading_depth, owl_node_limit);
 }
 
 static void 
 show_debug_flags(void)
 {
-  fprintf(stderr, DEBUG_FLAGS);
+  printf(DEBUG_FLAGS);
 }
 
 static void
 show_copyright(void)
 {
-  fputs(COPYRIGHT, stderr);
+  printf(COPYRIGHT);
 }
 
 
-
-
 /*
  * Local Variables:
  * tab-width: 8
