@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -224,7 +224,7 @@ ascii_showboard(void)
 	pos_is_move = 128;
       else
 	pos_is_move = 0;
-      dead = (dragon_status(POS(i, j))==DEAD) && showdead;
+      dead = (dragon_status(POS(i, j)) == DEAD) && showdead;
       switch (BOARD(i, j) + pos_is_move + last_pos_was_move) {
 	case EMPTY+128:
 	case EMPTY:
@@ -329,7 +329,7 @@ show_commands(void)
   printf("\n");
 }
 
-enum commands {INVALID=-1, END, EXIT, QUIT, RESIGN, 
+enum commands {INVALID = -1, END, EXIT, QUIT, RESIGN, 
 	       PASS, MOVE, FORCE, SWITCH,
 	       PLAY, PLAYBLACK, PLAYWHITE,
 	       SETHANDICAP, FREEHANDICAP, SETBOARDSIZE, SETKOMI,
@@ -446,6 +446,7 @@ computer_move(Gameinfo *gameinfo, int *passes)
   int i, j;
   int move_val;
   int resignation_declined = 0;
+  float upper_bound, lower_bound;
 
   init_sgf(gameinfo);
 
@@ -579,11 +580,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
   char *tmpstring;
   int state = 1;
   
-#ifdef HAVE_SETLINEBUF
-  setlinebuf(stdout); /* Need at least line buffer gnugo-gnugo */
-#else
-  setbuf(stdout, NULL); /* else set it to completely UNBUFFERED */
-#endif
+  setvbuf(stdout, (char *)NULL, _IONBF, 0); /* No buffering. */
   
   sgftree = *tree;
 
@@ -635,11 +632,11 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
       line_ptr = line;
       if (!fgets(line, 80, stdin)) {
 	printf("\nThanks! for playing GNU Go.\n\n");
-	return ;
+	return;
       }
 
-      while (state == 0 &&
-	     (command = strtok(line_ptr, ";"), line_ptr = 0, command)) {
+      while (state == 0
+	     && (command = strtok(line_ptr, ";"), line_ptr = 0, command)) {
 	/* Get the command or move. */
 	switch (get_command(command)) {
 	case RESIGN:
@@ -880,7 +877,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  break;
 
 	case CMD_DEAD:
-	  examine_position(gameinfo->to_move, FULL_EXAMINE_DRAGONS);
+	  examine_position(FULL_EXAMINE_DRAGONS);
 	  showdead = !showdead;
 	  break;
 
@@ -897,7 +894,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	case CMD_SHOWMOYO:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_MOYO;
-	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
+	  examine_position(EXAMINE_DRAGONS);
 	  print_moyo();
 	  printmoyo = tmp;
 	  break;
@@ -905,7 +902,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	case CMD_SHOWTERRI:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_TERRITORY;
-	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
+	  examine_position(EXAMINE_DRAGONS);
 	  print_moyo();
 	  printmoyo = tmp;
 	  break;
@@ -913,13 +910,13 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	case CMD_SHOWAREA:
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_AREA;
-	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
+	  examine_position(EXAMINE_DRAGONS);
 	  print_moyo();
 	  printmoyo = tmp;
 	  break;
 
 	case CMD_SHOWDRAGONS:
-	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
+	  examine_position(EXAMINE_DRAGONS);
 	  showboard(1);
 	  break;
 
@@ -966,7 +963,7 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  break;
 
 	case CMD_LISTDRAGONS:
-	  examine_position(gameinfo->to_move, EXAMINE_DRAGONS);
+	  examine_position(EXAMINE_DRAGONS);
 	  show_dragons();
 	  break;
 
@@ -1273,7 +1270,7 @@ ascii_free_handicap(Gameinfo *gameinfo, char *handicap)
 	  break;
       }
       else if (string_to_location(board_size, line, &x, &y)) {
-	pos = POS(x,y);
+	pos = POS(x, y);
 	if (board[pos] != EMPTY)
 	  printf("\nThere's already a stone there.\n");
 	else {
