@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -69,10 +69,10 @@
 #include <stdio.h>
 #include <string.h>
 #include <limits.h>
-#include "../sgf/sgftree.h"
 #include "liberty.h"
 #include "gg_utils.h"
 #include "random.h"
+#include "../sgf/sgftree.h"
 
 #define USAGE "\n\
 Usage: extract_fuseki files boardsize moves patterns handicap strength half_board [output file]\n\
@@ -565,14 +565,14 @@ store_pattern_if_winner(struct invariant_hash *pre,
 	    move_number++;
 	  }
 	  else { /* something is wrong */
-	    fprintf(stderr, "Error in store_pattern_if_winner: %d\n",k);
+	    fprintf(stderr, "Error in store_pattern_if_winner: %d\n", k);
 	    winning_moves[k].pattern[i][j] = '.';
 	  }
 	}
       winning_moves[k].pattern[m][n] = '*';
       /* add ? in areas far away from the move */
       if (half_board_patterns == 1 && move_number > 3 && board_size == 19)
-        region = find_region(m,n);
+        region = find_region(m, n);
       if (region != 8) {
         for (i = 0; i < board_size; i++) {
           for (j = 0; j < board_size; j++) {
@@ -641,7 +641,7 @@ examine_game(SGFNode *sgf, int collect_statistics)
       if (k > 0) {
 	/* something is wrong with the file */
 	if (0)
-	  fprintf(stderr, "move number:%d\n",k);
+	  fprintf(stderr, "move number:%d\n", k);
 	return 0;
       }
       continue;
@@ -1119,7 +1119,7 @@ main(int argc, char *argv[])
   
   /* Check number of arguments. */
   if (argc < 6) {
-    fprintf(stderr, USAGE);
+    printf(USAGE);
     exit(EXIT_FAILURE);
   }
   
@@ -1162,7 +1162,7 @@ main(int argc, char *argv[])
   number_of_games = read_sgf_filenames(argv[1], NULL);
   
   /* Allocate space for the list of unused files. */
-  unused_games = calloc(number_of_games, sizeof(int));
+  unused_games = calloc(number_of_games, sizeof(*unused_games));
   if (unused_games == NULL) {
     fprintf(stderr, "Fatal error, failed to allocate memory.\n");
     exit(EXIT_FAILURE);
@@ -1204,7 +1204,9 @@ main(int argc, char *argv[])
      */
     generate_patterns();
     fprintf(stderr, "generate OK.\n");
-    
+
+    printf("attribute_map value_only\n\n\n");
+
     /* Write the patterns to stdout in patterns.db format.
      */
     print_patterns();
