@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -26,13 +26,13 @@
  * describing the patterns.
  */
 
+#ifndef _PATTERN_H_
+#define _PATTERN_H_
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#ifndef __PATTERN_H__
-#define __PATTERN_H__
-
 /* local versions of absolute value, min and max */
 
 #define gg_abs(x) ((x) < 0 ? -(x) : (x))
@@ -46,15 +46,11 @@
 #define _NO_PROTO
 #endif
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#else
-#define GRID_OPT 0
-#endif
+/* =================================================================== */
+/*  Configure option that has been removed from the stable version.    */
+/* =================================================================== */
 
-#ifndef GRID_OPT
-#error GRID_OPT should be defined as 0, 1 or 2
-#endif
+#define GRID_OPT 1
 
 
 /* Include support for pattern profiling. May be turned off in stable
@@ -70,9 +66,8 @@ struct _unused_patterns_h {
 };
 
 
-#define ATTACK_MACRO(pos) ((stackp==0) ? (worm[pos].attack_codes[0]) : attack(pos, NULL))
-#define DEFEND_MACRO(pos) ((stackp==0) ? (worm[pos].defense_codes[0]) : find_defense(pos, NULL))
-#define DRAGON_WEAK(pos) dragon_weak(pos)
+#define ATTACK_MACRO(pos) ((stackp == 0) ? (worm[pos].attack_codes[0]) : attack(pos, NULL))
+#define DEFEND_MACRO(pos) ((stackp == 0) ? (worm[pos].defense_codes[0]) : find_defense(pos, NULL))
 
 struct pattern; /* forward reference to keep gcc happy */
 
@@ -126,8 +121,9 @@ typedef int (*autohelper_fn_ptr)(int rotation, int move,
 #define CLASS_s     0x0010   /* move is a sacrifice */
 #define CLASS_n     0x0020   /* X could also make this move if we do not */
 #define CLASS_D     0x0040   /* defense pattern */
-#define CLASS_C     0x0080   /* move connects two worms */ 
+#define CLASS_C     0x0080   /* move connects two worms */
 #define CLASS_c     0x0100   /* move weakly connects two worms */ 
+			     /* for owl databases: combinable pattern */
 #define CLASS_B     0x0200   /* move breaks connection between enemy worms */
 #define CLASS_A     0x0400   /* attack pattern */
 #define CLASS_b     0x0800   /* move is intended to block opponent */
@@ -152,20 +148,6 @@ typedef int (*autohelper_fn_ptr)(int rotation, int move,
 			    CLASS_J | CLASS_j | CLASS_U | CLASS_T | CLASS_t | \
                             CLASS_W | CLASS_c | CLASS_F)
 
-/* Values associated with patterns. Stored together with classes. */
-#define VALUE_MINVAL       0x01000000 /* pattern has a minimum value */
-#define VALUE_MAXVAL       0x02000000 /* pattern has a maximum value */
-#define VALUE_MINTERRITORY 0x04000000 /* pattern has a min territorial value */
-#define VALUE_MAXTERRITORY 0x08000000 /* pattern has a max territorial value */
-#define VALUE_SHAPE        0x10000000 /* pattern has a shape value */
-#define VALUE_FOLLOWUP     0x20000000 /* pattern has a followup value */
-#define VALUE_REV_FOLLOWUP 0x40000000 /* pattern has a reverse followup value */
-
-/* Collection of the classes inducing move values. */
-#define CLASS_MOVE_VALUES (VALUE_MINVAL | VALUE_MAXVAL | VALUE_MINTERRITORY \
-			   | VALUE_MAXTERRITORY | VALUE_SHAPE \
-			   | VALUE_FOLLOWUP | VALUE_REV_FOLLOWUP)
-
 /* directions for applying edge-constraints */
 #define NORTH_EDGE 1
 #define SOUTH_EDGE 2
@@ -196,10 +178,53 @@ typedef struct patval_b {
 } Patval_b;
 
 
+enum attribute_type {
+  MIN_VALUE,
+  MAX_VALUE,
+  MIN_TERRITORY,
+  MAX_TERRITORY,
+  SHAPE,
+  FOLLOWUP,
+  REVERSE_FOLLOWUP,
+
+  /* For `mkpat'. */
+  FIRST_OFFSET_ATTRIBUTE,
+
+  THREATENS_TO_CAPTURE = FIRST_OFFSET_ATTRIBUTE,
+  THREATENS_EYE,
+  REVERSE_SENTE,
+
+  NUM_ATTRIBUTES,
+  LAST_ATTRIBUTE = NUM_ATTRIBUTES
+};
+
+
+#ifdef HAVE_TRANSPARENT_UNIONS
+
+struct pattern_attribute {
+  enum attribute_type type;
+
+  /* GCC allows unnamed (and transparent) unions. */
+  union {
+    float value;
+    int offset;
+  };
+};
+
+#else
+
+struct pattern_attribute {
+  enum attribute_type type;
+  float value;
+  int offset;
+};
+
+#endif
+
+
 /*
  * Each pattern as a whole is compiled to an instance of this structure.
  */
-
 struct pattern {
   struct patval *patn;  /* array of elements */
   int patlen;           /* number of elements */
@@ -220,13 +245,14 @@ struct pattern {
 #endif
 
   unsigned int class;   /* classification of pattern */
-  float value;          /* value for pattern, if matched */
-  float maxvalue;
-  float minterritory;
-  float maxterritory;
-  float shape;
-  float followup;
-  float reverse_followup;
+
+  /* Value (owl-style, used for pattern sorting) is not stored as an
+   * attribute, because it is very common.
+   */
+  float value;
+
+  /* Pattern attributes like shape, followup etc. */
+  struct pattern_attribute *attributes;
 
   int autohelper_flag;  /* whether autohelper has constraint and/or action */
   pattern_helper_fn_ptr helper;  /* helper function, or NULL */
@@ -275,13 +301,11 @@ struct fullboard_pattern {
 
 #define DECLARE(x) int x(struct pattern *pattern, int transformation, int move, int color)
 
-DECLARE(basic_cut_helper);
 DECLARE(jump_out_helper);
 DECLARE(jump_out_far_helper);
 DECLARE(high_handicap_helper);
 DECLARE(reinforce_helper);
 DECLARE(throw_in_atari_helper);
-DECLARE(ugly_cutstone_helper);
 DECLARE(cutstone2_helper);
 DECLARE(thrash_around_helper);
 
@@ -300,8 +324,10 @@ int connect_and_cut_helper(int Apos, int bpos, int cpos);
 int connect_and_cut_helper2(int Apos, int bpos, int cpos, int color);
 int edge_double_sente_helper(int move, int apos, int bpos, int cpos);
 void test_attack_either_move(int move, int color, int worma, int wormb);
+int adjacent_to_stone_in_atari(int str);
 int adjacent_to_defendable_stone_in_atari(int str);
 void backfill_replace(int move, int str);
+int break_mirror_helper(int str, int color);
 
 
 void init_tree_conn(void);
@@ -408,7 +434,8 @@ struct corner_pattern {
   unsigned int class;	/* Pattern class. */
   const char *name;	/* Pattern name (optional). */
 
-  float shape;		/* Pattern shape value. */
+  /* Pattern attributes like shape (the only one used currently). */
+  struct pattern_attribute *attributes;
 
   int autohelper_flag;	/* Whether autohelper has constraint and/or action. */
   autohelper_fn_ptr autohelper; /* Automatically generated helper (or NULL). */
@@ -429,7 +456,7 @@ struct corner_variation_b {
 };
 
 
-#endif /* __PATTERN_H__ */
+#endif /* _PATTERN_H_ */
 
 
 /*
