@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -706,11 +706,12 @@ SGFNode *
 sgfStartVariant(SGFNode *node)
 {
   assert(node);
+  assert(node->parent);
 
   while (node->next)
     node = node->next;
   node->next = sgfNewNode();
-  node->next->parent = node;
+  node->next->parent = node->parent;
 
   return node->next;
 }
@@ -726,12 +727,13 @@ sgfStartVariantFirst(SGFNode *node)
   SGFNode *old_first_child = node;
   SGFNode *new_first_child = sgfNewNode();
 
+  assert(node);
+  assert(node->parent);
+
   new_first_child->next = old_first_child;
-  if (old_first_child->parent)
-    new_first_child->parent = old_first_child->parent;
-  old_first_child->parent = new_first_child;
-  if (new_first_child->parent)
-    new_first_child->parent->child = new_first_child;
+  new_first_child->parent = old_first_child->parent;
+
+  new_first_child->parent->child = new_first_child;
 
   return new_first_child;
 }
@@ -807,12 +809,28 @@ sgfWriteResult(SGFNode *node, float score, int overwrite)
 }
 
 
+static void
+sgf_write_header_reduced(SGFNode *root, int overwrite)
+{
+  time_t curtime = time(NULL);
+  struct tm *loctime = localtime(&curtime);
+  char str[128];
+  int dummy;
+
+  gg_snprintf(str, 128, "%4.4i-%2.2i-%2.2i",
+	      loctime->tm_year+1900, loctime->tm_mon+1, loctime->tm_mday);
+  if (overwrite || !sgfGetIntProperty(root, "DT", &dummy))
+    sgfOverwriteProperty(root, "DT", str);
+  if (overwrite || !sgfGetIntProperty(root, "AP", &dummy))
+    sgfOverwriteProperty(root, "AP", "GNU Go:"VERSION);
+  sgfOverwriteProperty(root, "FF", "4");
+}
+
+
 void
 sgf_write_header(SGFNode *root, int overwrite, int seed, float komi,
 		 int level, int rules)
 {
-  time_t curtime = time(NULL);
-  struct tm *loctime = localtime(&curtime);
   char str[128];
   int dummy;
 
@@ -820,16 +838,11 @@ sgf_write_header(SGFNode *root, int overwrite, int seed, float komi,
 	      VERSION, seed, level);
   if (overwrite || !sgfGetIntProperty(root, "GN", &dummy))
     sgfOverwriteProperty(root, "GN", str);
-  gg_snprintf(str, 128, "%4.4i-%2.2i-%2.2i",
-	      loctime->tm_year+1900, loctime->tm_mon+1, loctime->tm_mday);
-  if (overwrite || !sgfGetIntProperty(root, "DT", &dummy))
-    sgfOverwriteProperty(root, "DT", str);
-  if (overwrite || !sgfGetIntProperty(root, "AP", &dummy))
-    sgfOverwriteProperty(root, "AP", "GNU Go:"VERSION);
   if (overwrite || !sgfGetIntProperty(root, "RU", &dummy))
     sgfOverwriteProperty(root, "RU", rules ? "Chinese" : "Japanese");
-  sgfOverwriteProperty(root, "FF", "4");
   sgfOverwritePropertyFloat(root, "KM", komi);
+
+  sgf_write_header_reduced(root, overwrite);
 }
 
 
@@ -1049,7 +1062,7 @@ gametree(SGFNode **p, SGFNode *parent, int mode)
     last = sequence(head);
     p = &last->child;
     while (lookahead == '(') {
-      gametree(p, last->parent, STRICT_SGF);
+      gametree(p, last, STRICT_SGF);
       p = &((*p)->next);
     }
     if (mode == STRICT_SGF)
@@ -1105,7 +1118,7 @@ gametreefuseki(SGFNode **p, SGFNode *parent, int mode,
 	break;
       }
       else {
-	gametreefuseki(p, last->parent, mode, moves_per_game, i);
+	gametreefuseki(p, last, mode, moves_per_game, i);
 	p = &((*p)->next);
       }
     }
@@ -1487,6 +1500,8 @@ writesgf(SGFNode *root, const char *filename)
     return 0;
   }
 
+  sgf_write_header_reduced(root, 0);
+
   sgf_column = 0;
   unparse_game(outfile, root, 1);
   fclose(outfile);
